@@ -10,21 +10,26 @@
 (********************************************************************)
 
 open Why3
-open Mlw_module
+open Pmodule
 open Py_ast
 open Ptree
-open Stdlib
+open Wstdlib
 
 let debug = Debug.register_flag "python"
   ~desc:"mini-python plugin debug flag"
+
+(* NO! this will be executed at plugin load, thus
+disabling the warning for ALL WHY3 USERS even if they don't
+use the python front-end
 let () = Debug.set_flag Dterm.debug_ignore_unused_var
+*)
 
 let mk_id ~loc name =
-  { id_str = name; id_lab = []; id_loc = loc }
+  { id_str = name; id_ats = []; id_loc = loc }
 
-let infix  ~loc s = Qident (mk_id ~loc ("infix "  ^ s))
-let prefix ~loc s = Qident (mk_id ~loc ("prefix " ^ s))
-let mixfix ~loc s = Qident (mk_id ~loc ("mixfix " ^ s))
+let infix  ~loc s = Qident (mk_id ~loc (Ident.infix s))
+let prefix ~loc s = Qident (mk_id ~loc (Ident.prefix s))
+let mixfix ~loc s = Qident (mk_id ~loc (Ident.mixfix s))
 
 let mk_expr ~loc d =
   { expr_desc = d; expr_loc = loc }
@@ -44,8 +49,10 @@ let deref_id ~loc id =
   mk_expr ~loc (Eidapp (prefix ~loc "!", [mk_expr ~loc (Eident (Qident id))]))
 let array_set ~loc a i v =
   mk_expr ~loc (Eidapp (mixfix ~loc "[]<-", [a; i; v]))
-let constant ~loc s =
-  mk_expr ~loc (Econst (Number.ConstInt (Number.int_const_dec s)))
+let constant ~loc i =
+  mk_expr ~loc (Econst (Number.const_of_int i))
+let constant_s ~loc s =
+  mk_expr ~loc (Econst (Number.(ConstInt { ic_negative = false ; ic_abs = int_literal_dec s})))
 let len ~loc =
   Qident (mk_id ~loc "len")
 let break ~loc =
@@ -67,6 +74,7 @@ let empty_spec = {
   sp_xpost   = [];
   sp_reads   = [];
   sp_writes  = [];
+  sp_alias   = [];
   sp_variant = [];
   sp_checkrw = false;
   sp_diverge = false;
@@ -98,13 +106,6 @@ let deref env t =
     mk_term ~loc:t.term_loc (Tlet (id, tid, t)) in
   Mstr.fold deref env.vars t
 
-let loop_annotation env a =
-  { loop_invariant = List.map (deref env) a.loop_invariant;
-    loop_variant   = List.map (fun (t, o) -> deref env t, o) a.loop_variant }
-
-let add_loop_invariant i a =
-  { a with loop_invariant = i :: a.loop_invariant }
-
 let stmt_of_decl = function Dstmt s -> s | _ -> invalid_arg "not a statement"
 
 let rec has_stmt p = function
@@ -133,7 +134,7 @@ let rec stmt_has_call id s = match s.stmt_desc with
   | Sset (e1, e2, e3) ->
     expr_has_call id e1 || expr_has_call id e2 || expr_has_call id e3
   | Sif (e, s1, s2) -> expr_has_call id e || block_has_call id s1 || block_has_call id s2
-  | Sfor (_, e, _, s) | Swhile (e, _, s) -> expr_has_call id e || block_has_call id s
+  | Sfor (_, e, _, s) | Swhile (e, _, _, s) -> expr_has_call id e || block_has_call id s
 and block_has_call id = has_stmtl (stmt_has_call id)
 
 let rec expr env {Py_ast.expr_loc = loc; Py_ast.expr_desc = d } = match d with
@@ -142,7 +143,7 @@ let rec expr env {Py_ast.expr_loc = loc; Py_ast.expr_desc = d } = match d with
   | Py_ast.Ebool b ->
     mk_expr ~loc (if b then Etrue else Efalse)
   | Py_ast.Eint s ->
-    constant ~loc s
+    constant_s ~loc s
   | Py_ast.Estring _s ->
     mk_unit ~loc (*FIXME*)
   | Py_ast.Eident id ->
@@ -153,13 +154,13 @@ let rec expr env {Py_ast.expr_loc = loc; Py_ast.expr_desc = d } = match d with
     let e1 = expr env e1 in
     let e2 = expr env e2 in
     mk_expr ~loc (match op with
-      | Py_ast.Band -> Elazy (e1, LazyAnd, e2)
-      | Py_ast.Bor  -> Elazy (e1, LazyOr,  e2)
+      | Py_ast.Band -> Eand (e1, e2)
+      | Py_ast.Bor  -> Eor  (e1, e2)
       | Py_ast.Badd -> Eidapp (infix ~loc "+", [e1; e2])
       | Py_ast.Bsub -> Eidapp (infix ~loc "-", [e1; e2])
       | Py_ast.Bmul -> Eidapp (infix ~loc "*", [e1; e2])
-      | Py_ast.Bdiv -> Eidapp (Qident (mk_id ~loc "div"), [e1; e2])
-      | Py_ast.Bmod -> Eidapp (Qident (mk_id ~loc "mod"), [e1; e2])
+      | Py_ast.Bdiv -> Eidapp (infix ~loc "//", [e1; e2])
+      | Py_ast.Bmod -> Eidapp (infix ~loc "%", [e1; e2])
       | Py_ast.Beq  -> Eidapp (infix ~loc "=", [e1; e2])
       | Py_ast.Bneq -> Eidapp (infix ~loc "<>", [e1; e2])
       | Py_ast.Blt  -> Eidapp (infix ~loc "<", [e1; e2])
@@ -173,22 +174,22 @@ let rec expr env {Py_ast.expr_loc = loc; Py_ast.expr_desc = d } = match d with
     mk_expr ~loc (Eidapp (Qident (mk_id ~loc "not"), [expr env e]))
   | Py_ast.Ecall ({id_str="print"}, el) ->
     let eval res e =
-      mk_expr ~loc (Elet (mk_id ~loc "_", Gnone, expr env e, res)) in
+      mk_expr ~loc (Elet (mk_id ~loc "_", false, Expr.RKnone, expr env e, res)) in
     List.fold_left eval (mk_unit ~loc) el
   | Py_ast.Ecall (id, el) ->
     mk_expr ~loc (Eidapp (Qident id, List.map (expr env) el))
   | Py_ast.Emake (e1, e2) -> (* [e1]*e2 *)
     array_make ~loc (expr env e2) (expr env e1)
   | Py_ast.Elist [] ->
-    array_make ~loc (constant ~loc "0") (constant ~loc "0")
+    array_make ~loc (constant ~loc 0) (constant ~loc 0)
   | Py_ast.Elist el ->
     let n = List.length el in
-    let n = constant ~loc (string_of_int n) in
+    let n = constant ~loc n in
     let id = mk_id ~loc "new array" in
-    mk_expr ~loc (Elet (id, Gnone, array_make ~loc n (constant ~loc "0"),
+    mk_expr ~loc (Elet (id, false, Expr.RKnone, array_make ~loc n (constant ~loc 0),
     let i = ref (-1) in
     let init seq e =
-      incr i; let i = constant ~loc (string_of_int !i) in
+      incr i; let i = constant ~loc !i in
       let assign = array_set ~loc (mk_var ~loc id) i (expr env e) in
       mk_expr ~loc (Esequence (assign, seq)) in
     List.fold_left init (mk_var ~loc id) el))
@@ -226,10 +227,12 @@ let rec stmt env ({Py_ast.stmt_loc = loc; Py_ast.stmt_desc = d } as s) =
     array_set ~loc (expr env e1) (expr env e2) (expr env e3)
   | Py_ast.Sassert (k, t) ->
     mk_expr ~loc (Eassert (k, deref env t))
-  | Py_ast.Swhile (e, ann, s) ->
+  | Py_ast.Swhile (e, inv, var, s) ->
+    let inv = List.map (deref env) inv in
+    let var = List.map (fun (t, o) -> deref env t, o) var in
     let loop = mk_expr ~loc
-      (Ewhile (expr env e, loop_annotation env ann, block env ~loc s)) in
-    if has_breakl s then mk_expr ~loc (Etry (loop, break_handler ~loc))
+      (Ewhile (expr env e, inv, var, block env ~loc s)) in
+    if has_breakl s then mk_expr ~loc (Ematch (loop, [], break_handler ~loc))
     else loop
   | Py_ast.Sbreak ->
     mk_expr ~loc (Eraise (break ~loc, None))
@@ -242,9 +245,9 @@ let rec stmt env ({Py_ast.stmt_loc = loc; Py_ast.stmt_desc = d } as s) =
                  inv, body) ->
     let inv = List.map (deref env) inv in
     let e_to = expr env e2 in
-    let ub = mk_expr ~loc (Eidapp (infix ~loc "-", [e_to;constant ~loc "1"])) in
-    mk_expr ~loc (Efor (id, expr env e1, To, ub, inv,
-    mk_expr ~loc (Elet (id, Gnone, mk_ref ~loc (mk_var ~loc id),
+    let ub = mk_expr ~loc (Eidapp (infix ~loc "-", [e_to; constant ~loc 1])) in
+    mk_expr ~loc (Efor (id, expr env e1, Expr.To, ub, inv,
+    mk_expr ~loc (Elet (id, false, Expr.RKnone, mk_ref ~loc (mk_var ~loc id),
     let env = add_var env id in
     block ~loc env body))))
   (* otherwise, translate
@@ -262,19 +265,19 @@ let rec stmt env ({Py_ast.stmt_loc = loc; Py_ast.stmt_desc = d } as s) =
   | Py_ast.Sfor (id, e, inv, body) ->
     let i, l, env = for_vars ~loc env in
     let e = expr env e in
-    mk_expr ~loc (Elet (l, Gnone, e, (* evaluate e only once *)
-    let lb = constant ~loc "0" in
+    mk_expr ~loc (Elet (l, false, Expr.RKnone, e, (* evaluate e only once *)
+    let lb = constant ~loc 0 in
     let lenl = mk_expr ~loc (Eidapp (len ~loc, [mk_var ~loc l])) in
-    let ub = mk_expr ~loc (Eidapp (infix ~loc "-", [lenl;constant ~loc "1"])) in
+    let ub = mk_expr ~loc (Eidapp (infix ~loc "-", [lenl; constant ~loc 1])) in
     let invariant inv =
       let loc = inv.term_loc in
       let li = mk_term ~loc
         (Tidapp (mixfix ~loc "[]", [mk_tvar ~loc l; mk_tvar ~loc i])) in
       mk_term ~loc (Tlet (id, li, deref env inv)) in
-    mk_expr ~loc (Efor (i, lb, To, ub, List.map invariant inv,
+    mk_expr ~loc (Efor (i, lb, Expr.To, ub, List.map invariant inv,
     let li = mk_expr ~loc
       (Eidapp (mixfix ~loc "[]", [mk_var ~loc l; mk_var ~loc i])) in
-    mk_expr ~loc (Elet (id, Gnone, mk_ref ~loc li,
+    mk_expr ~loc (Elet (id, false, Expr.RKnone, mk_ref ~loc li,
     let env = add_var env id in
     block ~loc env body
     ))))))
@@ -283,66 +286,66 @@ and block env ~loc = function
   | [] ->
     mk_unit ~loc
   | Dstmt { stmt_loc = loc; stmt_desc = Slabel id } :: sl ->
-    mk_expr ~loc (Emark (id, block env ~loc sl))
+    mk_expr ~loc (Elabel (id, block env ~loc sl))
   | Dstmt { Py_ast.stmt_loc = loc; stmt_desc = Py_ast.Sassign (id, e) } :: sl
     when not (Mstr.mem id.id_str env.vars) ->
     let e = expr env e in (* check e *before* adding id to environment *)
     let env = add_var env id in
-    mk_expr ~loc (Elet (id, Gnone, mk_ref ~loc e, block env ~loc sl))
+    mk_expr ~loc (Elet (id, false, Expr.RKnone, mk_ref ~loc e, block env ~loc sl))
   | Dstmt ({ Py_ast.stmt_loc = loc } as s) :: sl ->
     let s = stmt env s in
     if sl = [] then s else mk_expr ~loc (Esequence (s, block env ~loc sl))
   | Ddef (id, idl, sp, bl) :: sl ->
-    let lam = def (id, idl, sp, bl) in
+    (* f(x1,...,xn): body ==>
+      let f x1 ... xn =
+        let x1 = ref x1 in ... let xn = ref xn in
+        try body with Return x -> x *)
+    let env' = List.fold_left add_var empty_env idl in
+    let body = block env' ~loc:id.id_loc bl in
+    let body = if not (has_returnl bl) then body else
+      let loc = id.id_loc in
+      mk_expr ~loc (Ematch (body, [], return_handler ~loc)) in
+    let local bl id =
+      let loc = id.id_loc in
+      let ref = mk_ref ~loc (mk_var ~loc id) in
+      mk_expr ~loc (Elet (id, false, Expr.RKnone, ref, bl)) in
+    let body = List.fold_left local body idl in
+    let param id = id.id_loc, Some id, false, None in
+    let params = if idl = [] then no_params ~loc else List.map param idl in
     let s = block env ~loc sl in
-    if block_has_call id bl then mk_expr ~loc (Erec ([id, Gnone, lam], s))
-    else mk_expr ~loc (Efun (id, Gnone, lam, s))
+    let e = if block_has_call id bl then
+      Erec ([id, false, Expr.RKnone, params, None, Ity.MaskVisible, sp, body], s)
+    else
+      let e = Efun (params, None, Ity.MaskVisible, sp, body) in
+      Elet (id, false, Expr.RKnone, mk_expr ~loc e, s) in
+    mk_expr ~loc e
   | (Dimport _ | Py_ast.Dlogic _) :: sl ->
     block env ~loc sl
 
-(* f(x1,...,xn): body
-
-   let f x1 ... xn =
-     let x1 = ref x1 in ... let xn = ref xn in
-     try body with Return x -> x *)
-and def (id, idl, sp, bl) =
-  let loc = id.id_loc in
-  let env = empty_env in
-  let env = List.fold_left add_var env idl in
-  let body = block env ~loc bl in
-  let body = if has_returnl bl then
-      mk_expr ~loc (Etry (body, return_handler ~loc)) else body in
-  let local bl id =
-    let loc = id.id_loc in
-    mk_expr ~loc (Elet (id, Gnone, mk_ref ~loc (mk_var ~loc id), bl)) in
-  let body = List.fold_left local body idl in
-  let param id = id.id_loc, Some id, false, None in
-  let params = if idl = [] then no_params ~loc else List.map param idl in
-  params, None, body, sp
-
 let fresh_type_var =
   let r = ref 0 in
   fun loc -> incr r;
-    PTtyvar ({ id_str = "a" ^ string_of_int !r; id_loc = loc; id_lab = [] }, false)
+    PTtyvar { id_str = "a" ^ string_of_int !r; id_loc = loc; id_ats = [] }
 
 let logic_param id =
   id.id_loc, Some id, false, fresh_type_var id.id_loc
 
-let logic inc = function
+let logic = function
   | Py_ast.Dlogic (func, id, idl) ->
     let d = { ld_loc = id.id_loc;
               ld_ident = id;
               ld_params = List.map logic_param idl;
               ld_type = if func then Some (fresh_type_var id.id_loc) else None;
               ld_def = None } in
-    inc.new_decl id.id_loc (Dlogic [d])
+    Typing.add_decl id.id_loc (Dlogic [d])
   | _ -> ()
 
-let translate ~loc inc dl =
-  List.iter (logic inc) dl;
-  let fd = (no_params ~loc, None, block empty_env ~loc dl, empty_spec) in
-  let main = Dfun (mk_id ~loc "main", Gnone, fd) in
-  inc.new_pdecl loc main
+let translate ~loc dl =
+  List.iter logic dl;
+  let bl = block empty_env ~loc dl in
+  let fd = Efun (no_params ~loc, None, Ity.MaskVisible, empty_spec, bl) in
+  let main = Dlet (mk_id ~loc "main", false, Expr.RKnone, mk_expr ~loc fd) in
+  Typing.add_decl loc main
 
 let read_channel env path file c =
   let f = Py_lexer.parse file c in
@@ -351,27 +354,25 @@ let read_channel env path file c =
   let file = Filename.chop_extension file in
   let name = Strings.capitalize file in
   Debug.dprintf debug "building module %s.@." name;
-  let inc = Mlw_typing.open_file env path in
-  let loc = Why3.Loc.user_position file 0 0 0 in
-  inc.open_module (mk_id ~loc name);
+  Typing.open_file env path;
+  let loc = Loc.user_position file 0 0 0 in
+  Typing.open_module (mk_id ~loc name);
   let use_import (f, m) =
-    let q = Qdot (Qident (mk_id ~loc f), mk_id ~loc m) in
-    let use = {use_theory = q; use_import = Some (true, m) }, None in
-    inc.use_clone loc use in
+    let m = mk_id ~loc m in
+    Typing.open_scope loc m;
+    Typing.add_decl loc (Ptree.Duse (Qdot (Qident (mk_id ~loc f), m)));
+    Typing.close_scope loc ~import:true in
   List.iter use_import
     ["int", "Int"; "ref", "Refint"; "python", "Python"];
-  translate ~loc inc f;
-  inc.close_module ();
-  let mm, _ as res = Mlw_typing.close_file () in
+  translate ~loc f;
+  Typing.close_module loc;
+  let mm = Typing.close_file () in
   if path = [] && Debug.test_flag debug then begin
     let add_m _ m modm = Ident.Mid.add m.mod_theory.Theory.th_name m modm in
-    let modm = Mstr.fold add_m mm Ident.Mid.empty in
-    let print_m _ m = Format.eprintf
-      "@[<hov 2>module %a@\n%a@]@\nend@\n@." Pretty.print_th m.mod_theory
-      (Pp.print_list Pp.newline2 Mlw_pretty.print_pdecl) m.mod_decls in
-    Ident.Mid.iter print_m modm
+    let print_m _ m = Pmodule.print_module Format.err_formatter m in
+    Ident.Mid.iter print_m (Mstr.fold add_m mm Ident.Mid.empty)
   end;
-  res
+  mm
 
 let () =
   Env.register_format mlw_language "python" ["py"] read_channel
