@@ -94,11 +94,13 @@ let print_type info fmt ty = try print_type info fmt ty
 let number_format = {
   Number.long_int_support = true;
   Number.extra_leading_zeros_support = false;
+  Number.negative_int_support = Number.Number_default;
   Number.dec_int_support = Number.Number_default;
   Number.hex_int_support = Number.Number_unsupported;
   Number.oct_int_support = Number.Number_unsupported;
   Number.bin_int_support = Number.Number_unsupported;
   Number.def_int_support = Number.Number_unsupported;
+  Number.negative_real_support = Number.Number_default;
   Number.dec_real_support = Number.Number_default;
   Number.hex_real_support = Number.Number_unsupported;
   Number.frac_real_support = Number.Number_custom
@@ -261,7 +263,7 @@ let print_decl info fmt d = match d.d_node with
       let head = if info.info_fmt = FOF then "fof" else "tff" in
       fprintf fmt "@[<hov 2>%s(%a, conjecture,@ %a).@]@\n"
         head print_pr pr (print_fmla info) f
-  | Dprop ((Plemma|Pskip), _, _) -> assert false
+  | Dprop (Plemma, _, _) -> assert false
 
 let print_decls fm nm =
 (*
