@@ -13,7 +13,7 @@ open Format
 open Tptp_ast
 
 open Why3
-open Stdlib
+open Wstdlib
 open Ident
 open Ty
 open Term
@@ -285,7 +285,7 @@ let rec ty denv env impl { e_loc = loc; e_node = n } = match n with
   | Enot _ | Eequ _ | Edob _ | Enum _ -> error ~loc TypeExpected
 
 let t_int_const s =
-  t_const (Number.ConstInt (Number.int_const_dec s)) ty_int
+  t_const (Number.(ConstInt { ic_negative = false; ic_abs = int_literal_dec s})) ty_int
 
 (* unused
 let t_real_const r = t_const (Number.ConstReal r)
@@ -308,8 +308,8 @@ let rec term denv env impl { e_loc = loc; e_node = n } = match n with
       find_dobj ~loc denv env impl s
   | Enum (Nint s) -> t_int_const s
   | Enum (Nreal (i,f,e)) ->
-      t_const (Number.ConstReal
-        (Number.real_const_dec i (Opt.get_def "0" f) e)) ty_real
+      t_const (Number.(ConstReal { rc_negative = false ;
+                                   rc_abs = real_const_dec i (Opt.get_def "0" f) e})) ty_real
   | Enum (Nrat (n,d)) ->
       let n = t_int_const n and d = t_int_const d in
       let frac = ns_find_ls denv.th_rat.th_export ["frac"] in
@@ -576,9 +576,9 @@ let typedecl denv env impl loc s (tvl,(el,e)) =
 let flush_impl ~strict env uc impl =
   let update_th _ e uc = match e with
     | Suse th ->
-        let uc = open_namespace uc th.th_name.id_string in
+        let uc = open_scope uc th.th_name.id_string in
         let uc = use_export uc th in
-        close_namespace uc false
+        close_scope uc ~import:false
     | _ -> uc
   in
   let update s e (env,uc) = match e with
