@@ -12,7 +12,7 @@
 open Format
 open Pp
 
-open Stdlib
+open Wstdlib
 open Ident
 open Ty
 open Term
@@ -24,6 +24,8 @@ open Task
 
 type prelude = string list
 type prelude_map = prelude Mid.t
+type interface = string list
+type interface_map = interface Mid.t
 type blacklist = string list
 
 type 'a pp = Pp.formatter -> 'a -> unit
@@ -31,7 +33,9 @@ type 'a pp = Pp.formatter -> 'a -> unit
 type printer_mapping = {
   lsymbol_m     : string -> Term.lsymbol;
   vc_term_loc   : Loc.position option;
-  queried_terms : Term.term Stdlib.Mstr.t;
+  queried_terms : Term.term Mstr.t;
+  list_projections: Sstr.t;
+  list_records: ((string * string) list) Mstr.t;
 }
 
 type printer_args = {
@@ -54,7 +58,9 @@ exception UnknownPrinter of string
 let get_default_printer_mapping = {
   lsymbol_m = (function _ -> raise Not_found);
   vc_term_loc = None;
-  queried_terms = Stdlib.Mstr.empty;
+  queried_terms = Mstr.empty;
+  list_projections = Sstr.empty;
+  list_records = Mstr.empty;
 }
 
 let register_printer ~desc s p =
@@ -165,7 +171,7 @@ exception BadSyntaxArity of int * int
 let int_of_string s =
   try int_of_string s
   with _ ->
-    Format.eprintf "bad argument for int_of_string : %s@." s;
+    Format.eprintf "bad argument for int_of_string: %s@." s;
     assert false
 
 let check_syntax s len =
@@ -224,9 +230,9 @@ let get_type_arguments t = match t.t_node with
       let m = oty_match Mtv.empty ls.ls_value t.t_ty in
       let m = List.fold_left2
         (fun m ty t -> oty_match m (Some ty) t.t_ty) m ls.ls_args tl in
-      let name tv = Stdlib.Mstr.add tv.tv_name.id_string in
-      let m = Mtv.fold name m Stdlib.Mstr.empty in
-      Array.of_list (Stdlib.Mstr.values m)
+      let name tv = Mstr.add tv.tv_name.id_string in
+      let m = Mtv.fold name m Mstr.empty in
+      Array.of_list (Mstr.values m)
   | _ ->
       [||]
 
@@ -254,6 +260,7 @@ let syntax_arguments_typed =
 
 let syntax_range_literal s fmt c =
   let f s b e fmt =
+    let v = Number.compute_int_literal c.Number.ic_abs in
     let base = match s.[e-1] with
       | 'x' -> 16
       | 'd' -> 10
@@ -267,8 +274,20 @@ let syntax_range_literal s fmt c =
       else
         None
     in
-    let v = Number.compute_int c in
-    Number.print_in_base base digits fmt v
+    if base = 10 then begin
+        if c.Number.ic_negative then fprintf fmt "-";
+        Number.print_in_base base digits fmt v
+      end
+    else
+      let v =
+        if c.Number.ic_negative then
+          match digits with
+            | Some d ->
+               BigInt.sub (BigInt.pow_int_pos base d) v
+            | None -> failwith ("number of digits must be given for printing negative literals in base " ^ string_of_int base)
+        else v
+      in
+      Number.print_in_base base digits fmt v
   in
   global_substitute_fmt opt_search_forward_literal_format f s fmt
 
@@ -287,9 +306,10 @@ let syntax_float_literal s fp fmt c =
       else
         None
     in
-    let e,m = Number.compute_float c fp in
+    let e,m = Number.compute_float c.Number.rc_abs fp in
+    let sg = if c.Number.rc_negative then BigInt.one else BigInt.zero in
     match s.[b] with
-    | 's' -> Number.print_in_base base digits fmt BigInt.zero
+    | 's' -> Number.print_in_base base digits fmt sg
     | 'e' -> Number.print_in_base base digits fmt e
     | 'm' -> Number.print_in_base base digits fmt m
     | _ -> assert false
@@ -302,6 +322,8 @@ let print_prelude fmt pl =
   let println fmt s = fprintf fmt "%s@\n" s in
   print_list nothing println fmt pl
 
+let print_interface = print_prelude
+
 let print_prelude_of_theories th_used fmt pm =
   let ht = Hid.create 5 in
   List.iter (fun { th_name = id } ->
@@ -311,7 +333,7 @@ let print_prelude_of_theories th_used fmt pm =
 
 let print_th_prelude task fmt pm =
   let th_used = task_fold (fun acc -> function
-    | { td_node = Clone (th,_) } -> th::acc
+    | { td_node = Use th | Clone (th,_) } -> th::acc
     | _ -> acc) [] task
   in
   print_prelude_of_theories th_used fmt pm
@@ -563,7 +585,7 @@ let () = Exn_printer.register (fun fmt exn -> match exn with
   | BadSyntaxArity (i1,i2) ->
       fprintf fmt "Bad argument index %d, must end with %d" i2 i1
   | Unsupported s ->
-      fprintf fmt "@[<hov 3> Uncatched exception 'Unsupported %s'@]" s
+      fprintf fmt "@[<hov 3> Uncaught exception 'Unsupported %s'@]" s
   | UnsupportedType (e,s) ->
       fprintf fmt "@[@[<hov 3> This type isn't supported:@\n%a@]@\n %s@]"
         Pretty.print_ty e s
