@@ -14,6 +14,8 @@
 open Ident
 open Ty
 
+(** {1 Terms and Formulas} *)
+
 (** {2 Variable symbols} *)
 
 type vsymbol = private {
@@ -38,7 +40,6 @@ type lsymbol = private {
   ls_name   : ident;
   ls_args   : ty list;
   ls_value  : ty option;
-  ls_opaque : Stv.t;
   ls_constr : int;
 }
 
@@ -51,14 +52,11 @@ val ls_compare : lsymbol -> lsymbol -> int
 val ls_equal : lsymbol -> lsymbol -> bool
 val ls_hash : lsymbol -> int
 
-val create_lsymbol :
-  ?opaque:Stv.t -> ?constr:int -> preid -> ty list -> ty option -> lsymbol
+val create_lsymbol : ?constr:int -> preid -> ty list -> ty option -> lsymbol
 
-val create_fsymbol :
-  ?opaque:Stv.t -> ?constr:int -> preid -> ty list -> ty -> lsymbol
+val create_fsymbol : ?constr:int -> preid -> ty list -> ty -> lsymbol
 
-val create_psymbol :
-  ?opaque:Stv.t -> preid -> ty list -> lsymbol
+val create_psymbol : preid -> ty list -> lsymbol
 
 val ls_ty_freevars : lsymbol -> Stv.t
 
@@ -82,7 +80,7 @@ type pattern = private {
   pat_ty   : ty;
 }
 
-and pattern_node = private
+and pattern_node =
   | Pwild
   | Pvar of vsymbol
   | Papp of lsymbol * pattern list
@@ -119,11 +117,11 @@ type binop =
 type term = private {
   t_node  : term_node;
   t_ty    : ty option;
-  t_label : Slab.t;
+  t_attrs : Sattr.t;
   t_loc   : Loc.position option;
 }
 
-and term_node = private
+and term_node =
   | Tvar of vsymbol
   | Tconst of Number.constant
   | Tapp of lsymbol * term list
@@ -152,6 +150,9 @@ module Hterm : Exthtbl.S with type key = term
 val t_compare : term -> term -> int
 val t_equal : term -> term -> bool
 val t_hash : term -> int
+(* Equality modulo attributes and triggers *)
+val t_equal_nt_na : term -> term -> bool
+module Hterm_nt_na : Exthtbl.S with type key = term
 
 (** {2 Bindings} *)
 
@@ -167,6 +168,9 @@ val t_open_bound : term_bound -> vsymbol * term
 val t_open_branch : term_branch -> pattern * term
 val t_open_quant : term_quant -> vsymbol list * trigger * term
 
+val t_open_bound_with : term -> term_bound -> term
+val t_clone_bound_id : term_bound -> preid
+
 (** open bindings with optimized closing callbacks *)
 
 val t_open_bound_cb :
@@ -203,6 +207,8 @@ val t_app_infer : lsymbol -> term list -> term
 val ls_arg_inst : lsymbol -> term list -> ty Mtv.t
 val ls_app_inst : lsymbol -> term list -> ty option -> ty Mtv.t
 
+val check_literal : Number.constant -> ty -> unit
+
 val t_var : vsymbol -> term
 val t_const : Number.constant -> ty -> term
 val t_if : term -> term -> term -> term
@@ -224,11 +230,20 @@ val t_false : term
 val t_nat_const : int -> term
 (** [t_nat_const n] builds the constant integer term [n],
     n must be non-negative *)
+val t_bigint_const : BigInt.t -> term
+
+val stop_split : attribute
+val asym_split : attribute
+
+val t_and_l : term list -> term
+val t_or_l : term list -> term
 
-val asym_label : label
 val t_and_asym : term -> term -> term
 val t_or_asym : term -> term -> term
 
+val t_and_asym_l : term list -> term
+val t_or_asym_l : term list -> term
+
 val t_let_close : vsymbol -> term -> term -> term
 val t_eps_close : vsymbol -> term -> term
 val t_case_close : term -> (pattern * term) list -> term
@@ -236,15 +251,13 @@ val t_quant_close : quant -> vsymbol list -> trigger -> term -> term
 val t_forall_close : vsymbol list -> trigger -> term -> term
 val t_exists_close : vsymbol list -> trigger -> term -> term
 
-val t_label : ?loc:Loc.position -> Slab.t -> term -> term
-val t_label_add : label -> term -> term
-val t_label_remove : label -> term -> term
-val t_label_copy : term -> term -> term
+val t_attr_set : ?loc:Loc.position -> Sattr.t -> term -> term
+val t_attr_add : attribute -> term -> term
+val t_attr_remove : attribute -> term -> term
+val t_attr_copy : term -> term -> term
 
 (** Constructors with propositional simplification *)
 
-val keep_on_simp_label : label
-
 val t_if_simp : term -> term -> term -> term
 val t_let_simp : term -> term_bound -> term
 val t_case_simp : term -> term_branch list -> term
@@ -369,6 +382,7 @@ val t_pred_app_beta_l : term -> term list -> term
 
 val t_map : (term -> term) -> term -> term
 val t_fold : ('a -> term -> 'a) -> 'a -> term -> 'a
+val t_iter : (term -> unit) -> term -> unit
 val t_map_fold : ('a -> term -> 'a * term) -> 'a -> term -> 'a * term
 
 val t_all : (term -> bool) -> term -> bool
@@ -456,9 +470,12 @@ val t_ty_subst : ty Mtv.t -> term Mvs.t -> term -> term
     mapping [mt] and term variables by mapping [mv] in term [t] *)
 
 val t_subst_types : ty Mtv.t -> term Mvs.t -> term -> term Mvs.t * term
-(** [t_subst_types mt mv t] substitutes type variables by
-    mapping [mt] simultaneously in substitution [mv] and in term [t].
-    This operation may rename the variables in [t]. *)
+(** [t_subst_types mt mv t] substitutes type variables of term [t] by
+    mapping [mt]. This operation may rename the variables in [t], and
+    the same renaming is simultaneously applied to the variables of
+    the substitution [mv] (both domain and codomain).
+    Example: [t_subst_types {'a -> int} {x:'a -> t:'a} (f x)]
+       returns ({z:int -> t:int},(f z))   *)
 
 (** {2 Find free variables and type variables} *)
 
