@@ -10,7 +10,7 @@
 (********************************************************************)
 
 open Format
-open Stdlib
+open Wstdlib
 open Ident
 open Ty
 open Term
@@ -39,29 +39,30 @@ let ns_replace eq chk x vo vn =
   if eq vo vn then vo else
   raise (ClashSymbol x)
 
-let rec merge_ns chk ns1 ns2 =
-  if ns1 == ns2 then ns1 else
-  let join eq x n o = Some (ns_replace eq chk x o n) in
-  let ns_union eq m1 m2 =
-    if m1 == m2 then m1 else Mstr.union (join eq) m1 m2 in
-  let fusion _ ns1 ns2 = Some (merge_ns chk ns1 ns2) in
-  { ns_ts = ns_union ts_equal ns1.ns_ts ns2.ns_ts;
-    ns_ls = ns_union ls_equal ns1.ns_ls ns2.ns_ls;
-    ns_pr = ns_union pr_equal ns1.ns_pr ns2.ns_pr;
-    ns_ns = Mstr.union fusion ns1.ns_ns ns2.ns_ns; }
+let merge_ts = ns_replace ts_equal
+let merge_ls = ns_replace ls_equal
+let merge_pr = ns_replace pr_equal
 
-let add_ns chk x ns m = Mstr.change (function
-  | Some os -> Some (merge_ns chk ns os)
-  | None    -> Some ns) x m
+let rec merge_ns chk _ no nn =
+  if no == nn then no else
+  let union merge o n =
+    let merge x vo vn = Some (merge chk x vo vn) in
+    if o == n then o else Mstr.union merge o n in
+  { ns_ts = union merge_ts no.ns_ts nn.ns_ts;
+    ns_ls = union merge_ls no.ns_ls nn.ns_ls;
+    ns_pr = union merge_pr no.ns_pr nn.ns_pr;
+    ns_ns = union merge_ns no.ns_ns nn.ns_ns }
 
-let ns_add eq chk x vn m = Mstr.change (function
-  | Some vo -> Some (ns_replace eq chk x vo vn)
+let ns_add merge chk x vn m = Mstr.change (function
+  | Some vo -> Some (merge chk x vo vn)
   | None    -> Some vn) x m
 
-let add_ts chk x ts ns = { ns with ns_ts = ns_add ts_equal chk x ts ns.ns_ts }
-let add_ls chk x ls ns = { ns with ns_ls = ns_add ls_equal chk x ls ns.ns_ls }
-let add_pr chk x pf ns = { ns with ns_pr = ns_add pr_equal chk x pf ns.ns_pr }
-let add_ns chk x nn ns = { ns with ns_ns = add_ns          chk x nn ns.ns_ns }
+let add_ts chk x ts ns = { ns with ns_ts = ns_add merge_ts chk x ts ns.ns_ts }
+let add_ls chk x ps ns = { ns with ns_ls = ns_add merge_ls chk x ps ns.ns_ls }
+let add_pr chk x xs ns = { ns with ns_pr = ns_add merge_pr chk x xs ns.ns_pr }
+let add_ns chk x nn ns = { ns with ns_ns = ns_add merge_ns chk x nn ns.ns_ns }
+
+let merge_ns chk nn no = merge_ns chk "" no nn (* swap arguments *)
 
 let rec ns_find get_map ns = function
   | []   -> assert false
@@ -153,16 +154,22 @@ let meta_range = register_meta "range_type" [MTtysymbol; MTlsymbol]
 let meta_float = register_meta "float_type" [MTtysymbol; MTlsymbol; MTlsymbol]
     ~desc:"Projection@ and@ finiteness@ of@ a@ floating-point@ type."
 
+let meta_projection = register_meta "model_projection" [MTlsymbol]
+  ~desc:"Declares@ the@ projection."
+
 (** Theory *)
 
 type theory = {
-  th_name   : ident;      (* theory name *)
-  th_path   : string list;(* environment qualifiers *)
-  th_decls  : tdecl list; (* theory declarations *)
-  th_export : namespace;  (* exported namespace *)
-  th_known  : known_map;  (* known identifiers *)
-  th_local  : Sid.t;      (* locally declared idents *)
-  th_used   : Sid.t;      (* used theories *)
+  th_name   : ident;          (* theory name *)
+  th_path   : string list;    (* environment qualifiers *)
+  th_decls  : tdecl list;     (* theory declarations *)
+  th_ranges : tdecl Mts.t;    (* range type projections *)
+  th_floats : tdecl Mts.t;    (* float type projections *)
+  th_crcmap : Coercion.t;     (* implicit coercions *)
+  th_export : namespace;      (* exported namespace *)
+  th_known  : known_map;      (* known identifiers *)
+  th_local  : Sid.t;          (* locally declared idents *)
+  th_used   : Sid.t;          (* used theories *)
 }
 
 and tdecl = {
@@ -177,6 +184,7 @@ and tdecl_node =
   | Meta  of meta * meta_arg list
 
 and symbol_map = {
+  sm_ty : ty Mts.t;
   sm_ts : tysymbol Mts.t;
   sm_ls : lsymbol Mls.t;
   sm_pr : prsymbol Mpr.t;
@@ -198,6 +206,7 @@ module Hstdecl = Hashcons.Make (struct
     | _,_ -> false
 
   let eq_smap sm1 sm2 =
+    Mts.equal ty_equal sm1.sm_ty sm2.sm_ty &&
     Mts.equal ts_equal sm1.sm_ts sm2.sm_ts &&
     Mls.equal ls_equal sm1.sm_ls sm2.sm_ls &&
     Mpr.equal pr_equal sm1.sm_pr sm2.sm_pr
@@ -211,6 +220,7 @@ module Hstdecl = Hashcons.Make (struct
         t1 = t2 && Lists.equal eq_marg al1 al2
     | _,_ -> false
 
+  let hs_cl_ty _ ty acc = Hashcons.combine acc (ty_hash ty)
   let hs_cl_ts _ ts acc = Hashcons.combine acc (ts_hash ts)
   let hs_cl_ls _ ls acc = Hashcons.combine acc (ls_hash ls)
   let hs_cl_pr _ pr acc = Hashcons.combine acc (pr_hash pr)
@@ -224,9 +234,10 @@ module Hstdecl = Hashcons.Make (struct
     | MAint i -> Hashtbl.hash i
 
   let hs_smap sm h =
-    Mts.fold hs_cl_ts sm.sm_ts
-      (Mls.fold hs_cl_ls sm.sm_ls
-        (Mpr.fold hs_cl_pr sm.sm_pr h))
+    Mts.fold hs_cl_ty sm.sm_ty
+      (Mts.fold hs_cl_ts sm.sm_ts
+        (Mls.fold hs_cl_ls sm.sm_ls
+          (Mpr.fold hs_cl_pr sm.sm_pr h)))
 
   let hash td = match td.td_node with
     | Decl d -> d_hash d
@@ -258,6 +269,9 @@ type theory_uc = {
   uc_name   : ident;
   uc_path   : string list;
   uc_decls  : tdecl list;
+  uc_ranges : tdecl Mts.t;
+  uc_floats : tdecl Mts.t;
+  uc_crcmap : Coercion.t;
   uc_prefix : string list;
   uc_import : namespace list;
   uc_export : namespace list;
@@ -273,6 +287,9 @@ let empty_theory n p = {
   uc_name   = id_register n;
   uc_path   = p;
   uc_decls  = [];
+  uc_ranges = Mts.empty;
+  uc_floats = Mts.empty;
+  uc_crcmap = Coercion.empty;
   uc_prefix = [];
   uc_import = [empty_ns];
   uc_export = [empty_ns];
@@ -282,10 +299,13 @@ let empty_theory n p = {
 }
 
 let close_theory uc = match uc.uc_export with
-  | [e] -> {
-      th_name   = uc.uc_name;
+  | [e] ->
+    { th_name   = uc.uc_name;
       th_path   = uc.uc_path;
       th_decls  = List.rev uc.uc_decls;
+      th_ranges = uc.uc_ranges;
+      th_floats = uc.uc_floats;
+      th_crcmap = uc.uc_crcmap;
       th_export = e;
       th_known  = uc.uc_known;
       th_local  = uc.uc_local;
@@ -293,41 +313,48 @@ let close_theory uc = match uc.uc_export with
   | _ -> raise CloseTheory
 
 let get_namespace uc = List.hd uc.uc_import
-let get_known uc = uc.uc_known
-let get_rev_decls uc = uc.uc_decls
 
-let open_namespace uc s = match uc.uc_import with
+let open_scope uc s = match uc.uc_import with
   | ns :: _ -> { uc with
       uc_prefix =        s :: uc.uc_prefix;
       uc_import =       ns :: uc.uc_import;
       uc_export = empty_ns :: uc.uc_export; }
   | [] -> assert false
 
-let close_namespace uc import =
+let close_scope uc ~import =
   match uc.uc_prefix, uc.uc_import, uc.uc_export with
   | s :: prf, _ :: i1 :: sti, e0 :: e1 :: ste ->
       let i1 = if import then merge_ns false e0 i1 else i1 in
-      let _  = if import then merge_ns true  e0 e1 else e1 in
       let i1 = add_ns false s e0 i1 in
       let e1 = add_ns true  s e0 e1 in
       { uc with uc_prefix = prf; uc_import = i1::sti; uc_export = e1::ste; }
   | [], [_], [_] -> raise NoOpenedNamespace
   | _ -> assert false
 
+let import_namespace ns ql =
+  let e0 = ns_find_ns ns ql in merge_ns false e0 ns
+
+let import_scope uc ql =
+  match uc.uc_import with
+  | i1 :: sti ->
+     let i1 = import_namespace i1 ql in
+     { uc with uc_import = i1::sti }
+  | _ -> assert false
+
 (* Base constructors *)
 
-let known_ts kn ts = match ts.ts_def with
-  | Alias ty -> ty_s_fold (fun () ts -> known_id kn ts.ts_name) () ty
-  | NoDef | Range _ | Float _ -> known_id kn ts.ts_name
+let known_ty kn ty =
+  ty_s_fold (fun () ts -> known_id kn ts.ts_name) () ty
 
 let known_clone kn sm =
-  Mts.iter (fun _ ts -> known_ts kn ts) sm.sm_ts;
+  Mts.iter (fun _ ty -> known_ty kn ty) sm.sm_ty;
+  Mts.iter (fun _ ts -> known_id kn ts.ts_name) sm.sm_ts;
   Mls.iter (fun _ ls -> known_id kn ls.ls_name) sm.sm_ls;
   Mpr.iter (fun _ pr -> known_id kn pr.pr_name) sm.sm_pr
 
 let known_meta kn al =
   let check = function
-    | MAty ty -> ty_s_fold (fun () ts -> known_id kn ts.ts_name) () ty
+    | MAty ty -> known_ty kn ty
     | MAts ts -> known_id kn ts.ts_name
     | MAls ls -> known_id kn ls.ls_name
     | MApr pr -> known_id kn pr.pr_name
@@ -335,6 +362,12 @@ let known_meta kn al =
   in
   List.iter check al
 
+(* FIXME: proper description *)
+let meta_coercion = register_meta ~desc:"coercion" "coercion" [MTlsymbol]
+
+exception RangeConflict of tysymbol
+exception FloatConflict of tysymbol
+
 let add_tdecl uc td = match td.td_node with
   | Decl d -> { uc with
       uc_decls = td :: uc.uc_decls;
@@ -347,7 +380,28 @@ let add_tdecl uc td = match td.td_node with
       uc_used  = Sid.union uc.uc_used (Sid.add th.th_name th.th_used) }
   | Clone (_,sm) -> known_clone uc.uc_known sm;
       { uc with uc_decls = td :: uc.uc_decls }
-  | Meta (_,al) -> known_meta uc.uc_known al;
+  | Meta (m,((MAts ts :: _) as al)) when meta_equal m meta_range ->
+      known_meta uc.uc_known al;
+      let add b = match b with
+        | None -> Some td
+        | Some d when td_equal d td -> b
+        | _ -> raise (RangeConflict ts) in
+      { uc with uc_ranges = Mts.change add ts uc.uc_ranges;
+                uc_decls = td :: uc.uc_decls }
+  | Meta (m,((MAts ts :: _) as al)) when meta_equal m meta_float ->
+      known_meta uc.uc_known al;
+      let add b = match b with
+        | None -> Some td
+        | Some d when td_equal d td -> b
+        | _ -> raise (FloatConflict ts) in
+      { uc with uc_floats = Mts.change add ts uc.uc_floats;
+                uc_decls = td :: uc.uc_decls }
+  | Meta (m,([MAls ls] as al)) when meta_equal m meta_coercion ->
+      known_meta uc.uc_known al;
+      { uc with uc_crcmap = Coercion.add uc.uc_crcmap ls;
+                uc_decls = td :: uc.uc_decls }
+  | Meta (_,al) ->
+      known_meta uc.uc_known al;
       { uc with uc_decls = td :: uc.uc_decls }
 
 (** Declarations *)
@@ -382,44 +436,23 @@ let add_symbol add id v uc =
   | _ -> assert false
 
 let add_symbol_ts uc ts = add_symbol add_ts ts.ts_name ts uc
-let add_symbol_ls uc ls = add_symbol add_ls ls.ls_name ls uc
+
+let add_symbol_ls uc ({ls_name = id} as ls) =
+  let {id_string = nm; id_loc = loc} = id in
+  if (nm = "infix =" || nm = "infix <>") &&
+      uc.uc_path <> ["why3";"BuiltIn"] then
+    Loc.errorm ?loc "Logical equality cannot be redefined";
+  add_symbol add_ls id ls uc
+
 let add_symbol_pr uc pr = add_symbol add_pr pr.pr_name pr uc
 
 let create_decl d = mk_tdecl (Decl d)
 
-let check_subst_opacity ls ls' sbs =
-  (* the definition of ls contains ls' instantiated with sbs *)
-  let sbs = Mtv.set_diff sbs ls'.ls_opaque in
-  let check () tv = if Stv.mem tv ls.ls_opaque then
-    Loc.errorm "type parameter '%s is not opaque in symbol `%s'"
-      tv.tv_name.id_string ls.ls_name.id_string in
-  Mtv.iter (fun _ ty -> ty_v_fold check () ty) sbs
-
-let check_decl_opacity d = match d.d_node with
-  (* All lsymbols declared in Ddata are safe, nothing to check.
-     We allow arbitrary ls_opaque in Dparam, but we check that
-     cloning in theories preserves opacity, see cl_init below. *)
-  | Dtype _ | Ddata _ | Dparam _ | Dprop _ -> ()
-  | Dlogic dl ->
-      let check (ols,ld) =
-        let check () ls args value =
-          let sbs = oty_match Mtv.empty ls.ls_value value in
-          let sbs = List.fold_left2 ty_match sbs ls.ls_args args in
-          check_subst_opacity ols ls sbs in
-        if not (Stv.is_empty ols.ls_opaque) then
-          t_app_fold check () (snd (open_ls_defn ld))
-      in
-      List.iter check dl
-  | Dind (_, dl) ->
-      (* TODO: are there safe classes of inductive predicates? *)
-      let check (ls,_) = if not (Stv.is_empty ls.ls_opaque) then
-        Loc.errorm ?loc:ls.ls_name.id_loc
-          "inductive predicates cannot have opaque type parameters" in
-      List.iter check dl
+let print_id fmt id = Format.fprintf fmt "%s" id.id_string
 
 let warn_dubious_axiom uc k p syms =
   match k with
-  | Plemma | Pgoal | Pskip -> ()
+  | Plemma | Pgoal -> ()
   | Paxiom ->
     try
       Sid.iter
@@ -429,18 +462,19 @@ let warn_dubious_axiom uc k p syms =
           | Dtype { ts_def = NoDef } | Dparam _ -> raise Exit
           | _ -> ())
         syms;
-      Warning.emit ?loc:p.id_loc "axiom %s does not contain any local abstract symbol"
-        p.id_string
+      Warning.emit ?loc:p.id_loc
+        "@[axiom %s does not contain any local abstract symbol@ \
+          (contains: @[%a@])@]" p.id_string
+        (Pp.print_list Pp.comma print_id) (Sid.elements syms)
     with Exit -> ()
 
-let lab_w_non_conservative_extension_no =
-  Ident.create_label "W:non_conservative_extension:N"
+let attr_w_non_conservative_extension_no =
+  Ident.create_attribute "W:non_conservative_extension:N"
 
 let should_be_conservative id =
-  not (Slab.mem lab_w_non_conservative_extension_no id.id_label)
+  not (Sattr.mem attr_w_non_conservative_extension_no id.id_attrs)
 
 let add_decl ?(warn=true) uc d =
-  check_decl_opacity d; (* we don't care about tasks *)
   let uc = add_tdecl uc (create_decl d) in
   match d.d_node with
   | Dtype ts  ->
@@ -469,19 +503,18 @@ let add_decl ?(warn=true) uc d =
       List.fold_left add_ind uc dl
   | Dprop (k,pr,_) ->
       if warn && should_be_conservative uc.uc_name &&
-        should_be_conservative pr.pr_name
+         should_be_conservative pr.pr_name
       then warn_dubious_axiom uc k pr.pr_name d.d_syms;
       add_symbol_pr uc pr
 
 (** Declaration constructors + add_decl *)
 
-let add_ty_decl uc ts = add_decl uc (create_ty_decl ts)
-let add_data_decl uc dl = add_decl uc (create_data_decl dl)
-let add_param_decl uc ls = add_decl uc (create_param_decl ls)
-let add_logic_decl uc dl = add_decl uc (create_logic_decl dl)
-let add_ind_decl uc s dl = add_decl uc (create_ind_decl s dl)
-let add_prop_decl ?warn uc k p f =
-  add_decl ?warn uc (create_prop_decl k p f)
+let add_ty_decl uc ts = add_decl ~warn:false uc (create_ty_decl ts)
+let add_data_decl uc dl = add_decl ~warn:false uc (create_data_decl dl)
+let add_param_decl uc ls = add_decl ~warn:false uc (create_param_decl ls)
+let add_logic_decl uc dl = add_decl ~warn:false uc (create_logic_decl dl)
+let add_ind_decl uc s dl = add_decl ~warn:false uc (create_ind_decl s dl)
+let add_prop_decl ?warn uc k p f = add_decl ?warn uc (create_prop_decl k p f)
 
 (** Use *)
 
@@ -489,48 +522,50 @@ let create_use th = mk_tdecl (Use th)
 
 let use_export uc th =
   let uc = add_tdecl uc (create_use th) in
+  let urng ts d1 d2 = if td_equal d1 d2 then Some d1
+                      else raise (RangeConflict ts) in
+  let uflt ts d1 d2 = if td_equal d1 d2 then Some d1
+                      else raise (FloatConflict ts) in
   match uc.uc_import, uc.uc_export with
   | i0 :: sti, e0 :: ste -> { uc with
       uc_import = merge_ns false th.th_export i0 :: sti;
-      uc_export = merge_ns true  th.th_export e0 :: ste }
+      uc_export = merge_ns true  th.th_export e0 :: ste;
+      uc_ranges = Mts.union urng uc.uc_ranges th.th_ranges;
+      uc_floats = Mts.union uflt uc.uc_floats th.th_floats;
+      uc_crcmap = Coercion.union uc.uc_crcmap th.th_crcmap }
   | _ -> assert false
 
 (** Clone *)
 
 type th_inst = {
-  inst_ts    : tysymbol Mts.t;
-  inst_ls    : lsymbol  Mls.t;
-  inst_lemma : Spr.t;
-  inst_goal  : Spr.t;
+  inst_ty : ty Mts.t;
+  inst_ts : tysymbol Mts.t;
+  inst_ls : lsymbol Mls.t;
+  inst_pr : prop_kind Mpr.t;
+  inst_df : prop_kind;
 }
 
 let empty_inst = {
-  inst_ts    = Mts.empty;
-  inst_ls    = Mls.empty;
-  inst_lemma = Spr.empty;
-  inst_goal  = Spr.empty;
+  inst_ty = Mts.empty;
+  inst_ts = Mts.empty;
+  inst_ls = Mls.empty;
+  inst_pr = Mpr.empty;
+  inst_df = Plemma;
 }
 
-let create_inst ~ts ~ls ~lemma ~goal =
-  let add_ts acc (tso,tsn) = Mts.add tso tsn acc in
-  let add_ls acc (lso,lsn) = Mls.add lso lsn acc in
-  let add_pr acc lem = Spr.add lem acc in {
-    inst_ts    = List.fold_left add_ts Mts.empty ts;
-    inst_ls    = List.fold_left add_ls Mls.empty ls;
-    inst_lemma = List.fold_left add_pr Spr.empty lemma;
-    inst_goal  = List.fold_left add_pr Spr.empty goal }
-
 exception CannotInstantiate of ident
 
 type clones = {
   cl_local : Sid.t;
+  mutable ty_table : ty Mts.t;
   mutable ts_table : tysymbol Mts.t;
   mutable ls_table : lsymbol Mls.t;
   mutable pr_table : prsymbol Mpr.t;
 }
 
-let empty_clones s = {
-  cl_local = s;
+let empty_clones th = {
+  cl_local = th.th_local;
+  ty_table = Mts.empty;
   ts_table = Mts.empty;
   ls_table = Mls.empty;
   pr_table = Mpr.empty;
@@ -538,7 +573,20 @@ let empty_clones s = {
 
 (* populate the clone structure *)
 
-let rec cl_find_ts cl ts =
+let rec sm_trans_ty tym tsm ty = match ty.ty_node with
+  | Tyapp (ts, tyl) ->
+      let tyl = List.map (sm_trans_ty tym tsm) tyl in
+      begin match Mts.find_opt ts tsm with
+      | Some ts -> ty_app ts tyl
+      | None -> begin match Mts.find_opt ts tym with
+      | Some ty -> ty_inst (ts_match_args ts tyl) ty
+      | None -> ty_app ts tyl
+      end end
+  | Tyvar _ -> ty
+
+let cl_trans_ty cl ty = sm_trans_ty cl.ty_table cl.ts_table ty
+
+let cl_find_ts cl ts =
   if not (Sid.mem ts.ts_name cl.cl_local) then
     match ts.ts_def with
       | Alias ty ->
@@ -547,109 +595,105 @@ let rec cl_find_ts cl ts =
           let id = id_clone ts.ts_name in
           create_tysymbol id ts.ts_args (Alias td)
       | NoDef | Range _ | Float _ -> ts
-  else try Mts.find ts cl.ts_table
-  with Not_found ->
-    let id' = id_clone ts.ts_name in
-    let td' = type_def_map (cl_trans_ty cl) ts.ts_def in
-    let ts' = create_tysymbol id' ts.ts_args td' in
-    cl.ts_table <- Mts.add ts ts' cl.ts_table;
-    ts'
+  else
+  try Mts.find ts cl.ts_table with Not_found -> raise EmptyDecl
 
-and cl_trans_ty cl ty = ty_s_map (cl_find_ts cl) ty
+let cl_clone_ts cl ts =
+  (* cl_clone_ts is only called for local non-instantiated symbols *)
+  let td' =
+    match ts.ts_def with
+    | Alias t -> Alias (cl_trans_ty cl t)
+    | NoDef -> NoDef
+    | Range _ -> assert false (* TODO *)
+    | Float _ -> assert false (* TODO *) in
+  let ts' = create_tysymbol (id_clone ts.ts_name) ts.ts_args td' in
+  cl.ts_table <- Mts.add ts ts' cl.ts_table;
+  ts'
 
 let cl_find_ls cl ls =
-  if not (Sid.mem ls.ls_name cl.cl_local) then ls
-  else try Mls.find ls cl.ls_table
-  with Not_found ->
-    let opaque = ls.ls_opaque in
+  if not (Sid.mem ls.ls_name cl.cl_local) then ls else
+  try Mls.find ls cl.ls_table with Not_found ->
     let constr = ls.ls_constr in
     let id  = id_clone ls.ls_name in
     let ta' = List.map (cl_trans_ty cl) ls.ls_args in
     let vt' = Opt.map (cl_trans_ty cl) ls.ls_value in
-    let stv = Opt.fold ty_freevars Stv.empty vt' in
-    let stv = List.fold_left ty_freevars stv ta' in
-    let opaque = Stv.inter opaque stv in
-    let ls' = create_lsymbol ~opaque ~constr id ta' vt' in
+    let ls' = create_lsymbol ~constr id ta' vt' in
     cl.ls_table <- Mls.add ls ls' cl.ls_table;
     ls'
 
 let cl_trans_fmla cl f = t_s_map (cl_trans_ty cl) (cl_find_ls cl) f
 
 let cl_find_pr cl pr =
-  if not (Sid.mem pr.pr_name cl.cl_local) then pr
-  else try Mpr.find pr cl.pr_table
-  with Not_found ->
-    let pr' = create_prsymbol (id_clone pr.pr_name) in
-    cl.pr_table <- Mpr.add pr pr' cl.pr_table;
-    pr'
+  if not (Sid.mem pr.pr_name cl.cl_local) then pr else
+  try Mpr.find pr cl.pr_table with Not_found -> raise EmptyDecl
+
+let cl_clone_pr cl pr =
+  let pr' = create_prsymbol (id_clone pr.pr_name) in
+  cl.pr_table <- Mpr.add pr pr' cl.pr_table;
+  pr'
 
 (* initialize the clone structure *)
 
 exception NonLocal of ident
-exception BadInstance of ident * ident
+exception BadInstance of ident
 
-let cl_init_ts cl ts ts' =
-  let id = ts.ts_name in
+let cl_init_ty cl ({ts_name = id} as ts) ty =
   if not (Sid.mem id cl.cl_local) then raise (NonLocal id);
-  if List.length ts.ts_args <> List.length ts'.ts_args
-    then raise (BadInstance (id, ts'.ts_name));
+  let stv = Stv.of_list ts.ts_args in
+  if not (ty_v_all (Stv.contains stv) ty) then raise (BadInstance id);
+  cl.ty_table <- Mts.add ts ty cl.ty_table
+
+let cl_init_ts cl ({ts_name = id} as ts) ts' =
+  if not (Sid.mem id cl.cl_local) then raise (NonLocal id);
+  if List.length ts.ts_args <> List.length ts'.ts_args then
+    raise (BadInstance id);
   cl.ts_table <- Mts.add ts ts' cl.ts_table
 
-let cl_init_ls cl ls ls' =
-  let id = ls.ls_name in
+let cl_init_ls cl ({ls_name = id} as ls) ls' =
   if not (Sid.mem id cl.cl_local) then raise (NonLocal id);
   let mtch sb ty ty' =
     try ty_match sb ty' (cl_trans_ty cl ty)
-    with TypeMismatch _ -> raise (BadInstance (id, ls'.ls_name))
+    with TypeMismatch _ -> raise (BadInstance id)
   in
   let sb = match ls.ls_value,ls'.ls_value with
     | Some ty, Some ty' -> mtch Mtv.empty ty ty'
     | None, None -> Mtv.empty
-    | _ -> raise (BadInstance (id, ls'.ls_name))
+    | _ -> raise (BadInstance id)
   in
-  let sb = try List.fold_left2 mtch sb ls.ls_args ls'.ls_args
-    with Invalid_argument _ -> raise (BadInstance (id, ls'.ls_name))
-  in
-  check_subst_opacity ls ls' sb;
+  ignore (try List.fold_left2 mtch sb ls.ls_args ls'.ls_args
+    with Invalid_argument _ -> raise (BadInstance id));
   cl.ls_table <- Mls.add ls ls' cl.ls_table
 
-let cl_init_pr cl pr =
-  let id = pr.pr_name in
+let cl_init_pr cl {pr_name = id} _ =
   if not (Sid.mem id cl.cl_local) then raise (NonLocal id)
 
 let cl_init th inst =
-  let cl = empty_clones th.th_local in
+  let cl = empty_clones th in
+  Mts.iter (cl_init_ty cl) inst.inst_ty;
   Mts.iter (cl_init_ts cl) inst.inst_ts;
   Mls.iter (cl_init_ls cl) inst.inst_ls;
-  Spr.iter (cl_init_pr cl) inst.inst_lemma;
-  Spr.iter (cl_init_pr cl) inst.inst_goal;
+  Mpr.iter (cl_init_pr cl) inst.inst_pr;
   cl
 
 (* clone declarations *)
 
 let cl_type cl inst ts =
-  if Mts.mem ts inst.inst_ts then
+  if Mts.mem ts inst.inst_ts || Mts.mem ts inst.inst_ty then
     if ts.ts_def = NoDef then raise EmptyDecl
     else raise (CannotInstantiate ts.ts_name);
-  create_ty_decl (cl_find_ts cl ts)
+  create_ty_decl (cl_clone_ts cl ts)
 
 let cl_data cl inst tdl =
   let add_ls ls =
-    if Mls.mem ls inst.inst_ls then
-      raise (CannotInstantiate ls.ls_name);
-    cl_find_ls cl ls
-  in
-  let add_constr (ls,pl) =
-    add_ls ls, List.map (Opt.map add_ls) pl
-  in
-  let add_type (ts,csl) =
-    if Mts.mem ts inst.inst_ts then
+    if Mls.mem ls inst.inst_ls then raise (CannotInstantiate ls.ls_name);
+    cl_find_ls cl ls in
+  let add_constr (ls,pl) = add_ls ls, List.map (Opt.map add_ls) pl in
+  let add_type ts' (_,csl) = ts', List.map add_constr csl in
+  let get_type (ts,_) =
+    if Mts.mem ts inst.inst_ts || Mts.mem ts inst.inst_ty then
       raise (CannotInstantiate ts.ts_name);
-    let ts' = cl_find_ts cl ts in
-    let td' = List.map add_constr csl in
-    (ts',td')
-  in
-  create_data_decl (List.map add_type tdl)
+    cl_clone_ts cl ts in
+  create_data_decl (List.map2 add_type (List.map get_type tdl) tdl)
 
 let extract_ls_defn f =
   let vl,_,f = match f.t_node with
@@ -666,18 +710,15 @@ let cl_param cl inst ls =
 
 let cl_logic cl inst ldl =
   let add_logic (ls,ld) =
-    if Mls.mem ls inst.inst_ls then
-      raise (CannotInstantiate ls.ls_name);
-    let f = ls_defn_axiom ld in
-    extract_ls_defn (cl_trans_fmla cl f)
+    if Mls.mem ls inst.inst_ls then raise (CannotInstantiate ls.ls_name);
+    extract_ls_defn (cl_trans_fmla cl (ls_defn_axiom ld))
   in
   create_logic_decl (List.map add_logic ldl)
 
 let cl_ind cl inst (s, idl) =
   let add_case (pr,f) =
-    if Spr.mem pr inst.inst_lemma || Spr.mem pr inst.inst_goal
-      then raise (CannotInstantiate pr.pr_name)
-      else cl_find_pr cl pr, cl_trans_fmla cl f
+    if Mpr.mem pr inst.inst_pr then raise (CannotInstantiate pr.pr_name);
+    cl_clone_pr cl pr, cl_trans_fmla cl f
   in
   let add_ind (ps,la) =
     if Mls.mem ps inst.inst_ls
@@ -687,14 +728,13 @@ let cl_ind cl inst (s, idl) =
   create_ind_decl s (List.map add_ind idl)
 
 let cl_prop cl inst (k,pr,f) =
-  let k' = match k with
-    | Pskip | Pgoal -> Pskip
-    | Plemma when Spr.mem pr inst.inst_goal -> Pskip
-    | Paxiom when Spr.mem pr inst.inst_goal -> Pgoal
-    | Paxiom when Spr.mem pr inst.inst_lemma -> Plemma
-    | Paxiom | Plemma -> Paxiom
-  in
-  let pr' = cl_find_pr cl pr in
+  let k' = match k, Mpr.find_opt pr inst.inst_pr with
+    | Pgoal, _ -> raise EmptyDecl
+    | Plemma, Some Pgoal -> raise EmptyDecl
+    | Plemma, _ -> Plemma
+    | Paxiom, Some k -> k
+    | Paxiom, None -> inst.inst_df in
+  let pr' = cl_clone_pr cl pr in
   let f' = cl_trans_fmla cl f in
   create_prop_decl k' pr' f'
 
@@ -714,6 +754,7 @@ let cl_marg cl = function
   | a -> a
 
 let cl_smap cl sm = {
+  sm_ty = Mts.map (cl_trans_ty cl) sm.sm_ty;
   sm_ts = Mts.map (cl_find_ts cl) sm.sm_ts;
   sm_ls = Mls.map (cl_find_ls cl) sm.sm_ls;
   sm_pr = Mpr.map (cl_find_pr cl) sm.sm_pr}
@@ -752,6 +793,7 @@ let clone_theory cl add_td acc th inst =
   in
   let acc = List.fold_left add acc th.th_decls in
   let sm = {
+    sm_ty = cl.ty_table;
     sm_ts = cl.ts_table;
     sm_ls = cl.ls_table;
     sm_pr = cl.pr_table}
@@ -762,23 +804,31 @@ let clone_export uc th inst =
   let cl = cl_init th inst in
   let uc = clone_theory cl add_tdecl uc th inst in
 
-  let g_ts _ ts = not (Mts.mem ts inst.inst_ts) in
-  let g_ls _ ls = not (Mls.mem ls inst.inst_ls) in
-
   let f_ts p ts =
-    try let ts = Mts.find ts cl.ts_table in store_path uc p ts.ts_name; ts
-    with Not_found -> ts in
+    if Mid.mem ts.ts_name th.th_local then
+    if Mts.mem ts inst.inst_ts then None else
+    if Mts.mem ts inst.inst_ty then None else
+    try let ts = Mts.find ts cl.ts_table in
+        store_path uc p ts.ts_name; Some ts
+    with Not_found -> None else Some ts in
+
   let f_ls p ls =
-    try let ls = Mls.find ls cl.ls_table in store_path uc p ls.ls_name; ls
-    with Not_found -> ls in
+    if Mid.mem ls.ls_name th.th_local then
+    if Mls.mem ls inst.inst_ls then None else
+    try let ls = Mls.find ls cl.ls_table in
+        store_path uc p ls.ls_name; Some ls
+    with Not_found -> None else Some ls in
+
   let f_pr p pr =
-    try let pr = Mpr.find pr cl.pr_table in store_path uc p pr.pr_name; pr
-    with Not_found -> pr in
+    if Mid.mem pr.pr_name th.th_local then
+    try let pr = Mpr.find pr cl.pr_table in
+        store_path uc p pr.pr_name; Some pr
+    with Not_found -> None else Some pr in
 
   let rec f_ns p ns = {
-    ns_ts = Mstr.map (f_ts p) (Mstr.filter g_ts ns.ns_ts);
-    ns_ls = Mstr.map (f_ls p) (Mstr.filter g_ls ns.ns_ls);
-    ns_pr = Mstr.map (f_pr p) ns.ns_pr;
+    ns_ts = Mstr.map_filter (f_ts p) ns.ns_ts;
+    ns_ls = Mstr.map_filter (f_ls p) ns.ns_ls;
+    ns_pr = Mstr.map_filter (f_pr p) ns.ns_pr;
     ns_ns = Mstr.mapi (fun n -> f_ns (n::p)) ns.ns_ns; } in
 
   let ns = f_ns [] th.th_export in
@@ -792,18 +842,13 @@ let clone_export uc th inst =
 let clone_theory add_td acc th inst =
   clone_theory (cl_init th inst) add_td acc th inst
 
-let create_null_clone th =
-  let sm = {
-    sm_ts = Mts.empty;
-    sm_ls = Mls.empty;
-    sm_pr = Mpr.empty}
-  in
-  mk_tdecl (Clone (th,sm))
+let add_clone_unsafe uc th sm =
+  add_tdecl uc (mk_tdecl (Clone (th, sm)))
 
-let is_empty_sm sm =
-  Mts.is_empty sm.sm_ts &&
-  Mls.is_empty sm.sm_ls &&
-  Mpr.is_empty sm.sm_pr
+let add_clone_internal =
+  let used = ref false in fun () -> if !used
+    then invalid_arg "Theory.add_clone_internal"
+    else begin used := true; add_clone_unsafe end
 
 (** Meta properties *)
 
@@ -833,48 +878,24 @@ let create_meta m al =
 
 let add_meta uc s al = add_tdecl uc (create_meta s al)
 
-let clone_meta tdt sm = match tdt.td_node with
+let clone_meta tdt th sm = match tdt.td_node with
   | Meta (t,al) ->
-      let find_ts ts = Mts.find_def ts ts sm.sm_ts in
-      let find_ls ls = Mls.find_def ls ls sm.sm_ls in
-      let find_pr pr = Mpr.find_def pr pr sm.sm_pr in
+      let find_ts ts = if Mid.mem ts.ts_name th.th_local
+        then Mts.find ts sm.sm_ts else ts in
+      let find_ls ls = if Mid.mem ls.ls_name th.th_local
+        then Mls.find ls sm.sm_ls else ls in
+      let find_pr pr = if Mid.mem pr.pr_name th.th_local
+        then Mpr.find pr sm.sm_pr else pr in
       let cl_marg = function
-        | MAty ty -> MAty (ty_s_map find_ts ty)
+        | MAty ty -> MAty (sm_trans_ty sm.sm_ty sm.sm_ts ty)
         | MAts ts -> MAts (find_ts ts)
         | MAls ls -> MAls (find_ls ls)
         | MApr pr -> MApr (find_pr pr)
         | a -> a
       in
-      mk_tdecl (Meta (t, List.map cl_marg al))
-  | _ -> invalid_arg "clone_meta"
-
-(** access to meta *)
-
-(*
-let theory_meta = Opt.fold (fun _ t -> t.task_meta) Mmeta.empty
-
-let find_meta_tds th (t : meta) = mm_find (theory_meta th) t
-
-let on_meta meta fn acc theory =
-  let add td acc = match td.td_node with
-    | Meta (_,ma) -> fn acc ma
-    | _ -> assert false
-  in
-  let tds = find_meta_tds theory meta in
-  Stdecl.fold add tds.tds_set acc
-*)
-
-(*
-let on_meta _meta fn acc theory =
-  let tdecls = theory.th_decls in
-  List.fold_left
-    (fun acc td ->
-       match td.td_node with
-         | Meta (_,ma) -> fn acc ma
-         | _ -> acc)
-    acc tdecls
-*)
-
+      begin try Some (mk_tdecl (Meta (t, List.map cl_marg al)))
+      with Not_found -> None end
+  | _ -> invalid_arg "Theory.clone_meta"
 
 (** Base theories *)
 
@@ -895,9 +916,7 @@ let bool_theory =
 
 let highord_theory =
   let uc = empty_theory (id_fresh "HighOrd") ["why3";"HighOrd"] in
-  let uc = use_export uc bool_theory in
   let uc = add_ty_decl uc ts_func in
-  let uc = add_ty_decl uc ts_pred in
   let uc = add_param_decl uc fs_func_app in
   close_theory uc
 
@@ -909,13 +928,6 @@ let tuple_theory = Hint.memo 17 (fun n ->
   let uc = add_data_decl uc [ts, [fs,pl]] in
   close_theory uc)
 
-let unit_theory =
-  let uc = empty_theory (id_fresh "Unit") ["why3";"Unit"] in
-  let ts = create_tysymbol (id_fresh "unit") [] (Alias (ty_tuple [])) in
-  let uc = use_export uc (tuple_theory 0) in
-  let uc = add_ty_decl uc ts in
-  close_theory uc
-
 let tuple_theory_name s =
   let l = String.length s in
   if l < 6 then None else
@@ -934,7 +946,7 @@ let add_decl_with_tuples uc d =
     | None -> s in
   let ixs = Sid.fold add ids Sint.empty in
   let add n uc = use_export uc (tuple_theory n) in
-  add_decl (Sint.fold add ixs uc) d
+  add_decl ~warn:false (Sint.fold add ixs uc) d
 
 (* Exception reporting *)
 
@@ -949,12 +961,11 @@ let print_meta_arg_type fmt = function
 let () = Exn_printer.register
   begin fun fmt exn -> match exn with
   | NonLocal id ->
-      Format.fprintf fmt "Non-local ident: %s" id.id_string
+      Format.fprintf fmt "Non-local symbol: %s" id.id_string
   | CannotInstantiate id ->
-      Format.fprintf fmt "Cannot instantiate a defined ident %s" id.id_string
-  | BadInstance (id1, id2) ->
-      Format.fprintf fmt "Cannot instantiate ident %s with %s"
-        id1.id_string id2.id_string
+      Format.fprintf fmt "Cannot instantiate a defined symbol %s" id.id_string
+  | BadInstance id ->
+      Format.fprintf fmt "Illegal instantiation for symbol %s" id.id_string
   | CloseTheory ->
       Format.fprintf fmt "Cannot close theory: some namespaces are still open"
   | NoOpenedNamespace ->
@@ -962,7 +973,7 @@ let () = Exn_printer.register
   | ClashSymbol s ->
       Format.fprintf fmt "Symbol %s is already defined in the current scope" s
   | UnknownMeta s ->
-      Format.fprintf fmt "Unknown metaproperty %s" s
+      Format.fprintf fmt "Unknown meta-property %s" s
   | KnownMeta m ->
       Format.fprintf fmt "Metaproperty %s is already registered with \
         a conflicting signature" m.meta_name
@@ -974,5 +985,11 @@ let () = Exn_printer.register
       Format.fprintf fmt "Metaproperty %s expects a %a argument but \
         is applied to %a"
         m.meta_name print_meta_arg_type t1 print_meta_arg_type t2
+  | RangeConflict ts ->
+      Format.fprintf fmt "Conflicting definitions for range type %s"
+        ts.ts_name.id_string
+  | FloatConflict ts ->
+      Format.fprintf fmt "Conflicting definitions for float type %s"
+        ts.ts_name.id_string
   | _ -> raise exn
   end
