@@ -127,6 +127,9 @@ val ty_match : ty Mtv.t -> ty -> ty -> ty Mtv.t
     exists.
 *)
 
+val ts_match_args : tysymbol -> ty list -> ty Mtv.t
+val ty_match_args : ty -> ty Mtv.t
+
 val ty_inst  : ty Mtv.t -> ty -> ty
 val ty_freevars : Stv.t -> ty -> Stv.t
 val ty_closed : ty -> bool
@@ -145,7 +148,6 @@ val ty_real : ty
 val ty_bool : ty
 
 val ts_func : tysymbol
-val ts_pred : tysymbol
 
 val ty_func : ty -> ty -> ty
 val ty_pred : ty -> ty (* ty_pred 'a == ty_func 'a bool *)
