@@ -40,7 +40,7 @@ The regexp must start with ^.
 *)
 
 open Format
-open Stdlib
+open Wstdlib
 open Whyconf
 module Wc = Whyconf
 open Rc
@@ -57,13 +57,13 @@ type prover_autodetection_data =
       prover_id : string;
       prover_name : string;
       prover_altern : string;
-      compile_time_support : bool;
+      support_library : string;
       execs : string list;
       version_switch : string;
       version_regexp : string;
-      versions_ok : Str.regexp list;
-      versions_old : Str.regexp list;
-      versions_bad : Str.regexp list;
+      versions_ok : (string * Str.regexp) list;
+      versions_old : (string * Str.regexp) list;
+      versions_bad : (string * Str.regexp) list;
       (** If none it's a fake prover (very bad version) *)
       prover_command : string option;
       prover_command_steps : string option;
@@ -77,20 +77,19 @@ type prover_autodetection_data =
 let prover_keys =
   let add acc k = Sstr.add k acc in
   List.fold_left add Sstr.empty
-    ["name";"compile_time_support";
+    ["name";"support_library";
      "exec";"version_switch";"version_regexp";
      "version_ok";"version_old";"version_bad";"command"; "command_steps";
      "editor";"driver";"in_place";"message";"alternative";"use_at_auto_level"]
 
 let load_prover kind (id,section) =
   check_exhaustive section prover_keys;
-  let reg_map = List.rev_map why3_regexp_of_string in
+  let reg_map = List.rev_map (fun s -> (s,why3_regexp_of_string s)) in
   let prover = { kind = kind;
     prover_id = id;
     prover_name = get_string section "name";
     prover_altern = get_string section ~default:"" "alternative";
-    compile_time_support =
-      get_bool section ~default:false "compile_time_support";
+    support_library = get_string section ~default:"" "support_library";
     execs = get_stringl section "exec";
     version_switch = get_string section ~default:"" "version_switch";
     version_regexp = get_string section ~default:"" "version_regexp";
@@ -158,7 +157,7 @@ type env =
                                   unknown version (neither good or bad)
                      | None               there is a good version *)
     prover_unknown_version :
-      (int * string * config_prover) option Hstr.t;
+      (prover_autodetection_data * int * string * config_prover) option Hstr.t;
     (* string -> priority * prover  *)
     prover_shortcuts : (int * prover) Hstr.t;
     mutable possible_prover_shortcuts : (filter_prover * string) list;
@@ -230,8 +229,8 @@ let sanitize_exec =
 let ask_prover_version exec_name version_switch =
   let out = Filename.temp_file "out" "" in
   let cmd = sprintf "%s %s" exec_name version_switch in
-  let c = sprintf "(%s) >%s 2>&1" cmd out in
-  Debug.dprintf debug "Run : %s@." c;
+  let c = sprintf "(%s) > %s 2>&1" cmd out in
+  Debug.dprintf debug "Run: %s@." c;
   try
     let ret = Sys.command c in
     let ch = open_in out in
@@ -254,15 +253,15 @@ let ask_prover_version env exec_name version_switch =
     Hstr2.add env.prover_output (exec_name,version_switch) res;
     res
 
-let check_version version schema = Str.string_match schema version 0
+let check_version version (_,schema) = Str.string_match schema version 0
 
 let known_version env exec_name =
   Hstr.replace env.prover_unknown_version exec_name None
 
-let unknown_version env exec_name prover_id prover_config priority =
+let unknown_version env exec_name prover_id prover_config data priority =
   if not (Hstr.mem env.prover_unknown_version exec_name)
   then Hstr.replace env.prover_unknown_version exec_name
-    (Some (priority,prover_id,prover_config))
+    (Some (data,priority,prover_id,prover_config))
 
 
 let empty_alt s = if s = "" then "alt" else s
@@ -327,48 +326,37 @@ let generate_auto_strategies config =
     (fun p (lev,b) ->
      if b then eprintf "  Prover %a will be used in Auto level >= %d@."
                        Whyconf.print_prover p lev) prover_auto_levels;
-  (* Split *)
-  let code = "t split_goal_wp exit" in
+  (* Split VCs *)
+  let code = "t split_vc exit" in
   let split = {
-      strategy_name = "Split";
-      strategy_desc = "Split@ the@ goal@ into@ subgoals";
+      strategy_name = "Split_VC";
+      strategy_desc = "Split@ the@ VC@ into@ subgoals";
       strategy_shortcut = "s";
       strategy_code = code }
   in
-  (* Inline *)
-  let code = "t introduce_premises next next: t inline_goal exit" in
-  let inline = {
-      strategy_name = "Inline";
-      strategy_desc = "Inline@ definitions@ in@ the@ conclusion@ of@ the@ goal";
-      strategy_shortcut = "i";
-      strategy_code = code }
-  in
   (* Auto level 0 and 1 *)
   let provers_level1 =
     Hprover.fold
       (fun p (lev,b) acc ->
        if b && lev = 1 then
-         let name =
-           p.Whyconf.prover_name ^ "," ^
-             p.Whyconf.prover_version ^ "," ^ p.Whyconf.prover_altern
-         in name :: acc
+         let name = Whyconf.prover_parseable_format p in name :: acc
        else acc) prover_auto_levels []
   in
   List.iter (fun s -> fprintf str_formatter "c %s 1 1000@\n" s) provers_level1;
   let code = flush_str_formatter () in
   let auto0 = {
-      strategy_name = "Auto level 0";
+      strategy_name = "Auto_level_0";
       strategy_desc = "Automatic@ run@ of@ main@ provers";
       strategy_shortcut = "0";
       strategy_code = code }
   in
   fprintf str_formatter "start:@\n";
   List.iter (fun s -> fprintf str_formatter "c %s 1 1000@\n" s) provers_level1;
-  fprintf str_formatter "t split_goal_wp start@\n";
+  fprintf str_formatter "t split_all_full start@\n";
   List.iter (fun s -> fprintf str_formatter "c %s 10 4000@\n" s) provers_level1;
   let code = flush_str_formatter () in
   let auto1 = {
-      strategy_name = "Auto level 1";
+      strategy_name = "Auto_level_1";
       strategy_desc = "Automatic@ run@ of@ main@ provers";
       strategy_shortcut = "1";
       strategy_code = code }
@@ -378,27 +366,24 @@ let generate_auto_strategies config =
     Hprover.fold
       (fun p (lev,b) acc ->
        if b && lev >= 1 && lev <= 2 then
-         let name =
-           p.Whyconf.prover_name ^ "," ^
-             p.Whyconf.prover_version ^ "," ^ p.Whyconf.prover_altern
-         in name :: acc
+         let name = Whyconf.prover_parseable_format p in name :: acc
        else acc) prover_auto_levels []
   in
   fprintf str_formatter "start:@\n";
   List.iter (fun s -> fprintf str_formatter "c %s 1 1000@\n" s) provers_level2;
-  fprintf str_formatter "t split_goal_wp start@\n";
+  fprintf str_formatter "t split_all_full start@\n";
   List.iter (fun s -> fprintf str_formatter "c %s 5 2000@\n" s) provers_level2;
   fprintf str_formatter "t introduce_premises afterintro@\n";
   fprintf str_formatter "afterintro:@\n";
   fprintf str_formatter "t inline_goal afterinline@\n";
   fprintf str_formatter "g trylongertime@\n";
   fprintf str_formatter "afterinline:@\n";
-  fprintf str_formatter "t split_goal_wp start@\n";
+  fprintf str_formatter "t split_all_full start@\n";
   fprintf str_formatter "trylongertime:@\n";
   List.iter (fun s -> fprintf str_formatter "c %s 30 4000@\n" s) provers_level2;
   let code = flush_str_formatter () in
   let auto2 = {
-      strategy_name = "Auto level 2";
+      strategy_name = "Auto_level_2";
       strategy_desc = "Automatic@ run@ of@ provers@ and@ most@ useful@ transformations";
       strategy_shortcut = "2";
       strategy_code = code }
@@ -406,138 +391,154 @@ let generate_auto_strategies config =
   add_strategy
     (add_strategy
        (add_strategy
-          (add_strategy
-             (add_strategy config inline)
-             split) auto0) auto1) auto2
+          (add_strategy config split) auto0) auto1) auto2
 
-let detect_exec env data acc exec_name =
+let check_support_library data ver =
+  let cmd_regexp = Str.regexp "%\\(.\\)" in
+  let replace s = match Str.matched_group 1 s with
+    | "l" -> Config.libdir
+    | "d" -> Config.datadir
+    | c -> c in
+  let sl = Str.global_substitute cmd_regexp replace data.support_library in
+  try
+    let f = open_in sl in
+    let support_ver = input_line f in
+    close_in f;
+    if support_ver = ver then true
+    else if support_ver = "" then raise Not_found
+    else begin
+      eprintf
+        "Found prover %s version %s, but the compiled Why3 library supports only version %s@."
+        data.prover_name ver support_ver;
+      false
+    end
+  with Sys_error _ | Not_found ->
+    eprintf
+      "Found prover %s version %s, but no Why3 libraries were compiled for it@."
+      data.prover_name ver;
+    false
+
+exception Skip    (* prover is ignored *)
+exception Discard (* prover is recognized, but unusable *)
+
+let detect_exec env data exec_name =
   let s = ask_prover_version env exec_name data.version_switch in
-  match s with
-  | None -> acc
-  | Some s ->
+  let s =
+    match s with
+    | None -> raise Skip
+    | Some s -> s in
   let re = Str.regexp data.version_regexp in
   let ver =
     try
       ignore (Str.search_forward re s 0);
       Str.matched_group 1 s
     with Not_found ->
-      begin
-        Debug.dprintf debug "Warning: found prover %s but name/version not \
+      Debug.dprintf debug "Warning: found prover %s but name/version not \
                        recognized by regexp `%s'@."
-          data.prover_name data.version_regexp;
-        Debug.dprintf debug "Answer was `%s'@." s;
-        ""
-      end
+        data.prover_name data.version_regexp;
+      Debug.dprintf debug "Answer was `%s'@." s;
+      ""
   in
-  (* bad mean here not good, it's not the same thing than a version
+  (* bad here means not good, it is not the same thing as a version
      of a prover with known problems *)
   let bad = List.exists (check_version ver) data.versions_bad in
-  if bad then (known_version env exec_name; acc)
-  else
-    (* check if this prover needs compile-time support *)
-    let missing_compile_time_support =
-      if data.compile_time_support then
-        try
-          let compile_time_ver =
-            List.assoc data.prover_name Config.compile_time_support
-          in
-          if compile_time_ver <> ver then begin
-            eprintf
-              "Found prover %s version %s, but Why3 was compiled with support for version %s@."
-            data.prover_name ver compile_time_ver;
-            true
-          end
-          else
-            false
-        with Not_found ->
-          eprintf
-            "Found prover %s version %s, but Why3 wasn't compiled with support for it@."
-            data.prover_name ver;
-          true
-      else false
-    in
-    if missing_compile_time_support then
-      (known_version env exec_name; acc)
-    else
-    let good = List.exists (check_version ver) data.versions_ok in
-    let old  = List.exists (check_version ver) data.versions_old in
+  if bad then raise Discard;
+  let good = List.exists (check_version ver) data.versions_ok in
+  let old  = List.exists (check_version ver) data.versions_old in
+  let prover_command =
     match data.prover_command with
     | None ->
-      (* Empty prover *)
-      if good || old then begin (* Known version with error *)
-        known_version env exec_name;
-        eprintf "Found prover %s version %s%s@."
-          data.prover_name ver
-          (Opt.get_def
-             ". This version of the prover is known to have problems."
-             data.message);
-        acc
-      end
-      else (* it's not a known bad version *) acc
-
-    | Some prover_command ->
-    (* create the prover config *)
-    let c = make_command exec_name prover_command in
-    let c_steps = (match data.prover_command_steps with
-      | None -> None
-      | Some prover_command_steps ->
-	Some (make_command exec_name prover_command_steps)) in
-    let prover = {Wc.prover_name = data.prover_name;
-                  prover_version      = ver;
-                  prover_altern       = data.prover_altern} in
-    let prover_config =
-      {prover  = prover;
-       command = c;
-       command_steps = c_steps;
-       driver  = data.prover_driver;
-       editor  = data.prover_editor;
-       in_place  = data.prover_in_place;
-       interactive   = (match data.kind with ITP -> true | ATP -> false);
-       extra_options = [];
-       extra_drivers = [] } in
-
-    let priority = next_priority () in
-    if good || old then begin
+      (* empty prover: a matching version means known problems *)
+      if not (good || old) then raise Skip;
       eprintf "Found prover %s version %s%s@."
         data.prover_name ver
         (Opt.get_def
-	   (if old then
-	      " (old version, please consider upgrading)."
-	    else
-	      if data.prover_altern <> "" then
-		" (alternative: " ^ data.prover_altern ^ ")"
-	      else
-		", OK.")
+           ". This version of the prover is known to have problems."
            data.message);
-      known_version env exec_name;
-      add_prover_shortcuts env prover;
-      add_id_prover_shortcut env data.prover_id prover priority;
-      let level = data.use_at_auto_level in
-      if level > 0 then record_prover_for_auto_mode prover level;
-      add_prover_with_uniq_id prover_config acc
-    end
-    else (unknown_version env exec_name data.prover_id prover_config priority;
-          acc)
+      raise Discard
+    | Some prover_command -> prover_command
+  in
+  (* create the prover config *)
+  let c = make_command exec_name prover_command in
+  let c_steps = Opt.map (make_command exec_name) data.prover_command_steps in
+  let prover =
+    { Wc.prover_name = data.prover_name;
+      prover_version = ver;
+      prover_altern = data.prover_altern } in
+  let prover_config =
+    { prover = prover;
+      command = c;
+      command_steps = c_steps;
+      driver = data.prover_driver;
+      editor = data.prover_editor;
+      in_place = data.prover_in_place;
+      interactive = (match data.kind with ITP -> true | ATP -> false);
+      extra_options = [];
+      extra_drivers = [] } in
+  (* if unknown, temporarily put the prover away *)
+  if not (good || old) then begin
+    let priority = next_priority () in
+    unknown_version env exec_name data.prover_id prover_config data priority;
+    raise Skip
+  end;
+  (* check if this prover needs compile-time support *)
+  if data.support_library <> "" && not (check_support_library data ver) then
+    raise Discard;
+  let priority = next_priority () in
+  eprintf "Found prover %s version %s%s@."
+    data.prover_name ver
+    (Opt.get_def
+       (if old then
+	   " (old version, please consider upgrading)."
+	else
+	   if data.prover_altern <> "" then
+	     " (alternative: " ^ data.prover_altern ^ ")"
+	   else
+	     ", OK.")
+       data.message);
+  add_prover_shortcuts env prover;
+  add_id_prover_shortcut env data.prover_id prover priority;
+  let level = data.use_at_auto_level in
+  if level > 0 then record_prover_for_auto_mode prover level;
+  prover_config
+
+let detect_exec env data acc exec_name =
+  try
+    let prover_config = detect_exec env data exec_name in
+    known_version env exec_name;
+    add_prover_with_uniq_id prover_config acc
+  with
+  | Skip -> acc
+  | Discard -> known_version env exec_name; acc
 
 let detect_prover env acc data =
   List.fold_left (detect_exec env data) acc data.execs
 
-(** add the prover unknown *)
+let pp_versions =
+  Pp.print_list Pp.comma
+                (Pp.print_pair_delim Pp.nothing Pp.nothing Pp.nothing Pp.string Pp.nothing)
+
+let detect_unknown env pc =
+  let (data,priority,prover_id,prover_config) =
+    match pc with
+    | None -> raise Skip
+    | Some pc -> pc in
+  let prover = prover_config.prover in
+  let ver = prover.prover_version in
+  if data.support_library <> "" && not (check_support_library data ver) then raise Skip;
+  Warning.emit "@[Prover %s version %s is not known to be supported.@\n\
+                Known versions for this prover:@ %a@\n\
+                Known old versions for this prover:@ %a@]@."
+    prover.Wc.prover_name ver
+    pp_versions data.versions_ok
+    pp_versions data.versions_old;
+  add_id_prover_shortcut env prover_id prover priority;
+  prover_config
+
 let detect_unknown env detected =
   Hstr.fold (fun _ pc acc ->
-    match pc with
-    | None -> acc
-    | Some (priority,prover_id,prover_config) ->
-      let prover = prover_config.prover in
-      Warning.emit "Warning: prover %s version %s is not known to be \
-                     supported.@."
-        prover.Wc.prover_name prover.prover_version;
-      (* Pb: Even if it match the first prover section (normally
-         highest priority) since it is unknown it has the lower
-         priority for its id as shortcut. Perhaps we don't want
-         that. *)
-      add_id_prover_shortcut env prover_id prover priority;
-      add_prover_with_uniq_id prover_config acc)
+    try add_prover_with_uniq_id (detect_unknown env pc) acc
+    with Skip -> acc)
     env.prover_unknown_version detected
 
 let convert_shortcuts env =
@@ -549,13 +550,15 @@ let run_auto_detection config =
   let main = get_main config in
   let l,env = read_auto_detection_data main in
   let detected = List.fold_left (detect_prover env) Mprover.empty l in
-  let length_detected = Mprover.cardinal detected in
+  let length_recognized = Mprover.cardinal detected in
   let detected = detect_unknown env detected in
-  let length_unsupported_version =
-    Mprover.cardinal detected - length_detected in
-  eprintf
-    "%d provers detected and %d provers detected with unsupported version@."
-    length_detected length_unsupported_version;
+  let length_detected = Mprover.cardinal detected in
+  if length_detected > length_recognized then
+    eprintf
+      "%d prover(s) added (including %d prover(s) with an unrecognized version)@."
+      length_detected (length_detected - length_recognized)
+  else
+    eprintf "%d prover(s) added@." length_detected;
   let shortcuts = convert_shortcuts env in
   let config = generate_auto_strategies config in
   let config = set_editors config (read_editors main) in
