@@ -84,7 +84,7 @@ val stepregexp : string -> int -> stepregexp
     [s] is a regular expression, [n] is the group number with steps number. *)
 
 type prover_result_parser = {
-  prp_regexps     : (Str.regexp * prover_answer) list;
+  prp_regexps     : (string * prover_answer) list;
   prp_timeregexps : timeregexp list;
   prp_stepregexps : stepregexp list;
   prp_exitcodes   : (int * prover_answer) list;
@@ -132,6 +132,7 @@ val limit_max : resource_limit -> resource_limit -> resource_limit
 
 val call_editor : command : string -> string -> prover_call
 
+(* internal use only
 val call_on_file :
   command         : string ->
   limit           : resource_limit ->
@@ -139,6 +140,7 @@ val call_on_file :
   printer_mapping : Printer.printer_mapping ->
   ?inplace        : bool ->
   string -> prover_call
+ *)
 
 val call_on_buffer :
   command         : string ->
@@ -146,9 +148,10 @@ val call_on_buffer :
   res_parser      : prover_result_parser ->
   filename        : string ->
   printer_mapping : Printer.printer_mapping ->
+  gen_new_file    : bool ->
   ?inplace        : bool ->
   Buffer.t -> prover_call
-(** Call a prover on the task printed in the {!type: Buffer.t} given.
+(** Build a prover call on the task already printed in the {!type: Buffer.t} given.
 
     @param limit : set the available time limit (def. 0 : unlimited), memory
     limit (def. 0 : unlimited) and step limit (def. -1 : unlimited)
@@ -156,10 +159,21 @@ val call_on_buffer :
     @param res_parser : prover result parser
 
     @param filename : the suffix of the proof task's file, if the prover
-    doesn't accept stdin. *)
+    doesn't accept stdin.
+
+    @param inplace : it is used to make a save of the file on which the
+    prover was called. It is renamed as %f.save if inplace=true and the command
+    [actualcommand] fails
+
+    @param gen_new_file: When set, this generates a new temp file to run the
+    prover on. Otherwise it reuses the filename already given.
+
+*)
 
 type prover_update =
   | NoUpdates
+  | ProverInterrupted
+  | InternalFailure of exn
   | ProverStarted
   | ProverFinished of prover_result
 
