@@ -9,17 +9,24 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Stdlib
+open Wstdlib
 open Smt2_model_defs
-open Strings
 
 exception Not_value
 
+
+(* Printing function *)
+let print_table t =
+  Format.eprintf "Table key and value@.";
+  Mstr.iter (fun key e -> Format.eprintf "%s %a@." key print_def (snd e)) t;
+  Format.eprintf "End table@."
+
+
 (* Adds all referenced cvc4 variables found in the term t to table *)
 let rec get_variables_term (table: correspondence_table) t =
   match t with
   | Variable _ | Function_Local_Variable _ | Boolean _ | Integer _
-  | Decimal _ | Float _ | Other _ | Bitvector _ -> table
+  | Decimal _ | Fraction _ | Float _ | Other _ | Bitvector _ -> table
   | Array a ->
     get_variables_array table a
   | Ite (t1, t2, t3, t4) ->
@@ -34,14 +41,17 @@ let rec get_variables_term (table: correspondence_table) t =
     else
       Mstr.add cvc (false, Noelement) table
   | Record (_, l) ->
-    List.fold_left (fun table t -> get_variables_term table t) table l
-  | Discr (_, l) ->
-    List.fold_left (fun table t -> get_variables_term table t) table l
+    List.fold_left (fun table (_f, t) -> get_variables_term table t) table l
   | To_array t ->
     get_variables_term table t
+  | Apply (_s, lt) ->
+      List.fold_left (fun table t -> get_variables_term table t) table lt
+
 
 and get_variables_array table a =
    match a with
+   | Array_var _v ->
+     table
    | Const t ->
     let table = get_variables_term table t in
     table
@@ -57,78 +67,78 @@ let get_all_var (table: correspondence_table) =
     | _, Function (_, t) -> get_variables_term table t
     | _, Term t -> get_variables_term table t) table table
 
-(* Return true if key is s suffixed with a number *)
-(* We should change the code of this function (Str still forbidden) *)
-let is_prefix_num key s =
-  if (String.length s >= String.length key) || String.length s = 0 then
-    false
-  else
-    try
-      let b = ref (has_prefix s key) in
-      for i = String.length s to String.length key - 1 do
-        b := !b && (String.get key i <= '9') && (String.get key i >= '0')
-      done;
-      !b
-    with
-    | _ -> false
-
-(* Add all variables referenced in the model to the table *)
-let add_all_cvc s table t =
-  Mstr.fold (fun key _element acc ->
-    if is_prefix_num key s then
-      try
-        if snd (Mstr.find key acc) = Noelement then
-          Mstr.add key t acc
-        else acc
-      with Not_found -> acc
-    else
-      acc) table table
-
 exception Bad_variable
 
 (* Get the "radical" of a variable *)
 let remove_end_num s =
   let n = ref (String.length s - 1) in
-  while String.get s !n <= '9' && String.get s !n >= '0' && !n >= 0 do
-    n := !n - 1
-  done;
-  try
-    String.sub s 0 (!n + 1)
-  with
-  | _ -> s
+  if !n <= 0 then s else
+  begin
+    while String.get s !n <= '9' && String.get s !n >= '0' && !n >= 0 do
+      n := !n - 1
+    done;
+    try
+      String.sub s 0 (!n + 1)
+    with
+    | _ -> s
+  end
 
 (* Add the variables that can be deduced from ITE to the table of variables *)
 let add_vars_to_table table value =
-  let rec add_vars_to_table (table: correspondence_table) value =
-    let t = match (snd value) with
-    | Term t -> t
-    | Function (_, t) -> t
-    | Noelement -> raise Bad_variable in
-    match t with
+  let rec add_vars_to_table ~type_value (table: correspondence_table) value =
+    match value with
     | Ite (Cvc4_Variable cvc, Function_Local_Variable _x, t1, t2) ->
         begin
           let table = Mstr.add cvc (false, Term t1) table in
-          add_vars_to_table table (false, Term t2)
+          add_vars_to_table ~type_value table t2
         end
     | Ite (Function_Local_Variable _x, Cvc4_Variable cvc, t1, t2) ->
         begin
           let table = Mstr.add cvc (false, Term t1) table in
-          add_vars_to_table table (false, Term t2)
+          add_vars_to_table ~type_value table t2
         end
     | Ite (t, Function_Local_Variable _x, Cvc4_Variable cvc, t2) ->
         begin
           let table = Mstr.add cvc (false, Term t) table in
-          add_vars_to_table table (false, Term t2)
+          add_vars_to_table ~type_value table t2
         end
     | Ite (Function_Local_Variable _x, t, Cvc4_Variable cvc, t2) ->
         begin
           let table = Mstr.add cvc (false, Term t) table in
-          add_vars_to_table table (false, Term t2)
+          add_vars_to_table ~type_value table t2
         end
     | Ite (_, _, _, _) -> table
-    | _ -> table
+    | _ ->
+      begin
+        match type_value with
+        | None -> table
+        | Some type_value ->
+            Mstr.fold (fun key (_b, elt) acc ->
+              let match_str_z3 = type_value ^ "!" in
+              let match_str_cvc4 = "_" ^ type_value ^ "_" in
+              let match_str = Str.regexp ("\\(" ^ match_str_z3 ^ "\\|" ^ match_str_cvc4 ^ "\\)") in
+              match Str.search_forward match_str (remove_end_num key) 0 with
+              | exception Not_found -> acc
+              | _ ->
+                  if elt == Noelement then
+                    Mstr.add key (false, Term value) acc
+                  else
+                    acc)
+              table table
+      end
   in
-  add_vars_to_table table value
+
+  let type_value, t = match (snd value) with
+  | Term t -> (None, t)
+  | Function (cvc_var_list, t) ->
+    begin
+      match cvc_var_list with
+      | [(_, type_value)] -> (type_value, t)
+      | _ -> (None, t)
+    end
+  | Noelement -> raise Bad_variable in
+
+  add_vars_to_table ~type_value table t
 
 let rec refine_definition table t =
   match t with
@@ -138,6 +148,7 @@ let rec refine_definition table t =
 
 and refine_array table a =
   match a with
+  | Array_var _v -> a
   | Const t ->
     let t = refine_function table t in
     Const t
@@ -153,7 +164,8 @@ and refine_array table a =
    their value. *)
 and refine_function table term =
   match term with
-  | Integer _ | Decimal _ | Float _ | Other _ | Bitvector _ | Boolean _ -> term
+  | Integer _ | Decimal _ | Float _ | Fraction _
+    | Other _ | Bitvector _ | Boolean _ -> term
   | Cvc4_Variable v ->
     begin
       try (
@@ -187,11 +199,11 @@ and refine_function table term =
   | Array a ->
     Array (refine_array table a)
   | Record (n, l) ->
-    Record (n, List.map (fun x -> refine_function table x) l)
-  | Discr (n, l) ->
-    Discr (n, List.map (fun x -> refine_function table x) l)
+    Record (n, List.map (fun (f, v) -> f, refine_function table v) l)
   | To_array t ->
     To_array (refine_function table t)
+  | Apply (s, lt) ->
+    Apply (s, List.map (refine_function table) lt)
 
 
 and refine_variable_value (table: correspondence_table) key v =
@@ -202,21 +214,13 @@ and refine_variable_value (table: correspondence_table) key v =
     let tv = refine_definition table t in
     Mstr.add key (true, tv) table
 
-let convert_float (f: Smt2_model_defs.float_type) : Model_parser.float_type =
-  match f with
-  | Plus_infinity           -> Model_parser.Plus_infinity
-  | Minus_infinity          -> Model_parser.Minus_infinity
-  | Plus_zero               -> Model_parser.Plus_zero
-  | Minus_zero              -> Model_parser.Minus_zero
-  | Not_a_number            -> Model_parser.Not_a_number
-  | Float_value (b, eb, sb) -> Model_parser.Float_value (b, eb, sb)
-
 (* Conversion to value referenced as defined in model_parser.
    We assume that array indices fit into an integer *)
 let convert_to_indice t =
   match t with
   | Integer i -> i
   | Bitvector bv -> bv
+  | Boolean b -> string_of_bool b
   | _ -> raise Not_value
 
 let rec convert_array_value (a: array) : Model_parser.model_array =
@@ -224,6 +228,7 @@ let rec convert_array_value (a: array) : Model_parser.model_array =
 
   let rec create_array_value a =
     match a with
+    | Array_var _v -> raise Not_value
     | Const t -> { Model_parser.arr_indices = !array_indices;
                    Model_parser.arr_others = convert_to_model_value t}
     | Store (a, t1, t2) ->
@@ -238,7 +243,8 @@ and convert_to_model_value (t: term): Model_parser.model_value =
   match t with
   | Integer i -> Model_parser.Integer i
   | Decimal (d1, d2) -> Model_parser.Decimal (d1, d2)
-  | Float f -> Model_parser.Float (convert_float f)
+  | Fraction (s1, s2) -> Model_parser.Fraction (s1, s2)
+  | Float f -> Model_parser.Float f
   | Bitvector bv -> Model_parser.Bitvector bv
   | Boolean b -> Model_parser.Boolean b
   | Other _s -> raise Not_value
@@ -248,7 +254,8 @@ and convert_to_model_value (t: term): Model_parser.model_value =
   | Cvc4_Variable _v -> raise Not_value (*Model_parser.Unparsed "!"*)
   (* TODO change the value returned for non populated Cvc4 variable '!' -> '?' ? *)
   | To_array t -> convert_to_model_value (Array (convert_z3_array t))
-  | Function_Local_Variable _ | Variable _ | Ite _ | Discr _ -> raise Not_value
+  | Apply (s, lt) -> Model_parser.Apply (s, List.map convert_to_model_value lt)
+  | Function_Local_Variable _ | Variable _ | Ite _ -> raise Not_value
 
 and convert_z3_array (t: term) : array =
 
@@ -272,21 +279,7 @@ and convert_z3_array (t: term) : array =
   convert_array t
 
 and convert_record l =
-  let acc = ref [] in
-  let rec convert_aux l =
-    match l with
-    | Discr (_n, l) :: tl ->
-        acc := List.map convert_to_model_value l;
-        convert_aux tl
-    | a :: tl ->
-        convert_to_model_value a :: convert_aux tl
-    | [] -> []
-  in
-  let record_field = convert_aux l in
-  {
-    Model_parser.discrs = !acc;
-    Model_parser.fields = record_field
-  }
+  List.map (fun (f, v) -> f, convert_to_model_value v) l
 
 let convert_to_model_element name t =
   match t with
@@ -295,93 +288,93 @@ let convert_to_model_element name t =
       let value = convert_to_model_value t in
       Model_parser.create_model_element ~name ~value ()
 
-(* Printing function *)
-let print_table t =
-  Format.eprintf "Table key and value@.";
-  Mstr.iter (fun key e -> Format.eprintf "%s %a@." key print_def (snd e)) t;
-  Format.eprintf "End table@."
+let default_apply_to_record (list_records: (string list) Mstr.t) (t: term) =
 
+  let rec array_apply_to_record (a: array) =
+    match a with
+    | Array_var _v -> raise Not_value
+    | Const x ->
+        let x = apply_to_record x in
+        Const x
+    | Store (a, t1, t2) ->
+        let a = array_apply_to_record a in
+        let t1 = apply_to_record t1 in
+        let t2 = apply_to_record t2 in
+        Store (a, t1, t2)
 
-(* Analysis function to get the value of Cvc4 variable contained in the model *)
+  and apply_to_record (v: term) =
+    match v with
+    | Integer _ | Decimal _ | Fraction _ | Float _ | Boolean _ | Bitvector _
+    | Cvc4_Variable _ | Function_Local_Variable _ | Variable _ | Other _ -> v
+    | Array a ->
+        Array (array_apply_to_record a)
+    | Record (s, l) ->
+        let l = List.map (fun (f,v) -> f, apply_to_record v) l in
+        Record (s, l)
+    | Apply (s, l) ->
+        let l = List.map apply_to_record l in
+        if Mstr.mem s list_records then
+          Record (s, List.combine (Mstr.find s list_records) l)
+        else
+          Apply (s, l)
+    | Ite (t1, t2, t3, t4) ->
+        let t1 = apply_to_record t1 in
+        let t2 = apply_to_record t2 in
+        let t3 = apply_to_record t3 in
+        let t4 = apply_to_record t4 in
+        Ite (t1, t2, t3, t4)
+    | To_array t1 ->
+        let t1 = apply_to_record t1 in
+        To_array t1
 
-(* Given a to_rep and its corresponding of_rep in the model gives a guessed
-   value to unknown variables using constant of_rep/to_rep and "else" case of
-   the ITE.*)
-let corres_else_element table to_rep of_rep =
-  let (_key1, (_b1, to_rep)) = to_rep in
-  let (_key2, (_b2, of_rep)) = of_rep in
-  let to_rep = match to_rep with
-  | Term t -> t
-  | Function (_, t) -> t
-  | Noelement -> raise Not_value in
-
-  let of_rep = match of_rep with
-  | Term t -> t
-  | Function (_, t) -> t
-  | Noelement -> raise Not_value in
-
-  let rec corres_else_element table to_rep of_rep =
-    match (to_rep, of_rep) with
-    | Ite (_, _, _, to_rep), _ -> corres_else_element table to_rep of_rep
-    | _, Ite (_, _, _, of_rep) -> corres_else_element table to_rep of_rep
-    | t, Cvc4_Variable cvc ->
-        (* Make all variables not already guessed equal to the else case *)
-        let s = remove_end_num cvc in
-        add_all_cvc s table (false, Term t)
-    | _ -> table
   in
-  (* Case where to_rep, of_rep are constant values *)
+  apply_to_record t
+
+let apply_to_records_ref = ref None
+
+let register_apply_to_records f =
+  apply_to_records_ref := Some f
+
+let apply_to_record list_records t =
+  match !apply_to_records_ref with
+  | None -> default_apply_to_record list_records t
+  | Some f -> f list_records t
+
+let definition_apply_to_record list_records d =
+    match d with
+    | Function (lt, t) ->
+        Function (lt, apply_to_record list_records t)
+    | Term t -> Term (apply_to_record list_records t)
+    | Noelement -> Noelement
+
+let create_list (projections_list: Sstr.t) (list_records: ((string * string) list) Mstr.t)
+    (table: correspondence_table) =
+
+  (* Convert list_records to take replace fields with model_trace when
+     necessary. *)
+  let list_records =
+    Mstr.fold (fun key l acc ->
+      Mstr.add key (List.map (fun (a, b) -> if b = "" then a else b) l) acc) list_records Mstr.empty
+  in
+
+  (* Convert Apply that were actually recorded as record to Record. *)
   let table =
-    match (to_rep, of_rep) with
-    | t, Cvc4_Variable cvc ->
-        Mstr.add cvc (false, Term t) table
-    | _ -> table
+    Mstr.fold (fun key (b, value) acc ->
+      let value = definition_apply_to_record list_records value in
+      Mstr.add key (b, value) acc) table Mstr.empty
   in
-  corres_else_element table to_rep of_rep
-
-let to_rep_of_rep (table: correspondence_table) =
-  let to_reps =
-    List.sort (fun x y -> String.compare (fst x) (fst y))
-      (Mstr.fold (fun key value acc ->
-        if has_prefix "to_rep" key then
-        (key,value) :: acc else acc) table []) in
-
-  let of_reps =
-    List.sort (fun x y -> String.compare (fst x) (fst y))
-      (Mstr.fold (fun key value acc -> if has_prefix "of_rep" key then
-        (key,value) :: acc else acc) table []) in
-
-  let rec to_rep_of_rep table to_reps of_reps =
-    match to_reps, of_reps with
-    | to_rep :: tl1, of_rep :: tl2 ->
-      let table = corres_else_element table to_rep of_rep in
-      to_rep_of_rep table tl1 tl2
-    | [], [] -> table
-    | _ -> table (* Error case *) in
-
-  to_rep_of_rep table to_reps of_reps
-
-
-let create_list (table: correspondence_table) =
 
   (* First populate the table with all references to a cvc variable *)
   let table = get_all_var table in
 
-  (* First recover the values of variables that can be recovered in to/of_rep *)
+  (* First recover values stored in projections that were registered *)
   let table =
     Mstr.fold (fun key value acc ->
-      if has_prefix "of_rep" key && not (String.contains key '!') then
-        add_vars_to_table acc value else acc) table table in
-
-  (* of_rep is done before to_rep because we complete from value with the
-     else branch of the function *)
-  let table =
-    Mstr.fold (fun key value acc ->
-      if has_prefix "to_rep" key && not (String.contains key '!') then
-        add_vars_to_table acc value else acc) table table in
-
-  (* Recover values from the combination of to_rep and of_rep *)
-  let table = to_rep_of_rep table in
+      if Sstr.mem key projections_list then
+        add_vars_to_table acc value
+      else
+        acc)
+      table table in
 
   (* Then substitute all variables with their values *)
   let table =
@@ -397,6 +390,7 @@ let create_list (table: correspondence_table) =
           Some t
       | _ -> None in
       try (convert_to_model_element key t :: list_acc)
-      with Not_value -> list_acc)
+      with Not_value when not (Debug.test_flag Debug.stack_trace) -> list_acc
+      | e -> raise e)
     table
     []
