@@ -9,26 +9,22 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Stdlib
+open Wstdlib
 
 type variable = string
 
-type float_type =
-  | Plus_infinity
-  | Minus_infinity
-  | Plus_zero
-  | Minus_zero
-  | Not_a_number
-  | Float_value of string * string * string
-
 type array =
+  (* Array_var is used by let-bindings only *)
+  | Array_var of variable
   | Const of term
   | Store of array * term * term
 
 and term =
   | Integer of string
   | Decimal of (string * string)
-  | Float of float_type
+  | Fraction of (string * string)
+  | Float of Model_parser.float_type
+  | Apply of (string * term list)
   | Other of string
   | Array of array
   | Bitvector of string
@@ -37,12 +33,11 @@ and term =
   | Function_Local_Variable of variable
   | Variable of variable
   | Ite of term * term * term * term
-  | Record of int * (term list)
-  | Discr of int * (term list)
+  | Record of string * ((string * term) list)
   | To_array of term
 
 type definition =
-  | Function of variable list * term
+  | Function of (variable * string option) list * term
   | Term of term (* corresponding value of a term *)
   | Noelement
 
@@ -56,12 +51,10 @@ val add_element: (string * definition) option ->
   correspondence_table -> bool -> correspondence_table
 
 
-val make_local: variable list -> term -> term
+val make_local: (variable * string option) list -> term -> term
 
 val print_term: Format.formatter -> term -> unit
 val print_def: Format.formatter -> definition -> unit
 
-val build_record_discr: term list -> term
-
 (* Used for let bindings of z3 *)
 val substitute: (string * term) list -> term -> term
