@@ -33,35 +33,35 @@ type t =
     { mutable window_width : int;
       mutable window_height : int;
       mutable tree_width : int;
+      mutable task_height : int;
       mutable font_size : int;
       mutable current_tab : int;
       mutable verbose : int;
-      mutable default_prover : string; (* "" means none *)
-      mutable default_editor : string;
-      mutable intro_premises : bool;
-      mutable show_labels : bool;
+      mutable show_full_context : bool;
+      mutable show_attributes : bool;
+      mutable show_coercions : bool;
       mutable show_locs : bool;
       mutable show_time_limit : bool;
       mutable max_boxes : int;
+      mutable allow_source_editing : bool;
       mutable saving_policy : int;
       (** 0 = always, 1 = never, 2 = ask *)
       mutable premise_color : string;
       mutable neg_premise_color : string;
       mutable goal_color : string;
       mutable error_color : string;
+      mutable error_color_bg : string;
+      mutable error_line_color : string;
       mutable iconset : string;
       (** colors *)
-      mutable env : Env.env;
       mutable config : Whyconf.config;
       original_config : Whyconf.config;
       (* mutable altern_provers : altern_provers; *)
       (* mutable replace_prover : conf_replace_prover; *)
-      (* hidden prover buttons *)
       mutable hidden_provers : string list;
       mutable session_time_limit : int;
       mutable session_mem_limit : int;
       mutable session_nb_processes : int;
-      mutable session_cntexample : bool;
     }
 
 
@@ -69,22 +69,25 @@ type ide = {
   ide_window_width : int;
   ide_window_height : int;
   ide_tree_width : int;
+  ide_task_height : int;
   ide_font_size : int;
   ide_current_tab : int;
   ide_verbose : int;
-  ide_intro_premises : bool;
-  ide_show_labels : bool;
+  ide_show_full_context : bool;
+  ide_show_attributes : bool;
+  ide_show_coercions : bool;
   ide_show_locs : bool;
   ide_show_time_limit : bool;
   ide_max_boxes : int;
+  ide_allow_source_editing : bool;
   ide_saving_policy : int;
   ide_premise_color : string;
   ide_neg_premise_color : string;
   ide_goal_color : string;
   ide_error_color : string;
+  ide_error_color_bg : string;
+  ide_error_line_color : string;
   ide_iconset : string;
-  ide_default_prover : string;
-  ide_default_editor : string;
   (* ide_replace_prover : conf_replace_prover; *)
   ide_hidden_provers : string list;
 }
@@ -93,24 +96,25 @@ let default_ide =
   { ide_window_width = 1024;
     ide_window_height = 768;
     ide_tree_width = 512;
+    ide_task_height = 400;
     ide_font_size = 10;
     ide_current_tab = 0;
     ide_verbose = 0;
-    ide_intro_premises = true;
-    ide_show_labels = false;
+    ide_show_full_context = false;
+    ide_show_attributes = false;
+    ide_show_coercions = true;
     ide_show_locs = false;
     ide_show_time_limit = false;
     ide_max_boxes = 16;
+    ide_allow_source_editing = true;
     ide_saving_policy = 2;
     ide_premise_color = "chartreuse";
     ide_neg_premise_color = "pink";
     ide_goal_color = "gold";
-    ide_error_color = "orange";
+    ide_error_color_bg = "yellow";
+    ide_error_color = "red";
+    ide_error_line_color = "yellow";
     ide_iconset = "fatcow";
-    ide_default_prover = "";
-    ide_default_editor =
-      (try Sys.getenv "EDITOR" ^ " %f"
-       with Not_found -> "editor %f");
     ide_hidden_provers = [];
   }
 
@@ -121,17 +125,21 @@ let load_ide section =
       get_int section ~default:default_ide.ide_window_height "window_height";
     ide_tree_width =
       get_int section ~default:default_ide.ide_tree_width "tree_width";
+    ide_task_height =
+      get_int section ~default:default_ide.ide_task_height "task_height";
     ide_current_tab =
       get_int section ~default:default_ide.ide_current_tab "current_tab";
     ide_font_size =
       get_int section ~default:default_ide.ide_font_size "font_size";
     ide_verbose =
       get_int section ~default:default_ide.ide_verbose "verbose";
-    ide_intro_premises =
-      get_bool section ~default:default_ide.ide_intro_premises
-        "intro_premises";
-    ide_show_labels =
-      get_bool section ~default:default_ide.ide_show_labels "print_labels";
+    ide_show_full_context =
+      get_bool section ~default:default_ide.ide_show_full_context
+        "show_full_context";
+    ide_show_attributes =
+      get_bool section ~default:default_ide.ide_show_attributes "print_attributes";
+    ide_show_coercions =
+      get_bool section ~default:default_ide.ide_show_attributes "print_coercions";
     ide_show_locs =
       get_bool section ~default:default_ide.ide_show_locs "print_locs";
     ide_show_time_limit =
@@ -140,6 +148,8 @@ let load_ide section =
     ide_max_boxes =
       get_int section ~default:default_ide.ide_max_boxes
         "max_boxes";
+    ide_allow_source_editing =
+      get_bool section ~default:default_ide.ide_allow_source_editing "allow_source_editing";
     ide_saving_policy =
       get_int section ~default:default_ide.ide_saving_policy "saving_policy";
     ide_premise_color =
@@ -154,21 +164,26 @@ let load_ide section =
     ide_error_color =
       get_string section ~default:default_ide.ide_error_color
         "error_color";
+    ide_error_color_bg =
+      get_string section ~default:default_ide.ide_error_color_bg
+        "error_color_bg";
+    ide_error_line_color =
+      get_string section ~default:default_ide.ide_error_line_color
+        "error_line_color";
     ide_iconset =
       get_string section ~default:default_ide.ide_iconset
         "iconset";
-    ide_default_editor =
-      get_string section ~default:default_ide.ide_default_editor
-        "default_editor";
-    ide_default_prover =
-      get_string section ~default:default_ide.ide_default_prover
-        "default_prover";
     ide_hidden_provers = get_stringl ~default:default_ide.ide_hidden_provers section "hidden_prover";
   }
 
 
-let set_labels_flag =
-  let fl = Debug.lookup_flag "print_labels" in
+let set_attr_flag =
+  let fl = Debug.lookup_flag "print_attributes" in
+  fun b ->
+    (if b then Debug.set_flag else Debug.unset_flag) fl
+
+let set_coercions_flag =
+  let fl = Debug.lookup_flag "print_coercions" in
   fun b ->
     (if b then Debug.set_flag else Debug.unset_flag) fl
 
@@ -177,45 +192,47 @@ let set_locs_flag =
   fun b ->
     (if b then Debug.set_flag else Debug.unset_flag) fl
 
-let load_config config original_config env =
+let load_config config original_config =
   let main = get_main config in
   let ide  = match Whyconf.get_section config "ide" with
     | None -> default_ide
     | Some s -> load_ide s
   in
-  set_labels_flag ide.ide_show_labels;
+  set_attr_flag ide.ide_show_attributes;
+  set_coercions_flag ide.ide_show_coercions;
   set_locs_flag ide.ide_show_locs;
   { window_height = ide.ide_window_height;
     window_width  = ide.ide_window_width;
     tree_width    = ide.ide_tree_width;
+    task_height   = ide.ide_task_height;
     current_tab   = ide.ide_current_tab;
     font_size     = ide.ide_font_size;
     verbose       = ide.ide_verbose;
-    intro_premises= ide.ide_intro_premises ;
-    show_labels   = ide.ide_show_labels ;
+    show_full_context= ide.ide_show_full_context ;
+    show_attributes   = ide.ide_show_attributes ;
+    show_coercions = ide.ide_show_coercions ;
     show_locs     = ide.ide_show_locs ;
     show_time_limit = ide.ide_show_time_limit;
     max_boxes = ide.ide_max_boxes;
+    allow_source_editing = ide.ide_allow_source_editing ;
     saving_policy = ide.ide_saving_policy ;
     premise_color = ide.ide_premise_color;
     neg_premise_color = ide.ide_neg_premise_color;
     goal_color = ide.ide_goal_color;
     error_color = ide.ide_error_color;
+    error_color_bg = ide.ide_error_color_bg;
+    error_line_color = ide.ide_error_line_color;
     iconset = ide.ide_iconset;
-    default_prover = ide.ide_default_prover;
-    default_editor = ide.ide_default_editor;
     config         = config;
     original_config = original_config;
-    env            = env;
     hidden_provers = ide.ide_hidden_provers;
     session_time_limit = Whyconf.timelimit main;
     session_mem_limit = Whyconf.memlimit main;
     session_nb_processes = Whyconf.running_provers_max main;
-    session_cntexample = Whyconf.cntexample main;
 }
 
 let save_config t =
-  Debug.dprintf debug "[GUI config] saving IDE config file@.";
+  Debug.dprintf debug "[config] saving IDE config file@.";
   (* taking original config, without the extra_config *)
   let config = t.original_config in
   (* copy possibly modified settings to original config *)
@@ -224,9 +241,6 @@ let save_config t =
   let mem = Whyconf.memlimit new_main in
   let nb = Whyconf.running_provers_max new_main in
   let config = set_main config (set_limits (get_main config) time mem nb) in
-  let new_main = Whyconf.get_main t.config in
-  let cntexample = Whyconf.cntexample new_main in
-  let config = set_main config (set_cntexample (get_main config) cntexample) in
   (* copy also provers section since it may have changed (the editor
      can be set via the preferences dialog) *)
   let config = set_provers config (get_provers t.config) in
@@ -236,22 +250,25 @@ let save_config t =
   let ide = set_int ide "window_height" t.window_height in
   let ide = set_int ide "window_width" t.window_width in
   let ide = set_int ide "tree_width" t.tree_width in
+  let ide = set_int ide "task_height" t.task_height in
   let ide = set_int ide "current_tab" t.current_tab in
   let ide = set_int ide "font_size" t.font_size in
   let ide = set_int ide "verbose" t.verbose in
-  let ide = set_bool ide "intro_premises" t.intro_premises in
-  let ide = set_bool ide "print_labels" t.show_labels in
+  let ide = set_bool ide "show_full_context" t.show_full_context in
+  let ide = set_bool ide "print_attributes" t.show_attributes in
+  let ide = set_bool ide "print_coercions" t.show_coercions in
   let ide = set_bool ide "print_locs" t.show_locs in
   let ide = set_bool ide "print_time_limit" t.show_time_limit in
   let ide = set_int ide "max_boxes" t.max_boxes in
+  let ide = set_bool ide "allow_source_editing" t.allow_source_editing in
   let ide = set_int ide "saving_policy" t.saving_policy in
   let ide = set_string ide "premise_color" t.premise_color in
   let ide = set_string ide "neg_premise_color" t.neg_premise_color in
   let ide = set_string ide "goal_color" t.goal_color in
   let ide = set_string ide "error_color" t.error_color in
+  let ide = set_string ide "error_color_bg" t.error_color_bg in
+  let ide = set_string ide "error_line_color" t.error_line_color in
   let ide = set_string ide "iconset" t.iconset in
-  let ide = set_string ide "default_prover" t.default_prover in
-  let ide = set_string ide "default_editor" t.default_editor in
   let ide = set_stringl ide "hidden_prover" t.hidden_provers in
   let config = Whyconf.set_section config "ide" ide in
   Whyconf.save_config config
@@ -262,20 +279,58 @@ let config,load_config =
     match !config with
       | None -> invalid_arg "configuration not yet loaded"
       | Some conf -> conf),
-  (fun conf base_conf env ->
-    let c = load_config conf base_conf env in
+  (fun conf base_conf ->
+    let c = load_config conf base_conf in
     config := Some c)
 
 let save_config () = save_config (config ())
 
 let get_main () = (get_main (config ()).config)
 
+(*
+
+
+  font size
+
+
+ *)
+
+
+let sans_font_family = "Sans"
+let mono_font_family = "Monospace"
+
+let modifiable_sans_font_views = ref []
+let modifiable_mono_font_views = ref []
+
+let add_modifiable_sans_font_view v =
+  modifiable_sans_font_views := v :: !modifiable_sans_font_views
+
+let add_modifiable_mono_font_view v =
+  modifiable_mono_font_views := v :: !modifiable_mono_font_views
+
+let change_font size =
+(*
+  Tools.resize_images (!Colors.font_size * 2 - 4);
+*)
+  let sff = sans_font_family ^ " " ^ string_of_int size in
+  let mff = mono_font_family ^ " " ^ string_of_int size in
+  let sf = Pango.Font.from_string sff in
+  let mf = Pango.Font.from_string mff in
+  List.iter (fun v -> v#modify_font sf) !modifiable_sans_font_views;
+  List.iter (fun v -> v#modify_font mf) !modifiable_mono_font_views
+
 let incr_font_size n =
   let c = config () in
   let s = max (c.font_size + n) 4 in
   c.font_size <- s;
   s
 
+let enlarge_fonts () = change_font (incr_font_size 1)
+
+let reduce_fonts () = change_font (incr_font_size (-1))
+
+let set_fonts () = change_font (incr_font_size 0)
+
 (*
 
   images, icons
@@ -324,11 +379,13 @@ let image ?size f =
     Filename.concat (datadir main)
       (Filename.concat "images" (f^".png"))
   in
+  try (
   match size with
     | None ->
         GdkPixbuf.from_file n
     | Some s ->
         GdkPixbuf.from_file_at_size ~width:s ~height:s n
+  ) with _ -> !image_default
 
 let iconname_default = ref ""
 let iconname_undone = ref ""
@@ -463,7 +520,7 @@ let resize_images size =
   ()
 
 let init () =
-  Debug.dprintf debug "[GUI config] reading icons...@?";
+  Debug.dprintf debug "[config] reading icons...@?";
   load_icon_names ();
   why_icon := image "logo-why";
   resize_images 20;
@@ -520,6 +577,12 @@ let show_legend_window () =
   i "   Valid but obsolete result\n";
   ib image_unknown_obs;
   i "   Answer not conclusive and obsolete\n";
+  ib image_timeout_obs;
+  i "   Time limit reached, obsolete\n";
+  ib image_outofmemory_obs;
+  i "   Out of memory, obsolete\n";
+  ib image_steplimitexceeded_obs;
+  i "   Step limit exceeded, obsolete\n";
   ib image_invalid_obs;
   i "   Prover disproved goal, but obsolete\n";
   ib image_failure_obs;
@@ -548,9 +611,8 @@ let show_about_window () =
                 "Martin Clochard";
                 "Simon Cruanes";
                 "Sylvain Dailler";
-                "Jacques-Pascal Deplaix";
                 "Clément Fumex";
-                "Leon Gondelman";
+                "Léon Gondelman";
                 "David Hauzar";
                 "Daisuke Ishii";
                 "Johannes Kanig";
@@ -559,6 +621,8 @@ let show_about_window () =
                 "Benjamin Monate";
                 "Kim Nguyễn";
                 "Thi-Minh-Tuyen Nguyen";
+                "Mário Pereira";
+                "Raphaël Rieu-Helft";
                 "Simão Melo de Sousa";
                 "Asma Tafat";
                 "Piotr Trojanek";
@@ -645,6 +709,7 @@ let general_settings (c : t) (notebook:GPack.notebook) =
       (fun () -> c.session_nb_processes <- nb_processes_spin#value_as_int)
   in
   (* counter-example *)
+(*
   let cntexample_check = GButton.check_button ~label:"get counter-example"
     ~packing:vb#add ()
     ~active:c.session_cntexample
@@ -653,6 +718,16 @@ let general_settings (c : t) (notebook:GPack.notebook) =
     cntexample_check#connect#toggled ~callback:
       (fun () -> c.session_cntexample <- not c.session_cntexample)
   in
+*)
+  (* source editing allowed *)
+  let source_editing_check = GButton.check_button ~label:"allow editing source files"
+    ~packing:vb#add ()
+    ~active:c.allow_source_editing
+  in
+  let (_: GtkSignal.id) =
+    source_editing_check#connect#toggled ~callback:
+      (fun () -> c.allow_source_editing <- not c.allow_source_editing)
+  in
   (* session saving policy *)
   let set_saving_policy n () = c.saving_policy <- n in
   let saving_policy_frame =
@@ -731,26 +806,38 @@ let appearance_settings (c : t) (notebook:GPack.notebook) =
     GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
       ~packing:vb#add ()
   in
-  let intropremises =
-    GButton.check_button ~label:"introduce premises"
+  let showfullcontext =
+    GButton.check_button ~label:"show full task context"
       ~packing:display_options_box#add ()
-      ~active:c.intro_premises
+      ~active:c.show_full_context
   in
   let (_ : GtkSignal.id) =
-    intropremises#connect#toggled ~callback:
-      (fun () -> c.intro_premises <- not c.intro_premises)
+    showfullcontext#connect#toggled ~callback:
+      (fun () -> c.show_full_context <- not c.show_full_context)
   in
-  let showlabels =
+  let showattrs =
     GButton.check_button
-      ~label:"show labels in formulas"
+      ~label:"show attributes in formulas"
       ~packing:display_options_box#add ()
-      ~active:c.show_labels
+      ~active:c.show_attributes
   in
   let (_ : GtkSignal.id) =
-    showlabels#connect#toggled ~callback:
+    showattrs#connect#toggled ~callback:
       (fun () ->
-         c.show_labels <- not c.show_labels;
-         set_labels_flag c.show_labels)
+         c.show_attributes <- not c.show_attributes;
+         set_attr_flag c.show_attributes)
+  in
+  let showcoercions =
+    GButton.check_button
+      ~label:"show coercions in formulas"
+      ~packing:display_options_box#add ()
+      ~active:c.show_coercions
+  in
+  let (_ : GtkSignal.id) =
+    showcoercions#connect#toggled ~callback:
+      (fun () ->
+         c.show_coercions <- not c.show_coercions;
+         set_coercions_flag c.show_coercions)
   in
   let showlocs =
     GButton.check_button
@@ -864,7 +951,7 @@ let provers_page c (notebook:GPack.notebook) =
   let hbox_pack = hbox#pack ~fill:true ~expand:true ?from:None ?padding:None in
   (* show/hide provers *)
   let frame =
-    GBin.frame ~label:"Prover button in the left toolbar" ~packing:hbox_pack ()
+    GBin.frame ~label:"Provers visible in the context menu" ~packing:hbox_pack ()
   in
   let provers_box =
     GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
@@ -889,6 +976,7 @@ let provers_page c (notebook:GPack.notebook) =
       in ())
     (Whyconf.get_provers c.config);
   (* default prover *)
+(*
   let frame2 =
     GBin.frame ~label:"Default prover" ~packing:hbox_pack () in
   let provers_box =
@@ -897,9 +985,9 @@ let provers_page c (notebook:GPack.notebook) =
   let group =
     let b =
       GButton.radio_button ~label:"(none)" ~packing:provers_box#add
-                           ~active:(c.default_prover = "") () in
+                           ~active:(c.config.default_prover = "") () in
     let (_ : GtkSignal.id) =
-      b#connect#toggled ~callback:(fun () -> c.default_prover <- "") in
+      b#connect#toggled ~callback:(fun () -> c.config.default_prover <- "") in
     b#group in
   Mprover.iter
     (fun _ p ->
@@ -907,18 +995,18 @@ let provers_page c (notebook:GPack.notebook) =
       let label = Pp.string_of_wnl print_prover p.prover in
       let b =
         GButton.radio_button ~label ~group ~packing:provers_box#add
-                             ~active:(name = c.default_prover) () in
+                             ~active:(name = c.config.default_prover) () in
       let (_ : GtkSignal.id) =
-        b#connect#toggled ~callback:(fun () -> c.default_prover <- name)
+        b#connect#toggled ~callback:(fun () -> c.config.default_prover <- name)
       in ())
     (Whyconf.get_provers c.config)
-
-
+ *)
+  ()
 
 (* Page "Uninstalled provers" *)
 
 let alternatives_frame c (notebook:GPack.notebook) =
-  let label = GMisc.label ~text:"Uninstalled provers" () in
+  let label = GMisc.label ~text:"Uninstalled provers policies" () in
   let page =
     GPack.vbox ~homogeneous:false ~packing:
       (fun w -> ignore(notebook#append_page ~tab_label:label#coerce w)) ()
@@ -933,7 +1021,7 @@ let alternatives_frame c (notebook:GPack.notebook) =
   in
   let remove button p () =
     button#destroy ();
-    c.config <- set_policies c.config (Mprover.remove p (get_policies c.config))
+    c.config <- set_policies c.config (Mprover.remove p (get_policies c.config));
   in
   let iter p policy =
     let label =
@@ -972,12 +1060,16 @@ let editors_page c (notebook:GPack.notebook) =
   let default_editor_frame =
     GBin.frame ~label:"Default editor" ~packing:vbox_pack ()
   in
+  let main = Whyconf.get_main c.config in
   let editor_entry =
-   GEdit.entry ~text:c.default_editor ~packing:default_editor_frame#add ()
+   GEdit.entry ~text:(default_editor main) ~packing:default_editor_frame#add ()
   in
   let (_ : GtkSignal.id) =
-    editor_entry#connect#changed ~callback:
-      (fun () -> c.default_editor <- editor_entry#text)
+    editor_entry#connect#changed
+      ~callback:
+      (fun () ->
+       c.config <- Whyconf.set_main c.config
+	(Whyconf.set_default_editor main editor_entry#text))
   in
   let frame = GBin.frame ~label:"Specific editors" ~packing:vbox_pack () in
   let box = GPack.vbox ~border_width:5 ~packing:frame#add () in
@@ -1021,7 +1113,7 @@ let editors_page c (notebook:GPack.notebook) =
                   try Meditor.find s map
                   with Not_found -> assert false
             in
-	    (* Debug.dprintf debug "prover %a : selected editor '%s'@." *)
+	    (* Debug.dprintf debug "prover %a: selected editor '%s'@." *)
             (*   print_prover p data; *)
             let provers = Whyconf.get_provers c.config in
             c.config <-
@@ -1061,7 +1153,7 @@ let preferences (c : t) =
   in
   let (_ : GtkSignal.id) =
     color_sel#connect ColorSelection.S.color_changed ~callback:
-      (fun c -> Format.eprintf "Gconfig.color_sel : %s@."
+      (fun c -> Format.eprintf "Gconfig.color_sel: %s@."
          c)
   in
 *)
@@ -1075,9 +1167,6 @@ let preferences (c : t) =
       c.config <- Whyconf.set_main c.config
         (Whyconf.set_limits (Whyconf.get_main c.config)
            c.session_time_limit c.session_mem_limit c.session_nb_processes);
-      c.config <- Whyconf.set_main c.config
-	(Whyconf.set_cntexample (Whyconf.get_main c.config)
-	   c.session_cntexample);
       save_config ()
     | `CLOSE | `DELETE_EVENT -> ()
   end;
@@ -1095,120 +1184,132 @@ let run_auto_detection gconfig =
   ()
 *)
 
-(*let () = Debug.dprintf debug "[GUI config] end of configuration initialization@."*)
+(*let () = Debug.dprintf debug "[config] end of configuration initialization@."*)
 
-let uninstalled_prover c eS unknown =
-  try
-    Whyconf.get_prover_upgrade_policy c.config unknown
-  with Not_found ->
-    let others,names,versions = Session_tools.unknown_to_known_provers
-      (Whyconf.get_provers eS.Session.whyconf) unknown in
-    let dialog = GWindow.dialog
-      ~icon:(!why_icon) ~modal:true
-      ~title:"Why3: Uninstalled prover" ()
+let uninstalled_prover_dialog ~height ~callback c unknown =
+  let others,names,versions =
+    Whyconf.unknown_to_known_provers
+      (Whyconf.get_provers c.config) unknown
+  in
+  let dialog = GWindow.dialog
+                 ~icon:(!why_icon) ~modal:true
+                 ~title:"Why3: Uninstalled prover" ()
+  in
+  let vbox = dialog#vbox in
+  let vbox_pack = vbox#pack ~fill:true ~expand:true ?from:None ?padding:None in
+  let hbox = GPack.hbox ~packing:vbox_pack () in
+  let hbox_pack = hbox#pack ~fill:true ~expand:true ?from:None ?padding:None in
+  let scrollview =
+    GBin.scrolled_window ~hpolicy:`NEVER ~vpolicy:`AUTOMATIC ~height
+      ~packing:hbox_pack ()
+  in
+  let () = scrollview#set_shadow_type `OUT in
+  let vbox = GPack.vbox ~packing:scrollview#add_with_viewport () in
+  (* header *)
+  let hb = GPack.hbox ~packing:vbox#add () in
+  let _ = GMisc.image ~stock:`DIALOG_WARNING ~packing:hb#add () in
+  let (_:GMisc.label) =
+    let text =
+      Pp.sprintf "The prover %a is not installed"
+        Whyconf.print_prover unknown
     in
-    let vbox = dialog#vbox in
-(* Does not work: why ??
-    let vbox_pack = vbox#pack ~fill:true ~expand:true ?from:None ?padding:None in
-    let hbox = GPack.hbox ~packing:vbox_pack () in
-    let hbox_pack = hbox#pack ~fill:true ~expand:true ?from:None ?padding:None in
-    let scrollview =
-      GBin.scrolled_window ~hpolicy:`NEVER ~vpolicy:`AUTOMATIC
-        ~packing:hbox_pack ()
+    GMisc.label ~ypad:20 ~text ~xalign:0.5 ~packing:hb#add ()
+  in
+  let (_:GMisc.label) =
+    let text =
+      "WARNING: this policy will not be taken into account immediately \
+        but only if you replay again the proofs."
     in
-    let () = scrollview#set_shadow_type `OUT in
-    let vbox = GPack.vbox ~packing:scrollview#add_with_viewport () in
-*)
-    (* header *)
-    let hb = GPack.hbox ~packing:vbox#add () in
-    let _ = GMisc.image ~stock:`DIALOG_WARNING ~packing:hb#add () in
-    let _ =
-      let text =
-        Pp.sprintf "The prover %a is not installed"
-          Whyconf.print_prover unknown
-      in
-      GMisc.label ~ypad:20 ~text ~xalign:0.5 ~packing:hb#add ()
+    GMisc.label ~text ~line_wrap:true ~packing:vbox#add ()
+  in
+  let (_:GMisc.label) =
+    let text =
+      "WARNING: do not forget to save preferences to keep this policy in future sessions"
     in
-    (* choices *)
-    let vbox_pack =
-      vbox#pack ~fill:true ~expand:true ?from:None ?padding:None
-    in
-    let label = "Please select a policy for associated proof attempts" in
-    let policy_frame = GBin.frame ~label ~packing:vbox_pack () in
-    let choice = ref 1 in
-    let prover_choosed = ref None in
-    let set_prover prover () = prover_choosed := Some prover in
-    let box =
-      GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
-        ~packing:policy_frame#add ()
-    in
-    let choice1 = GButton.radio_button
+    GMisc.label ~text ~line_wrap:true ~packing:vbox#add ()
+  in
+  (* choices *)
+  let vbox_pack =
+    vbox#pack ~fill:true ~expand:true ?from:None ?padding:None
+  in
+  let label = "Please select a policy for associated proof attempts" in
+  let policy_frame = GBin.frame ~label ~packing:vbox_pack () in
+  let choice = ref 1 in
+  let prover_choosed = ref None in
+  let set_prover prover () = prover_choosed := Some prover in
+  let box =
+    GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
+      ~packing:policy_frame#add ()
+  in
+  let choice1 = GButton.radio_button
       ~label:"move proofs to the selected prover below"
       ~active:true
       ~packing:box#add () in
-    let choice2 = GButton.radio_button
+  let choice2 = GButton.radio_button
       ~label:"duplicate proofs to the selected prover below"
       ~active:false ~group:choice1#group
       ~packing:box#add () in
-    let choice0 = GButton.radio_button
+  let choice0 = GButton.radio_button
       ~label:"keep proofs as they are, do not try to play them"
       ~active:false  ~group:choice1#group
       ~packing:box#add () in
-    let first = ref None in
-    let alternatives_section acc label alternatives =
-      if alternatives <> [] then
-        let frame = GBin.frame ~label ~packing:vbox#add () in
-        let box =
-          GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
-            ~packing:frame#add ()
+  let first = ref None in
+  let alternatives_section acc label alternatives =
+    if alternatives <> [] then
+      let frame = GBin.frame ~label ~packing:vbox#add () in
+      let box =
+        GPack.button_box `VERTICAL ~border_width:5 ~spacing:5
+          ~packing:frame#add ()
+      in
+      let iter_alter prover =
+        let choice_button =
+          let label = Pp.string_of_wnl print_prover prover in
+          match !first with
+          | None ->
+              let choice_button =
+                GButton.radio_button ~label ~active:true ~packing:box#add ()
+              in
+              prover_choosed := Some prover;
+              first := Some choice_button;
+              choice_button
+          | Some first ->
+              GButton.radio_button ~label ~group:first#group
+                ~active:false ~packing:box#add ()
         in
-        let iter_alter prover =
-          let choice_button =
-            let label = Pp.string_of_wnl print_prover prover in
-            match !first with
-              | None ->
-                let choice_button =
-                  GButton.radio_button ~label ~active:true ~packing:box#add ()
-                in
-                prover_choosed := Some prover;
-                first := Some choice_button;
-                choice_button
-              | Some first ->
-                GButton.radio_button ~label ~group:first#group
-                  ~active:false ~packing:box#add ()
-          in
-          ignore (choice_button#connect#toggled ~callback:(set_prover prover))
-        in
-        List.iter iter_alter alternatives;
-        frame#misc :: (* box#misc :: *) acc
-      else acc
-    in
-    let boxes = alternatives_section [] "Same name and same version" versions in
-    let boxes = alternatives_section boxes "Same name and different version" names in
-    let boxes = alternatives_section boxes "Different name" others in
-    let hide_provers () = List.iter (fun b -> b#set_sensitive false) boxes in
-    let show_provers () = List.iter (fun b -> b#set_sensitive true) boxes in
-    ignore (choice0#connect#toggled
-              ~callback:(fun () -> choice := 0; hide_provers ()));
-    ignore (choice1#connect#toggled
-              ~callback:(fun () -> choice := 1; show_provers ()));
-    ignore (choice2#connect#toggled
-              ~callback:(fun () -> choice := 2; show_provers ()));
-    dialog#add_button "Ok" `CLOSE ;
-    ignore (dialog#run ());
-    dialog#destroy ();
-    let policy =
-      match !choice, !prover_choosed with
-        | 0,_ -> CPU_keep
-        | 1, Some p -> CPU_upgrade p
-        | 2, Some p -> CPU_duplicate p
-        | _ -> assert false
-    in
-    c.config <- set_prover_upgrade_policy c.config unknown policy;
-    policy
+        ignore (choice_button#connect#toggled ~callback:(set_prover prover))
+      in
+      List.iter iter_alter alternatives;
+      frame#misc :: (* box#misc :: *) acc
+    else acc
+  in
+  let boxes = alternatives_section [] "Same name and same version" versions in
+  let boxes = alternatives_section boxes "Same name and different version" names in
+  let boxes = alternatives_section boxes "Different name" others in
+  let hide_provers () = List.iter (fun b -> b#set_sensitive false) boxes in
+  let show_provers () = List.iter (fun b -> b#set_sensitive true) boxes in
+  ignore (choice0#connect#toggled
+            ~callback:(fun () -> choice := 0; hide_provers ()));
+  ignore (choice1#connect#toggled
+            ~callback:(fun () -> choice := 1; show_provers ()));
+  ignore (choice2#connect#toggled
+            ~callback:(fun () -> choice := 2; show_provers ()));
+  dialog#add_button "Ok" `CLOSE ;
+  ignore (dialog#run ());
+  dialog#destroy ();
+  let policy =
+    match !choice, !prover_choosed with
+    | 0,_ -> CPU_keep
+    | 1, Some p -> CPU_upgrade p
+    | 2, Some p -> CPU_duplicate p
+    | _ -> assert false
+  in
+  c.config <- set_prover_upgrade_policy c.config unknown policy;
+  let () = callback unknown policy in
+  ()
+
 
 (*
 Local Variables:
-compile-command: "unset LANG; make -C ../.. bin/why3ide.byte"
+compile-command: "unset LANG; make -C ../.. bin/why3ide.opt"
 End:
 *)
