@@ -9,12 +9,12 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Stdlib
+open Wstdlib
 open Ident
-open Ty
 open Term
 open Ity
 open Expr
+open Pmodule
 
 (** Program types *)
 
@@ -24,12 +24,13 @@ val dity_fresh : unit -> dity
 
 val dity_of_ity : ity -> dity
 
+val dity_int  : dity
+val dity_real : dity
+val dity_bool : dity
+val dity_unit : dity
+
 type dvty = dity list * dity (* A -> B -> C == ([A;B],C) *)
 
-val dity_is_bool : dity -> bool
-
-val dvty_is_chainable : dvty -> bool
-
 (** Patterns *)
 
 type dpattern = private {
@@ -41,11 +42,11 @@ type dpattern = private {
 
 type dpattern_node =
   | DPwild
-  | DPvar  of preid
+  | DPvar  of preid * bool
   | DPapp  of rsymbol * dpattern list
+  | DPas   of dpattern * preid * bool
   | DPor   of dpattern * dpattern
-  | DPas   of dpattern * preid
-  | DPcast of dpattern * ity
+  | DPcast of dpattern * dity
 
 (** Binders *)
 
@@ -57,26 +58,28 @@ type dbinder = preid option * ghost * dity
 
 exception UnboundLabel of string
 
-type register_old = pvsymbol -> string -> pvsymbol
-  (** Program variables occurring under [old] or [at] are passed to
-      a registrar function. The label string must be ['0] for [old]. *)
+val old_label : string
 
-type 'a later = pvsymbol Mstr.t -> register_old -> 'a
+type register_old = string -> pvsymbol -> pvsymbol
+  (** Program variables occurring under [old] or [at] are passed to
+      a registrar function. The label string must be ["'Old"] for [old]. *)
+
+type 'a later = pvsymbol Mstr.t -> xsymbol Mstr.t -> register_old -> 'a
   (** Specification terms are parsed and typechecked after the program
       expressions, when the types of locally bound program variables are
       already established. *)
 
 type dspec_final = {
   ds_pre     : term list;
-  ds_post    : (vsymbol option * term) list;
-  ds_xpost   : (vsymbol option * term) list Mexn.t;
-  ds_reads   : vsymbol list;
+  ds_post    : (pvsymbol * term) list;
+  ds_xpost   : (pvsymbol * term) list Mxs.t;
+  ds_reads   : pvsymbol list;
   ds_writes  : term list;
   ds_diverge : bool;
   ds_checkrw : bool;
 }
 
-type dspec = ty -> dspec_final
+type dspec = ity -> dspec_final
   (* Computation specification is also parametrized by the result type.
      All vsymbols in the postcondition clauses in the [ds_post] field
      must have this type. All vsymbols in the exceptional postcondition
@@ -86,6 +89,10 @@ type dspec = ty -> dspec_final
 
 type dinvariant = term list
 
+type dxsymbol =
+  | DElexn of string * dity
+  | DEgexn of xsymbol
+
 type dexpr = private {
   de_node : dexpr_node;
   de_dvty : dvty;
@@ -94,41 +101,48 @@ type dexpr = private {
 
 and dexpr_node =
   | DEvar of string * dvty
-  | DEpv of pvsymbol
-  | DErs of rsymbol
-  | DEconst of Number.constant
+  | DEsym of prog_symbol
+  | DEconst of Number.constant * dity
   | DEapp of dexpr * dexpr
-  | DEfun of dbinder list * dspec later * dexpr
-  | DEany of dbinder list * dspec later * dity
+  | DEfun of dbinder list * dity * mask * dspec later * dexpr
+  | DEany of dbinder list * dity * mask * dspec later
   | DElet of dlet_defn * dexpr
   | DErec of drec_defn * dexpr
   | DEnot of dexpr
   | DEand of dexpr * dexpr
   | DEor of dexpr * dexpr
   | DEif of dexpr * dexpr * dexpr
-  | DEcase of dexpr * (dpattern * dexpr) list
+  | DEmatch of dexpr * dreg_branch list * dexn_branch list
   | DEassign of (dexpr * rsymbol * dexpr) list
   | DEwhile of dexpr * dinvariant later * variant list later * dexpr
   | DEfor of preid * dexpr * for_direction * dexpr * dinvariant later * dexpr
-  | DEtry of dexpr * (xsymbol * dpattern * dexpr) list
-  | DEraise of xsymbol * dexpr
+  | DEraise of dxsymbol * dexpr
   | DEghost of dexpr
+  | DEexn of preid * dity * mask * dexpr
+  | DEoptexn of preid * dity * mask * dexpr
   | DEassert of assertion_kind * term later
-  | DEpure of term later
+  | DEpure of term later * dity
+  | DEvar_pure of string * dvty
+  | DEls_pure of lsymbol * bool
+  | DEpv_pure of pvsymbol
   | DEabsurd
   | DEtrue
   | DEfalse
-  | DEmark of preid * dexpr
-  | DEcast of dexpr * ity
+  | DElabel of preid * dexpr
+  | DEcast of dexpr * dity
   | DEuloc of dexpr * Loc.position
-  | DElabel of dexpr * Slab.t
+  | DEattr of dexpr * Sattr.t
+
+and dreg_branch = dpattern * dexpr
+
+and dexn_branch = dxsymbol * dpattern * dexpr
 
 and dlet_defn = preid * ghost * rs_kind * dexpr
 
 and drec_defn = private { fds : dfun_defn list }
 
-and dfun_defn = preid * ghost * rs_kind *
-  dbinder list * dspec later * variant list later * dexpr
+and dfun_defn = preid * ghost * rs_kind * dbinder list *
+  dity * mask * dspec later * variant list later * dexpr
 
 (** Environment *)
 
@@ -142,12 +156,27 @@ val denv_add_let : denv -> dlet_defn -> denv
 
 val denv_add_args : denv -> dbinder list -> denv
 
-val denv_add_pat : denv -> dpattern -> denv
+val denv_add_pat : denv -> dpattern -> dity -> denv
+val denv_add_expr_pat : denv -> dpattern -> dexpr -> denv
+val denv_add_exn_pat : denv -> dpattern -> dxsymbol -> denv
+
+val denv_add_for_index : denv -> preid -> dvty -> denv
+
+val denv_add_exn : denv -> preid -> dity -> denv
 
 val denv_get : denv -> string -> dexpr_node (** raises UnboundVar *)
-
 val denv_get_opt : denv -> string -> dexpr_node option
 
+val denv_get_pure : denv -> string -> dexpr_node (** raises UnboundVar *)
+val denv_get_pure_opt : denv -> string -> dexpr_node option
+
+val denv_get_exn : denv -> string -> dxsymbol (** raises Not_found *)
+val denv_get_exn_opt : denv -> string -> dxsymbol option
+
+val denv_names : denv -> Sstr.t
+
+val denv_pure : denv -> (Dterm.denv -> Dterm.dty) -> dity
+
 (** Constructors *)
 
 val dpattern : ?loc:Loc.position -> dpattern_node -> dpattern
@@ -155,7 +184,7 @@ val dpattern : ?loc:Loc.position -> dpattern_node -> dpattern
 val dexpr : ?loc:Loc.position -> dexpr_node -> dexpr
 
 type pre_fun_defn = preid * ghost * rs_kind * dbinder list *
-  dity * (denv -> dspec later * variant list later * dexpr)
+  dity * mask * (denv -> dspec later * variant list later * dexpr)
 
 val drec_defn : denv -> pre_fun_defn list -> denv * drec_defn
 
