@@ -15,14 +15,14 @@ val flag: Debug.flag
 
 val dummy_id: ident
 
-val def: ident -> unit
+val def: kind:string -> ident -> unit
   (** [def id] registers that [id] is defined at position [id.id_loc] *)
 
-val use: Loc.position -> ident -> unit
+val use: kind:string -> Loc.position -> ident -> unit
   (** [use loc id] registers that [id] is used at position [loc] *)
 
 type def_use = Def | Use
 
-val find: Loc.position -> ident * def_use
-  (** [find pos] returns the ident used/defined at position [pos], if any,
-      or raises [Not_found] *)
+val find: Loc.position -> ident * def_use * string
+  (** [find pos] returns the ident at position [pos], if any, as well as its
+      used/defined status and its kind, or raises [Not_found] *)
