@@ -146,11 +146,13 @@ let print_ident fmt id =
 let number_format = {
     Number.long_int_support = true;
     Number.extra_leading_zeros_support = true;
+    Number.negative_int_support = Number.Number_custom "-%a";
     Number.dec_int_support = Number.Number_default;
     Number.hex_int_support = Number.Number_default;
     Number.oct_int_support = Number.Number_unsupported;
     Number.bin_int_support = Number.Number_unsupported;
     Number.def_int_support = Number.Number_unsupported;
+    Number.negative_real_support = Number.Number_custom "-%a";
     Number.dec_real_support = Number.Number_default;
     Number.hex_real_support = Number.Number_default;
     Number.frac_real_support = Number.Number_unsupported;
@@ -162,12 +164,10 @@ type constant = Enum of term * int | Value of term | Varying
 let rec constant_value defs t =
   match t.t_node with
   | Tconst c ->
-      fprintf str_formatter "%a" (Number.print number_format) c;
-      flush_str_formatter ()
+      asprintf "%a" (Number.print number_format) c
   | Tapp (ls, [{ t_node = Tconst c}])
       when ls_equal ls !int_minus || ls_equal ls !real_minus ->
-      fprintf str_formatter "-%a" (Number.print number_format) c;
-      flush_str_formatter ()
+      asprintf "-%a" (Number.print number_format) c
   | Tapp (ls, []) ->
     begin
       match Hid.find defs ls.ls_name with
@@ -210,7 +210,7 @@ let rec print_term info defs fmt t =
   | Tcase _ -> unsupportedTerm t
       "gappa: you must eliminate match"
   | Teps _ -> unsupportedTerm t
-      "gappa : you must eliminate epsilon"
+      "gappa: you must eliminate epsilon"
   | Tquant _ | Tbinop _ | Tnot _ | Ttrue | Tfalse -> raise (TermExpected t)
 
 
@@ -413,7 +413,7 @@ let prepare defs ints truths acc d =
       split_hyp defs truths pr acc (simpl_fmla defs truths f)
   | Dprop (Pgoal, pr, f) ->
       split_hyp defs truths pr acc (simpl_fmla defs truths (t_not f))
-  | Dprop ((Plemma|Pskip), _, _) ->
+  | Dprop (Plemma, _, _) ->
       unsupportedDecl d "gappa: lemmas are not supported"
 
 let filter_hyp defs (eqs, hyps) ((pr, f) as hyp) =
