@@ -108,11 +108,13 @@ let rec print_term info fmt t = match t.t_node with
       let number_format = {
           Number.long_int_support = true;
           Number.extra_leading_zeros_support = true;
+          Number.negative_int_support = Number.Number_custom "(- 0 %a)";
           Number.dec_int_support = Number.Number_default;
           Number.hex_int_support = Number.Number_unsupported;
           Number.oct_int_support = Number.Number_unsupported;
           Number.bin_int_support = Number.Number_unsupported;
           Number.def_int_support = Number.Number_unsupported;
+          Number.negative_real_support = Number.Number_custom "(- 0 %a)";
           Number.dec_real_support = Number.Number_unsupported;
           Number.hex_real_support = Number.Number_unsupported;
           Number.frac_real_support = Number.Number_custom
@@ -265,7 +267,7 @@ let print_decl info fmt d = match d.d_node with
         | Some loc -> fprintf fmt " @[;; %a@]@\n" Loc.gen_report_position loc
         | None -> ());
       fprintf fmt "  @[(not %a)@])@]@\n(check)@\n" (print_fmla info) f
-  | Dprop ((Plemma|Pskip), _, _) -> assert false
+  | Dprop (Plemma, _, _) -> assert false
 
 let print_decls =
   let print_decl (sm,ct) fmt d =
