@@ -16,6 +16,9 @@
 #include "options.h"
 #include "logging.h"
 
+// Global pointers are initialized with NULL by C semantics
+pqueue queue;
+
 //count the semicolons in <buf>, up to <len>
 int count_semicolons(char* buf, int len);
 
@@ -29,8 +32,7 @@ int copy_up_to_semicolon(char* buf, int begin, int len, char** result);
 
 int count_semicolons(char* buf, int len) {
   int cnt = 0;
-  int i = 0;
-  for (i = 0; i < len; i++) {
+  for (int i = 0; i < len; i++) {
     if (buf[i] == ';') {
       cnt++;
     }
@@ -57,7 +59,6 @@ int copy_up_to_semicolon(char* buf, int begin, int len, char** result) {
 
 prequest parse_request(char* str_req, int len, int key) {
   int numargs, semic, parallel_arg;
-  int i = 0;
   int pos = 0;
   prequest req;
   char* tmp;
@@ -70,8 +71,8 @@ prequest parse_request(char* str_req, int len, int key) {
   if (semic == 0) {
     return NULL;
   }
-  //  might be a 'parallel' command
   if (semic == 1) {
+    //  might be a 'parallel' or a 'interrupt' command
     pos = copy_up_to_semicolon (str_req, pos, len, &tmp);
     if (strncmp(tmp, "parallel", pos) == 0) {
       free(tmp);
@@ -81,6 +82,14 @@ prequest parse_request(char* str_req, int len, int key) {
         parallel = parallel_arg;
       }
     }
+    else if (strncmp(tmp, "interrupt", pos) == 0) {
+      free(tmp);
+      req = (prequest) malloc(sizeof(request));
+      req->key = key;
+      req->req_type = REQ_INTERRUPT;
+      pos = copy_up_to_semicolon(str_req, pos, len, &(req->id));
+      return req;
+    }
     free(tmp);
     return NULL;
   }
@@ -101,6 +110,7 @@ prequest parse_request(char* str_req, int len, int key) {
   free(tmp);
   req = (prequest) malloc(sizeof(request));
   req->key = key;
+  req->req_type = REQ_RUN;
   req->numargs = numargs;
   req->usestdin = runstdin;
   pos = copy_up_to_semicolon(str_req, pos, len, &(req->id));
@@ -112,17 +122,16 @@ prequest parse_request(char* str_req, int len, int key) {
   free(tmp);
   pos = copy_up_to_semicolon(str_req, pos, len, &(req->cmd));
   req->args = (char**)malloc(sizeof(char*) * (numargs));
-  for (i = 0; i < numargs; i++) {
+  for (int i = 0; i < numargs; i++) {
     pos = copy_up_to_semicolon(str_req, pos, len, &(req->args[i]));
   }
   return req;
 }
 
 void print_request(prequest r) {
-  int i;
   if (r) {
     printf("%s %d %d %s", r->id, r->timeout, r->memlimit, r->cmd);
-    for (i = 0; i < r->numargs; i++) {
+    for (int i = 0; i < r->numargs; i++) {
        printf(" %s", r->args[i]);
     }
   } else {
@@ -131,11 +140,28 @@ void print_request(prequest r) {
 }
 
 void free_request(prequest r) {
-  int i;
   free(r->cmd);
-  for (i = 0;i < r->numargs; i++) {
+  for (int i = 0;i < r->numargs; i++) {
     free(r->args[i]);
   }
   free(r->args);
   free(r);
 }
+
+void init_request_queue () {
+  queue = init_queue(100);
+}
+
+void remove_from_queue(char *id) {
+  // inefficient, but what else?
+  pqueue tmp = init_queue(queue->capacity);
+  while (!queue_is_empty(queue)) {
+    prequest r = queue_pop(queue);
+    if (strcmp(r->id,id)) queue_push(tmp, r);
+  }
+  while (!queue_is_empty(tmp)) {
+    prequest r = queue_pop(tmp);
+    queue_push(queue, r);
+  }
+  free_queue(tmp);
+}
