@@ -11,5 +11,4 @@
 
 exception SyntaxError of string
 
-val parse : 'a Session.env_session -> string -> Strategy.t
-
+val parse : Env.env -> Whyconf.config -> string -> Strategy.t
