@@ -22,19 +22,19 @@ let arg_extra_expl_prefix =
    Arg.String (fun s -> expl_prefixes := s :: !expl_prefixes),
    "<s> register s as an additional prefix for VC explanations")
 
-let collect_expls lab =
-  Ident.Slab.fold
-    (fun lab acc ->
-       let lab = lab.Ident.lab_string in
+let collect_expls attr =
+  Ident.Sattr.fold
+    (fun attr acc ->
+       let attr = attr.Ident.attr_string in
        let rec aux l =
          match l with
            | [] -> acc
            | p :: r ->
               try
-                let s = Strings.remove_prefix p lab in s :: acc
+                let s = Strings.remove_prefix p attr in s :: acc
               with Not_found -> aux r
        in aux !expl_prefixes)
-    lab
+    attr
     []
 
 let concat_expls = function
@@ -42,13 +42,12 @@ let concat_expls = function
   | [l] -> Some l
   | l :: ls -> Some (l ^ " (" ^ String.concat ". " ls ^ ")")
 
-
-let search_labels callback =
+let search_attrs callback =
   let rec aux acc f =
     if f.t_ty <> None then acc
-    else if Ident.Slab.mem Split_goal.stop_split f.Term.t_label then acc
+    else if Ident.Sattr.mem Term.stop_split f.Term.t_attrs then acc
     else
-      let res = callback f.Term.t_label in
+      let res = callback f.Term.t_attrs in
       if res = [] then match f.t_node with
         | Term.Ttrue | Term.Tfalse | Term.Tapp _ -> acc
         | Term.Tbinop (Term.Timplies, _, f) -> aux acc f
@@ -61,7 +60,7 @@ let search_labels callback =
   aux []
 
 let get_expls_fmla =
-  let search = search_labels collect_expls in
+  let search = search_attrs collect_expls in
   fun f -> try search f with Exit -> []
 
 let goal_expl_task ~root task =
@@ -71,7 +70,7 @@ let goal_expl_task ~root task =
     concat_expls
       (if res <> [] && not root
        then res
-       else collect_expls gid.Ident.id_label)
+       else collect_expls gid.Ident.id_attrs)
   in
   let info =
     match info with
@@ -227,9 +226,9 @@ let debug = Debug.register_info_flag "session_pairing"
   ~desc:"Print@ debugging@ messages@ about@ reconstruction@ of@ \
          session@ trees@ after@ modification@ of@ source@ files."
 
-let current_shape_version = 4
+let current_shape_version = 5
 
-type shape_version = SV1 | SV2 | SV3
+type shape_version = SV1 | SV2 | SV3 | SV4
 
 (* similarity code of terms, or of "shapes"
 
@@ -260,15 +259,17 @@ let tag_var = 'V'
 let tag_wild = 'w'
 let tag_as = 'z'
 
+exception ShapeTooLong
+
 let shape_buffer = Buffer.create 17
 
 let push s =
   Buffer.add_string shape_buffer s;
-  if Buffer.length shape_buffer >= 256 then raise Exit
+  if Buffer.length shape_buffer >= 1024 then raise ShapeTooLong
 
 let pushc c =
   Buffer.add_char shape_buffer c;
-  if Buffer.length shape_buffer >= 256 then raise Exit
+  if Buffer.length shape_buffer >= 1024 then raise ShapeTooLong
 
 let ident h id =
   let x =
@@ -301,6 +302,7 @@ let ident_shape id = id_string_shape id.Ident.id_string
 open Number
 
 let integer_const_shape = function
+  | IConstRaw i -> push (BigInt.to_string i)
   | IConstDec s -> push s
   | IConstHex s -> push "0x"; push s
   | IConstOct s -> push "0o"; push s
@@ -312,11 +314,13 @@ let real_const_shape = function
   | RConstHex (i,f,Some e) -> push "0x"; push i; push "."; push f; push "p"; push e
   | RConstHex (i,f,None)   -> push "0x"; push i; push "."; push f
 
+let sign_shape is_negative =
+  if is_negative then pushc '-'
 
 let const_shape c =
   match c with
-  | ConstInt c -> integer_const_shape c
-  | ConstReal c -> real_const_shape c
+  | ConstInt c -> sign_shape c.ic_negative; integer_const_shape c.ic_abs
+  | ConstReal c -> sign_shape c.rc_negative; real_const_shape c.rc_abs
 
 
 let rec pat_shape c m p : 'a =
@@ -343,7 +347,7 @@ let rec t_shape ~version c m t =
     | Tif (f,t1,t2) ->
       begin match version with
       | SV1 | SV2 -> pushc tag_if; fn f; fn t1; fn t2
-      | SV3 -> pushc tag_if; fn t2; fn t1; fn f
+      | SV3 | SV4 -> pushc tag_if; fn t2; fn t1; fn f
       end
     | Tcase (t1,bl) ->
         let br_shape b =
@@ -353,7 +357,7 @@ let rec t_shape ~version c m t =
             pat_shape c m p;
             pat_rename_alpha c m p;
             t_shape ~version c m t2
-          | SV3 ->
+          | SV3 | SV4 ->
             pat_rename_alpha c m p;
             t_shape ~version c m t2;
             pat_shape c m p
@@ -363,7 +367,7 @@ let rec t_shape ~version c m t =
                  pushc tag_case;
                  fn t1;
                  List.iter br_shape bl
-        | SV3 ->
+        | SV3 | SV4 ->
            pushc tag_case;
            List.iter br_shape bl;
            fn t1
@@ -405,30 +409,52 @@ let rec t_shape ~version c m t =
           match version with
             | SV1 ->
                pushc tag_let; fn t1; t_shape ~version c m t2
-            | SV2 | SV3 ->
+            | SV2 | SV3 | SV4 ->
                      (* t2 first, intentionally *)
                      t_shape ~version c m t2; pushc tag_let; fn t1
         end
     | Tnot f ->
       begin match version with
       | SV1 | SV2 -> fn f; pushc tag_not
-      | SV3 -> pushc tag_not; fn f
+      | SV3 | SV4 -> pushc tag_not; fn f
       end
     | Ttrue -> pushc tag_true
     | Tfalse -> pushc tag_false
 
-
 let t_shape_task ~version ~expl t =
   Buffer.clear shape_buffer;
-  begin match version with
-  | SV1 | SV2 -> ()
-  | SV3 -> push expl
-  end;
   let f = Task.task_goal_fmla t in
+  let c = ref (-1) in
+  let m = ref Ident.Mid.empty in
   let () =
     try
-      t_shape ~version (ref (-1)) (ref Ident.Mid.empty) f
-    with Exit -> ()
+      (* expl *)
+      begin match version with
+      | SV1 | SV2 -> ()
+      | SV3 | SV4 -> push expl end;
+      (* goal shape *)
+      t_shape ~version c m f;
+      (* introduced premises shape *)
+      begin match version with
+      | SV1 | SV2 | SV3 -> ()
+      | SV4 ->
+         let open Decl in
+         let introduced id = Ident.Sattr.mem
+                               Introduction.intro_attr
+                               id.Ident.id_attrs in
+         let do_td td = match td.Theory.td_node with
+           | Theory.Decl d ->
+              begin match d.d_node with
+              | Dparam _ls -> ()
+              | Dprop (_, pr, f) when introduced pr.pr_name ->
+                 t_shape ~version c m f
+              | _ -> ()
+              end
+           | _ -> () in
+         let _, prev = Task.task_separate_goal t in
+         Task.task_iter do_td prev
+      end
+    with ShapeTooLong -> ()
   in
   Buffer.contents shape_buffer
 
@@ -438,7 +464,7 @@ let time = ref 0.0
 
 let t_shape_task ?(version=current_shape_version) ~expl t =
   let version = match version with
-    | 1 -> SV1 | 2 -> SV2 | 3 | 4 -> SV3
+    | 1 -> SV1 | 2 -> SV2 | 3 | 4 -> SV3 | 5 -> SV4
     | _ -> assert false
   in
 (*
@@ -497,14 +523,8 @@ module Checksum = struct
     | CV1 -> ident_v1 b id
     | CV2 -> ident_v2 b id
 
-(*
-  let _integer_constant b c =
-    Number.print_integer_constant Format.str_formatter c;
-    let s = Format.flush_str_formatter () in
-    string b s
-*)
-
   let integer_const b = function
+    | IConstRaw i -> raw_string b (BigInt.to_string i)
     | IConstDec s -> raw_string b s
     | IConstHex s -> raw_string b "0x"; raw_string b s
     | IConstOct s -> raw_string b "0o"; raw_string b s
@@ -521,10 +541,13 @@ module Checksum = struct
     | RConstHex (i,f,None)   ->
        raw_string b "0x"; raw_string b i; raw_string b "."; raw_string b f
 
+  let sign b is_negative =
+    if is_negative then raw_string b "-"
+
   let const b c =
     match c with
-    | ConstInt c -> integer_const b c
-    | ConstReal c -> real_const b c
+    | ConstInt c -> sign b c.ic_negative; integer_const b c.ic_abs
+    | ConstReal c -> sign b c.rc_negative; real_const b c.rc_abs
 
   let tvsymbol b tv = ident b tv.Ty.tv_name
 
@@ -594,7 +617,6 @@ module Checksum = struct
     ident b ls.ls_name;
     list ty b ls.ls_args;
     option ty b ls.ls_value;
-    list tvsymbol b (Ty.Stv.elements ls.ls_opaque);
     int b ls.ls_constr
 
   (* start: T G F D R L I P (C M) *)
@@ -626,7 +648,6 @@ module Checksum = struct
           | Decl.Plemma -> "PL"
           | Decl.Paxiom -> "PA"
           | Decl.Pgoal  -> "PG"
-          | Decl.Pskip  -> "PS"
         in
         string b tag;
         ident b n.Decl.pr_name;
@@ -653,6 +674,19 @@ module Checksum = struct
     | Theory.MAstr s -> char b 's'; string b s
     | Theory.MAint i -> char b 'i'; int b i
 
+  let tdecl b d = match d.Theory.td_node with
+    | Theory.Decl d -> decl b d
+    | Theory.Use th
+    | Theory.Clone (th, _) ->
+        char b 'C'; ident b th.Theory.th_name; list string b th.Theory.th_path
+    | Theory.Meta (m, mal) -> char b 'M'; meta b m; list meta_arg b mal
+
+(* not used anymore
+
+  NOTE: if we come back to checksumming theories, use the separate recursive
+        [tdecl] function for it. [Use] in a theory requires a recursive call
+        (as below), [Use] in a task is just a witness declaration.
+
   let rec tdecl b d = match d.Theory.td_node with
     | Theory.Decl d -> decl b d
     | Theory.Use th ->
@@ -679,7 +713,6 @@ module Checksum = struct
         Ident.Wid.set table t.Theory.th_name v;
         v
 
-(* not used anymore
   let theory ~version t = match version with
     | CV1 -> assert false
     | CV2 -> theory_v2 t
@@ -733,7 +766,7 @@ let time = ref 0.0
 let task_checksum ?(version=current_shape_version) t =
   let version = match version with
     | 1 | 2 | 3 -> CV1
-    | 4 -> CV2
+    | 4 | 5 -> CV2
     | _ -> assert false
   in
 (*
