@@ -16,10 +16,10 @@ val arg_extra_expl_prefix : string * Arg.spec * string
 val goal_expl_task:
   root:bool -> Task.task -> Ident.ident * string * Task.task
 
-val search_labels :
-  (Ident.Slab.t -> 'a list) -> Term.term -> 'a list
-  (* [search_labels callback f] traverses [f] in a top-down manner and calls the
-     [callback] on the label set of all encountered nodes. As soon as the
+val search_attrs :
+  (Ident.Sattr.t -> 'a list) -> Term.term -> 'a list
+  (* [search_attrs callback f] traverses [f] in a top-down manner and calls the
+     [callback] on the attribute set of all encountered nodes. As soon as the
      callback returns a non-empty list, the traversal is stopped and that list
      is returned. Raises exception Exit if the entire term has been traversed.
    *)
