@@ -23,6 +23,7 @@ let opt_list_printers = ref false
 let opt_list_provers = ref false
 let opt_list_formats = ref false
 let opt_list_metas = ref false
+let opt_list_attrs = ref false
 
 let option_list = [
   "--list-transforms", Arg.Set opt_list_transforms,
@@ -35,6 +36,7 @@ let option_list = [
       " list known input formats";
   "--list-metas", Arg.Set opt_list_metas,
       " list known metas";
+  "--list-attributes", Arg.Set opt_list_attrs, "list used attributes";
   "--print-libdir",
       Arg.Unit (fun _ -> printf "%s@." Config.libdir; exit 0),
       " print location of binary components (plugins, etc)";
@@ -151,6 +153,11 @@ let () = try
     printf "@[<hov 2>Known metas:@\n%a@]@\n@."
       (Pp.print_list Pp.newline2 print) (List.sort cmp (Theory.list_metas ()))
   end;
+  if !opt_list_attrs then begin
+    opt_list := true;
+    let l = List.sort String.compare (Ident.list_attributes ()) in
+    List.iter (fun x -> Format.eprintf "%s@." x) l
+  end;
   if !opt_list then exit 0;
 
   printf "@[%s%a@]" usage_msg extra_help ()
@@ -158,9 +165,3 @@ let () = try
   with e when not (Debug.test_flag Debug.stack_trace) ->
     eprintf "%a@." Exn_printer.exn_printer e;
     exit 1
-
-(*
-Local Variables:
-compile-command: "unset LANG; make -C ../.. byte"
-End:
-*)
