@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2017   --   INRIA - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -44,7 +44,9 @@ let meta_monomorphic_types_only =
 
 
 let check_ts ign_ts ts =
-  ts.Ty.ts_args <> [] && not (Ty.Sts.mem ts ign_ts)
+  ts.Ty.ts_args <> [] &&
+  ts.Ty.ts_def = Ty.NoDef &&
+  not (Ty.Sts.mem ts ign_ts)
 
 let check_ls ign_ls ls =
   not (Term.Sls.mem ls ign_ls) &&
@@ -54,18 +56,18 @@ let check_ls ign_ls ls =
     (Ty.oty_cons ls.Term.ls_args ls.Term.ls_value)
 
 let detect_polymorphism_in_decl ign_ts ign_ls ign_pr d =
-  Debug.dprintf debug "[detect_polymorphism] |sts|=%d |sls|=%d |spr|=%d@."
+  Debug.dprintf debug "|sts|=%d |sls|=%d |spr|=%d@."
                 (Ty.Sts.cardinal ign_ts)
                 (Term.Sls.cardinal ign_ls)
                 (Spr.cardinal ign_pr);
-  Debug.dprintf debug "[detect_polymorphism] decl %a@."
+  Debug.dprintf debug "decl %a@."
                 Pretty.print_decl d;
   match d.d_node with
   | Dtype ts -> check_ts ign_ts ts
   | Ddata dl ->
      List.fold_left (fun acc (ts,_) -> acc || check_ts ign_ts ts) false dl
   | Dparam ls ->
-     Debug.dprintf debug "[detect_polymorphism] param %a@."
+     Debug.dprintf debug "param %a@."
                 Pretty.print_ls ls;
      check_ls ign_ls ls
   | Dlogic dl ->
