@@ -19,21 +19,23 @@ type split = {
   byso_split : bool;
   side_split : bool;
   stop_split : bool;
+  cpos_split : bool;
+  cneg_split : bool;
   asym_split : bool;
+  intro_mode : bool;
   comp_match : known_map option;
 }
 
-let stop_split = Ident.create_label "stop_split"
-let compiled = Ident.create_label "split_goal: compiled match"
-let case_label = Ident.create_label "case_split"
+let stop f = Sattr.mem Term.stop_split f.t_attrs
+let asym f = Sattr.mem Term.asym_split f.t_attrs
 
-let stop f = Slab.mem stop_split f.t_label
-let asym f = Slab.mem Term.asym_label f.t_label
-let keep f = Slab.mem Term.keep_on_simp_label f.t_label
-let case f = Slab.mem case_label f.t_label
+let case_split = Ident.create_attribute "case_split"
+let case f = Sattr.mem case_split f.t_attrs
+
+let compiled = Ident.create_attribute "split_goal: compiled match"
 
 let unstop f =
-  t_label ?loc:f.t_loc (Slab.remove stop_split f.t_label) f
+  t_attr_set ?loc:f.t_loc (Sattr.remove stop_split f.t_attrs) f
 
 (* Represent monoid of formula interpretation for conjonction and disjunction *)
 module M = struct
@@ -51,28 +53,11 @@ module M = struct
   (* inject formula into monoid. *)
   let (!+) a = Comb (Base a)
 
-  let rec filter c = match c with
-    | Base a when keep a -> Some c
-    | Base _ -> None
-    | Op (a,b) ->
-        match filter a, filter b with
-        | None, u | u, None -> u
-        | Some a, Some b -> Some (Op (a,b))
-
   (* monoid law. *)
   let (++) a b =
     match a, b with
-    | _, Unit -> a
-    | Unit, _ -> b
-    | Zero ta, Comb b -> begin match filter b with
-        | None -> a
-        | Some b -> Comb (Op (Base ta,b))
-        end
-    | Comb a, Zero tb -> begin match filter a with
-        | None -> b
-        | Some a -> Comb (Op (a,Base tb))
-        end
-    | Zero _, Zero _ -> a
+    | _, Unit | Zero _, _ -> a
+    | Unit, _ | _, Zero _ -> b
     | Comb ca, Comb cb -> Comb (Op (ca, cb))
 
   (* (base -> base) morphism application. *)
@@ -190,17 +175,15 @@ let fold_cond = function
   | x -> x
 
 let rec split_core sp f =
-  let rc = split_core sp in
-  let (~-) = t_label_copy f in
+  let (~-) = t_attr_copy f in
   let ro = sp.right_only in
   let alias fo1 unop f1 =
     if fo1 == f1 then f else - unop f1 in
   let alias2 fo1 fo2 binop f1 f2 =
     if fo1 == f1 && fo2 == f2 then f else - binop f1 f2 in
   let rec trivial n = function
-    | [] -> true
     | x :: q -> let m = n + degree x in (m <= 1 && trivial m q)
-  in
+    | [] -> true in
   let trivial bs = trivial 0 bs in
   let pcaset bs sf =
     let test = not ro || (sf.cpos && trivial bs) in
@@ -210,25 +193,34 @@ let rec split_core sp f =
     let test = not ro || (sf.cneg && trivial bs) in
     (if test then sf.neg else !+(sf.fwd)), test in
   let ncase bs sf = let x, _ = ncaset bs sf in x in
+  let ps_csp sp = { sp with cpos_split = false } in
+  let ng_csp sp = { sp with cneg_split = false } in
+  let no_csp sp = { sp with cpos_split = false;
+                            cneg_split = false } in
+  let in_csp sp = { sp with cpos_split = sp.cneg_split;
+                            cneg_split = sp.cpos_split } in
   let ngt _ a = t_not a and cpy _ a = a in
   let bimap = bimap (fun _ t -> Zero t) cpy in
   let iclose = bimap ngt t_implies in
   let aclose = bimap cpy t_and in
-  let nclose ps = map (fun t -> Zero (t_label_copy t t_true)) t_not ps in
+  let nclose ps = map (fun t -> Zero (t_attr_copy t t_true)) t_not ps in
   let ret pos neg bwd fwd side cpos cneg =
       { pos; neg; bwd; fwd; side; cpos; cneg } in
   let r = match f.t_node with
   | _ when sp.stop_split && stop f ->
       let df = drop_byso f in
       ret !+(unstop f) !+(unstop df) f df Unit false false
-  | (Ttrue | Tfalse) when keep f -> ret !+f !+f f f Unit false false
+  | Tbinop (Tiff,_,_) | Tif _ | Tcase _ | Tquant _ when sp.intro_mode ->
+      let df = drop_byso f in
+      ret !+f !+df f df Unit false false
   | Ttrue -> ret Unit (Zero f) f f Unit false false
   | Tfalse -> ret (Zero f) Unit f f Unit false false
   | Tapp _ -> let uf = !+f in ret uf uf f f Unit false false
     (* f1 so f2 *)
   | Tbinop (Tand,f1,{ t_node = Tbinop (Tor,f2,{ t_node = Ttrue }) }) ->
-      if not (sp.byso_split && asym f2) then rc f1 else
+      if not (sp.byso_split && asym f2) then split_core sp f1 else
       let (&&&) f1 f2 = - t_and f1 f2 in
+      let rc = split_core (no_csp sp) in
       let sf1 = rc f1 and sf2 = rc f2 in
       let fwd = sf1.fwd &&& sf2.fwd in
       let nf2, cn2 = ncaset [] sf2 in
@@ -241,11 +233,12 @@ let rec split_core sp f =
       ret sf1.pos neg sf1.bwd fwd side sf1.cpos (cn1 || cn2)
   | Tbinop (Tand,f1,f2) ->
       let (&&&) = alias2 f1 f2 t_and in
+      let rc = split_core (ps_csp sp) in
       let sf1 = rc f1 and sf2 = rc f2 in
       let fwd = sf1.fwd &&& sf2.fwd and bwd = sf1.bwd &&& sf2.bwd in
       let asym = sp.asym_split && asym f1 in
       let nf2, cn2 = ncaset [] sf2 in
-      let sd = if asym then [sf1.side] else [] in
+      let sd = if asym then [sf2.side] else [] in
       let dp = nf2::sd in
       let nf1, cn1 = ncaset dp sf1 in
       let neg = bimap cpy (&&&) nf1 nf2 in
@@ -257,7 +250,8 @@ let rec split_core sp f =
       ret pos neg bwd fwd side false (cn1 || cn2)
     (* f1 by f2 *)
   | Tbinop (Timplies,{ t_node = Tbinop (Tor,f2,{ t_node = Ttrue }) },f1) ->
-      if not (sp.byso_split && asym f2) then rc f1 else
+      if not (sp.byso_split && asym f2) then split_core sp f1 else
+      let rc = split_core (no_csp sp) in
       let sf1 = rc f1 and sf2 = rc f2 in
       let close = iclose (ncase [sf1.pos;sf1.side] sf2) in
       let lside = if sp.side_split then close sf1.pos else
@@ -266,10 +260,11 @@ let rec split_core sp f =
       ret sf2.pos sf1.neg sf2.bwd sf1.fwd side sf2.cpos sf1.cneg
   | Tbinop (Timplies,f1,f2) ->
       let (>->) = alias2 f1 f2 t_implies in
-      let sf1 = rc f1 and sf2 = rc f2 in
+      let sp2 = ng_csp sp in let sp1 = in_csp sp2 in
+      let sf1 = split_core sp1 f1 and sf2 = split_core sp2 f2 in
       let fwd = sf1.bwd >-> sf2.fwd and bwd = sf1.fwd >-> sf2.bwd in
       let asym = sp.asym_split && asym f1 in
-      let sd = [sf1.side] in
+      let sd = [sf2.side] in
       let neg1 = nclose sf1.pos in
       let neg2 = if not asym then sf2.neg else
         let nf1 = ncase (sf2.neg::sd) sf1 in
@@ -283,6 +278,7 @@ let rec split_core sp f =
       ret pos neg bwd fwd side (cn1 || sf2.cpos) false
   | Tbinop (Tor,f1,f2) ->
       let (|||) = alias2 f1 f2 t_or in
+      let rc = split_core (ng_csp sp) in
       let sf1 = rc f1 and sf2 = rc f2 in
       let fwd = sf1.fwd ||| sf2.fwd and bwd = sf1.bwd ||| sf2.bwd in
       let asym = sp.asym_split && asym f1 in
@@ -298,6 +294,7 @@ let rec split_core sp f =
         bimap cpy (|||) pf1 sf2.side in
       ret pos (sf1.neg ++ neg2) bwd fwd (sf1.side ++ side2) (cp1 || cp2) false
   | Tbinop (Tiff,f1,f2) ->
+      let rc = split_core (no_csp sp) in
       let sf1 = rc f1 and sf2 = rc f2 in
       let df = if sf1.fwd == sf1.bwd && sf2.fwd == sf2.bwd
         then alias2 f1 f2 t_iff sf1.fwd sf2.fwd else drop_byso f in
@@ -309,6 +306,7 @@ let rec split_core sp f =
       let neg_bot = aclose (nclose pf1) (nclose pf2) in
       ret pos (neg_top ++ neg_bot) df df (sf1.side ++ sf2.side) false false
   | Tif (fif,fthen,felse) ->
+      let rc = split_core (no_csp sp) in
       let sfi = rc fif and sft = rc fthen and sfe = rc felse in
       let dfi = if sfi.fwd == sfi.bwd then sfi.fwd else drop_byso fif in
       let rebuild fif2 fthen2 felse2 =
@@ -328,24 +326,27 @@ let rec split_core sp f =
       let side = sfi.side ++ iclose nfi sft.side ++ eside in
       ret pos neg bwd fwd side false false
   | Tnot f1 ->
-      let sf = rc f1 in
+      let sf = split_core (in_csp sp) f1 in
       let (!) = alias f1 t_not in
-      let (|>) zero = map (fun t -> !+(t_label_copy t zero)) (!) in
+      let (|>) zero = map (fun t -> !+(t_attr_copy t zero)) (!) in
       let pos = t_false |> sf.neg and neg = t_true |> sf.pos in
       ret pos neg !(sf.fwd) !(sf.bwd) sf.side sf.cneg sf.cpos
   | Tlet (t,fb) ->
       let vs, f1 = t_open_bound fb in
       let (!) = alias f1 (t_let_close vs t) in
-      let sf = rc f1 in
+      let sf = split_core sp f1 in
       let (!!) = map (fun t -> Zero t) (!) in
       ret !!(sf.pos) !!(sf.neg) !(sf.bwd) !(sf.fwd) !!(sf.side) sf.cpos sf.cneg
   | Tcase (t,bl) ->
+      let rc = match bl with
+        | [_] -> split_core sp
+        | _   -> split_core (no_csp sp) in
       let k join =
         let case_close bl2 =
           if Lists.equal (==) bl bl2 then f else - t_case t bl2 in
         let sbl = List.map (fun b ->
           let p, f, close = t_open_branch_cb b in
-          p, close, split_core sp f) bl in
+          p, close, rc f) bl in
         let blfwd = List.map (fun (p, close, sf) -> close p sf.fwd) sbl in
         let fwd = case_close blfwd in
         let blbwd = List.map (fun (p, close, sf) -> close p sf.bwd) sbl in
@@ -377,18 +378,18 @@ let rec split_core sp f =
           in
           k join
       | Some kn ->
-          if Slab.mem compiled f.t_label
+          if Sattr.mem compiled f.t_attrs
           then
-            (* keep the labels for single-case match *)
-            let lab = match bl with
-              | [_] -> Slab.remove case_label
-                    (Slab.remove compiled f.t_label)
-              | _ -> Slab.empty in
+            (* keep the attributes for single-case match *)
+            let attrs = match bl with
+              | [_] -> Sattr.remove case_split
+                    (Sattr.remove compiled f.t_attrs)
+              | _ -> Sattr.empty in
             let join sbl =
               let vs = create_vsymbol (id_fresh "q") (t_type t) in
               let tv = t_var vs in
               let (~-) fb =
-                t_label ?loc:f.t_loc lab (t_let_close_simp vs t fb) in
+                t_attr_set ?loc:f.t_loc attrs (t_let_close_simp vs t fb) in
               let _, pos, neg, side =
                 List.fold_left (fun (cseen, pos, neg, side) (p, _, sf) ->
                   let cseen, vl, cond = pat_condition kn tv cseen p in
@@ -409,16 +410,15 @@ let rec split_core sp f =
             k join
           else
             let mk_let = t_let_close_simp in
-            let mk_case t bl = t_label_add compiled (t_case_close t bl) in
+            let mk_case t bl = t_attr_add compiled (t_case_close t bl) in
             let mk_b b = let p, f = t_open_branch b in [p], f in
             let bl = List.map mk_b bl in
-            let f = - Pattern.compile_bare ~mk_case ~mk_let [t] bl in
-            split_core sp f
+            rc (- Pattern.compile_bare ~mk_case ~mk_let [t] bl)
       end
   | Tquant (qn,fq) ->
       let vsl, trl, f1 = t_open_quant fq in
       let close = alias f1 (t_quant_close qn vsl trl) in
-      let sf = rc f1 in
+      let sf = split_core sp f1 in
       let bwd = close sf.bwd and fwd = close sf.fwd in
       let pos, neg, cpos, cneg = match qn with
         | Tforall ->
@@ -432,7 +432,8 @@ let rec split_core sp f =
       ret pos neg bwd fwd side cpos cneg
   | Tvar _ | Tconst _ | Teps _ -> raise (FmlaExpected f)
   in
-  let r = if case f then { r with cpos = true; cneg = true } else r in
+  let r = if case f then
+    { r with cpos = sp.cpos_split; cneg = sp.cneg_split } else r in
   match r with
   | { side = M.Zero _ } ->
       { r with pos = Unit; neg = Unit; fwd = t_false; bwd = t_true }
@@ -444,7 +445,10 @@ let full_split kn = {
   byso_split = false;
   side_split = true;
   stop_split = false;
+  cpos_split = true;
+  cneg_split = true;
   asym_split = true;
+  intro_mode = false;
   comp_match = kn;
 }
 
@@ -453,6 +457,7 @@ let full_proof  kn = { (full_split kn) with stop_split = true;
                                             byso_split = true }
 let right_proof kn = { (full_proof kn) with right_only = true }
 let full_intro  kn = { (full_split kn) with asym_split = false;
+                                            intro_mode = true;
                                             stop_split = true }
 let right_intro kn = { (full_intro kn) with right_only = true }
 
@@ -521,15 +526,12 @@ let prep_premise split = Trans.store (fun t ->
 
 let split_goal_full  = prep_goal full_proof
 let split_goal_right = prep_goal right_proof
-let split_goal_wp    = split_goal_right
 
 let split_all_full  = prep_all full_proof
 let split_all_right = prep_all right_proof
-let split_all_wp    = split_all_right
 
 let split_premise_full  = prep_premise full_proof
 let split_premise_right = prep_premise right_proof
-let split_premise_wp    = split_premise_right
 
 let () = Trans.register_transform_l "split_goal_full" split_goal_full
   ~desc:"Put@ the@ goal@ in@ a@ conjunctive@ form,@ \
@@ -548,11 +550,3 @@ let () = Trans.register_transform_l "split_all_right" split_all_right
   ~desc:"Same@ as@ split_goal_right,@ but@ also@ split@ premises."
 let () = Trans.register_transform "split_premise_right" split_premise_right
   ~desc:"Same@ as@ split_all_right,@ but@ split@ only@ premises."
-
-let () = Trans.register_transform_l "split_goal_wp" split_goal_wp
-  ~desc:"Same@ as@ split_goal_right."
-let () = Trans.register_transform_l "split_all_wp" split_all_wp
-  ~desc:"Same@ as@ split_goal_wp,@ but@ also@ split@ premises."
-let () = Trans.register_transform "split_premise_wp" split_premise_wp
-  ~desc:"Same@ as@ split_all_wp,@ but@ split@ only@ premises."
-
