@@ -9,9 +9,15 @@
 (*                                                                  *)
 (********************************************************************)
 
+(* Replace with Char.uppercase_ascii as soon as we can assume
+  OCaml version at least 4.03.0  *)
+let char_is_uppercase c = c = Char.uppercase c
+
+let lowercase = String.lowercase
 let capitalize = String.capitalize
 let uncapitalize = String.uncapitalize
 
+
 let rev_split c s =
   let rec aux acc i =
     try
@@ -42,14 +48,6 @@ let rec join sep l =
   | [x] -> x
   | x :: rest -> x ^ sep ^ join sep rest
 
-let ends_with s suf =
-  let rec aux s suf suflen offset i =
-    i >= suflen || (s.[i + offset] = suf.[i]
-                   && aux s suf suflen offset (i+1)) in
-  let slen = String.length s in
-  let suflen = String.length suf in
-  slen >= suflen && aux s suf suflen (slen - suflen) 0
-
 let pad_right c s i =
   let sl = String.length s in
   if sl < i then
@@ -62,15 +60,41 @@ let pad_right c s i =
   else s
 
 let has_prefix pref s =
+  let sl = String.length s in
   let l = String.length pref in
-  if String.length s < l then false else
-    try
-      for i = 0 to l - 1 do if s.[i] <> pref.[i] then raise Exit done;
-      true
-    with Exit -> false
+  let rec aux i =
+    i >= l || (s.[i] = pref.[i] && aux (i+1)) in
+  sl >= l && aux 0
 
 let remove_prefix pref s =
+  let sl = String.length s in
   let l = String.length pref in
-  if String.length s < l then raise Not_found else
-  for i = 0 to l - 1 do if s.[i] <> pref.[i] then raise Not_found done;
-  String.sub s l (String.length s - l)
+  if sl < l then raise Not_found else
+  for i = 0 to l - 1 do
+    if s.[i] <> pref.[i] then raise Not_found
+  done;
+  String.sub s l (sl - l)
+
+let has_suffix suff s =
+  let sl = String.length s in
+  let l = String.length suff in
+  let rec aux i =
+    i >= l || (s.[sl - l + i] = suff.[i] && aux (i+1)) in
+  sl >= l && aux 0
+
+let remove_suffix suff s =
+  let sl = String.length s in
+  let l = String.length suff in
+  if sl < l then raise Not_found else
+  for i = 0 to l - 1 do
+    if s.[sl - l + i] <> suff.[i] then raise Not_found
+  done;
+  String.sub s 0 (sl - l)
+
+let ends_with s suf =
+  let rec aux s suf suflen offset i =
+    i >= suflen || (s.[i + offset] = suf.[i]
+                   && aux s suf suflen offset (i+1)) in
+  let slen = String.length s in
+  let suflen = String.length suf in
+  slen >= suflen && aux s suf suflen (slen - suflen) 0
