@@ -13,11 +13,14 @@
 
 (** {2 Wrappers for deprecated string functions of OCaml stdlib} *)
 
+val lowercase : string -> string
 val capitalize : string -> string
 val uncapitalize : string -> string
 
 (** {2 Other useful functions on strings} *)
 
+val char_is_uppercase : char -> bool
+
 val rev_split : char -> string -> string list
 
 val split : char -> string -> string list
@@ -35,9 +38,6 @@ val join : string -> string list -> string
 (** [join sep l] joins all the strings in [l] together, in the same
     order, separating them by [sep] *)
 
-val ends_with : string -> string -> bool
-(** test if a string ends with another *)
-
 val pad_right : char -> string -> int -> string
 (** chop or pad the given string on the right up to the given length *)
 
@@ -45,5 +45,15 @@ val has_prefix : string -> string -> bool
 (** [has_prefix pref s] returns true if s [s] starts with prefix [pref] *)
 
 val remove_prefix : string -> string -> string
-(** [remove_prefix pref s] removes the prefix [pref] from [s]. Raises
-    [Not_found] if [s] does not start with [pref] *)
+(** [remove_prefix pref s] removes the prefix [pref] from [s].
+    Raises [Not_found] if [s] does not start with [pref] *)
+
+val has_suffix : string -> string -> bool
+(** [has_suffix suff s] returns true if s [s] ends with suffix [suff] *)
+
+val remove_suffix : string -> string -> string
+(** [remove_suffix suff s] removes the suffix [suff] from [s].
+    Raises [Not_found] if [s] does not end with [suff] *)
+
+val ends_with : string -> string -> bool
+(** test if a string ends with another *)
