@@ -18,8 +18,8 @@ open Why3
 open Why3session_lib
 open Whyconf
 open Format
-open Session
-open Stdlib
+open Session_itp
+open Wstdlib
 
 let opt_print_provers = ref false
 let opt_print_edited = ref false
@@ -142,30 +142,29 @@ let update_perf_stats stats ((_,t) as prover_and_time) =
 
 let string_of_prover p = Pp.string_of_wnl print_prover p
 
-let rec stats_of_goal ~root prefix_name stats goal =
+let rec stats_of_goal ~root prefix_name stats ses goal =
   if root
   then stats.nb_root_goals <- stats.nb_root_goals + 1
   else stats.nb_sub_goals <- stats.nb_sub_goals + 1;
   let proof_list =
-    PHprover.fold
-      (fun prover proof_attempt acc ->
-        match proof_attempt.proof_state with
-          | Done result ->
+    List.fold_left
+      (fun acc pa ->
+        match pa.proof_state with
+          | Some result ->
             begin
               match result.Call_provers.pr_answer with
                 | Call_provers.Valid ->
-                  (prover, result.Call_provers.pr_time) :: acc
+                  (pa.prover, result.Call_provers.pr_time) :: acc
                 | _ ->
                   acc
             end
           | _ -> acc)
-      (goal_external_proofs goal)
-      []
+      [] (get_proof_attempts ses goal)
   in
   List.iter (update_perf_stats stats) proof_list;
-  PHstr.iter (stats_of_transf prefix_name stats) (goal_transformations goal);
-  if not (Opt.inhabited (goal_verified goal)) then
-    let goal_name = prefix_name ^ (goal_name goal).Ident.id_string in
+  List.iter (stats_of_transf prefix_name stats ses) (get_transformations ses goal);
+  let goal_name = prefix_name ^ (get_proof_name ses goal).Ident.id_string in
+  if not (pn_proved ses goal) then
     stats.no_proof <- Sstr.add goal_name stats.no_proof
   else
     begin
@@ -175,85 +174,82 @@ let rec stats_of_goal ~root prefix_name stats goal =
         stats.nb_proved_sub_goals <- stats.nb_proved_sub_goals + 1;
       match proof_list with
       | [ (prover, _) ] ->
-        let goal_name = prefix_name ^ (goal_name goal).Ident.id_string in
         stats.only_one_proof <-
           Sstr.add
-          (goal_name ^ " : " ^ (string_of_prover prover))
+          (goal_name ^ ": " ^ (string_of_prover prover))
           stats.only_one_proof
       | _ -> ()
     end
 
-and stats_of_transf prefix_name stats _ transf =
-  let prefix_name = prefix_name ^ transf.transf_name  ^ " / " in
-  List.iter (stats_of_goal ~root:false prefix_name stats) transf.transf_goals
+and stats_of_transf prefix_name stats ses transf =
+  let prefix_name = prefix_name ^ (get_transf_string ses transf) ^ " / " in
+  List.iter (stats_of_goal ~root:false prefix_name stats ses) (get_sub_tasks ses transf)
 
-let stats_of_theory file stats theory =
-  let goals = theory.theory_goals in
-  let prefix_name = file.file_name ^ " / " ^ theory.theory_name.Ident.id_string
+let stats_of_theory file stats ses theory =
+  let goals = theory_goals theory in
+  let prefix_name = file_name file ^ " / " ^ (theory_name theory).Ident.id_string
     ^  " / " in
-  List.iter (stats_of_goal ~root:true prefix_name stats) goals
+  List.iter (stats_of_goal ~root:true prefix_name stats ses) goals
 
-let stats_of_file stats _ file =
-  let theories = file.file_theories in
-  List.iter (stats_of_theory file stats) theories
+let stats_of_file stats ses _ file =
+  let theories = file_theories file in
+  List.iter (stats_of_theory file stats ses) theories
 
 
 
-type 'a goal_stat =
-  | No of ('a transf * ('a goal * 'a goal_stat) list) list
-  | Yes of (prover * float) list * ('a transf * ('a goal * 'a goal_stat) list) list
+type goal_stat =
+  | No of (transID * (proofNodeID * goal_stat) list) list
+  | Yes of (prover * float) list * (transID * (proofNodeID * goal_stat) list) list
 
-let rec stats2_of_goal ~nb_proofs g : 'a goal_stat =
+let rec stats2_of_goal ~nb_proofs ses g : goal_stat =
   let proof_list =
-    PHprover.fold
-      (fun prover proof_attempt acc ->
+    List.fold_left
+      (fun acc proof_attempt ->
         match proof_attempt.proof_state with
-          | Done result ->
+          | Some result ->
             begin
               match result.Call_provers.pr_answer with
                 | Call_provers.Valid ->
-                  (prover, result.Call_provers.pr_time) :: acc
+                  (proof_attempt.prover, result.Call_provers.pr_time) :: acc
                 | _ ->
                   acc
             end
           | _ -> acc)
-      (goal_external_proofs g)
-      []
+      [] (get_proof_attempts ses g)
   in
   let l =
-    PHstr.fold
-      (fun _ tr acc ->
-        match stats2_of_transf ~nb_proofs tr with
+    List.fold_left
+      (fun acc tr ->
+        match stats2_of_transf ~nb_proofs ses tr with
           | [] -> acc
           | r -> (tr,List.rev r)::acc)
-      (goal_transformations g)
-      []
+      [] (get_transformations ses g)
   in
   if match nb_proofs with
-    | 0 -> not (Opt.inhabited (goal_verified g))
+    | 0 -> not (pn_proved ses g)
     | 1 -> List.length proof_list = 1
     | _ -> assert false
       then Yes(proof_list,l) else No(l)
 
-and stats2_of_transf ~nb_proofs tr : ('a goal * 'a goal_stat) list =
+and stats2_of_transf ~nb_proofs ses tr : (proofNodeID * goal_stat) list =
   List.fold_left
     (fun acc g ->
-      match stats2_of_goal ~nb_proofs g with
+      match stats2_of_goal ~nb_proofs ses g with
         | No [] -> acc
         | r -> (g,r)::acc)
-    [] tr.transf_goals
+    [] (get_sub_tasks ses tr)
 
 let print_res ~time fmt (p,t) =
   fprintf fmt "%a" print_prover p;
   if time then fprintf fmt " (%.2f)" t
 
-let rec print_goal_stats ~time depth (g,l) =
+let rec print_goal_stats ~time depth ses (g,l) =
   for _i=1 to depth do printf "  " done;
-  printf "+-- goal %s" (goal_name g).Ident.id_string;
+  printf "+-- goal %s" (get_proof_name ses g).Ident.id_string;
   match l with
     | No l ->
       printf "@\n";
-      List.iter (print_transf_stats ~time (depth+1)) l
+      List.iter (print_transf_stats ~time (depth+1) ses) l
     | Yes(pl,l) ->
       begin
         match pl with
@@ -261,46 +257,46 @@ let rec print_goal_stats ~time depth (g,l) =
           | _ -> printf ": %a@\n"
             (Pp.print_list pp_print_space (print_res ~time)) pl
       end;
-      List.iter (print_transf_stats ~time (depth+1)) l
+      List.iter (print_transf_stats ~time (depth+1) ses) l
 
-and print_transf_stats ~time depth (tr,l) =
+and print_transf_stats ~time depth ses (tr,l) =
   for _i=1 to depth do printf "  " done;
-  printf "+-- transformation %s@\n" tr.transf_name;
-  List.iter (print_goal_stats ~time (depth+1)) l
+  printf "+-- transformation %s@\n" (get_transf_string ses tr);
+  List.iter (print_goal_stats ~time (depth+1) ses) l
 
-let stats2_of_theory ~nb_proofs th =
+let stats2_of_theory ~nb_proofs ses th =
   List.fold_left
     (fun acc g ->
-      match stats2_of_goal ~nb_proofs g with
+      match stats2_of_goal ~nb_proofs ses g with
         | No [] -> acc
         | r -> (g,r)::acc)
-    [] th.theory_goals
+    [] (theory_goals th)
 
-let print_theory_stats ~time (th,r) =
-  printf "  +-- theory %s@\n" th.theory_name.Ident.id_string;
-  List.iter (print_goal_stats ~time 2) r
+let print_theory_stats ~time ses (th,r) =
+  printf "  +-- theory %s@\n" (theory_name th).Ident.id_string;
+  List.iter (print_goal_stats ~time 2 ses) r
 
-let stats2_of_file ~nb_proofs file =
+let stats2_of_file ~nb_proofs ses file =
   List.fold_left
     (fun acc th ->
-      match stats2_of_theory ~nb_proofs th with
+      match stats2_of_theory ~nb_proofs ses th with
         | [] -> acc
         | r -> (th,List.rev r)::acc)
-    [] file.file_theories
+    [] (file_theories file)
 
-let stats2_of_session ~nb_proofs s acc =
-  PHstr.fold
+let stats2_of_session ~nb_proofs ses acc =
+  Hstr.fold
     (fun _ f acc ->
-      match stats2_of_file ~nb_proofs f with
+      match stats2_of_file ~nb_proofs ses f with
         | [] -> acc
         | r -> (f,List.rev r)::acc)
-    s.session_files acc
+    (get_files ses) acc
 
-let print_file_stats ~time (f,r) =
-  printf "+-- file %s@\n" f.file_name;
-  List.iter (print_theory_stats ~time) r
+let print_file_stats ~time ses (f,r) =
+  printf "+-- file %s@\n" (file_name f);
+  List.iter (print_theory_stats ~time ses) r
 
-let print_session_stats ~time = List.iter (print_file_stats ~time)
+let print_session_stats ~time ses = List.iter (print_file_stats ~time ses)
 
 
 (*
@@ -322,7 +318,7 @@ let finalize_stats stats =
     stats.prover_avg_time
 *)
 
-let print_stats r0 r1 stats =
+let print_stats ses r0 r1 stats =
   printf "== Number of root goals ==@\n  total: %d  proved: %d@\n@\n"
     stats.nb_root_goals stats.nb_proved_root_goals;
 
@@ -330,19 +326,19 @@ let print_stats r0 r1 stats =
     stats.nb_sub_goals stats.nb_proved_sub_goals;
 
   printf "== Goals not proved ==@\n  @[";
-  print_session_stats ~time:false r0;
+  print_session_stats ~time:false ses r0;
   (* Sstr.iter (fun s -> printf "%s@\n" s) stats.no_proof; *)
   printf "@]@\n";
 
   printf "== Goals proved by only one prover ==@\n  @[";
-  print_session_stats ~time:false r1;
+  print_session_stats ~time:false ses r1;
   (* Sstr.iter (fun s -> printf "%s@\n" s) stats.only_one_proof; *)
   printf "@]@\n";
 
   printf "== Statistics per prover: number of proofs, time (minimum/maximum/average) in seconds ==@\n  @[";
   Hprover.iter (fun prover n ->
     let sum_times = Hprover.find stats.prover_sum_time prover in
-    printf "%-20s : %3d %6.2f %6.2f %6.2f@\n"
+    printf "%-20s: %3d %6.2f %6.2f %6.2f@\n"
       (string_of_prover prover) n
       (Hprover.find stats.prover_min_time prover)
       (Hprover.find stats.prover_max_time prover)
@@ -352,28 +348,39 @@ let print_stats r0 r1 stats =
 
 
 let run_one stats r0 r1 fname =
-  let project_dir = Session.get_project_dir fname in
-  if !opt_project_dir then printf "%s@." project_dir;
-  let session,_use_shapes = Session.read_session project_dir in
+  let ses,_ = read_session fname in
   let sep = if !opt_print0 then Pp.print0 else Pp.newline in
   if !opt_print_provers then
     printf "%a@."
       (Pp.print_iter1 Sprover.iter sep print_prover)
-      (get_used_provers session);
-  if !opt_print_edited then
-    session_iter_proof_attempt
+      (get_used_provers ses);
+  if !opt_print_edited then begin
+      failwith "option print_edited non implemented" (* TODO *)
+(*    session_iter_proof_attempt
       (fun pr ->
         Opt.iter (fun s -> printf "%s%a" s sep ())
           (get_edited_as_abs session pr))
       session;
+ *)
+    end;
   if !opt_tree_print then
-    printf "%a@." print_session session;
+    begin
+      failwith "option print_tree not implemented"
+(*
+      printf "%a@." print_session ses;
+ *)
+    end;
   if !opt_stats_print || !opt_hist_print then
     begin
       (* fill_prover_data stats session; *)
-      PHstr.iter (stats_of_file stats) session.session_files;
-      r0 := stats2_of_session ~nb_proofs:0 session !r0;
-      r1 := stats2_of_session ~nb_proofs:1 session !r1
+      Hstr.iter (stats_of_file stats ses) (get_files ses);
+      r0 := stats2_of_session ~nb_proofs:0 ses !r0;
+      r1 := stats2_of_session ~nb_proofs:1 ses !r1
+    end;
+  if !opt_stats_print then
+    begin
+      (* finalize_stats stats; *)
+      print_stats ses !r0 !r1 stats
     end
 
 (**** print histograms ******)
@@ -437,16 +444,11 @@ let print_hist stats =
 (****** run on all files  ******)
 
 let run () =
-  let _,_,should_exit1 = read_env_spec () in
+  let _env,_config,should_exit1 = read_env_spec () in
   if should_exit1 then exit 1;
   let stats = new_proof_stats () in
   let r0 = ref [] and r1 = ref [] in
   iter_files (run_one stats r0 r1);
-  if !opt_stats_print then
-    begin
-      (* finalize_stats stats; *)
-      print_stats !r0 !r1 stats
-    end;
   if !opt_hist_print then print_hist stats
 
 
