@@ -9,12 +9,13 @@
 (*                                                                  *)
 (********************************************************************)
 
-open Why3
-open Why3session_lib
 open Format
+open Why3
+open Wstdlib
+open Session_itp
+open Why3session_lib
 
 module Hprover = Whyconf.Hprover
-module S = Session
 
 let opt_style = ref 1
 let opt_output_dir = ref ""
@@ -46,22 +47,9 @@ let spec =
 
 (* Statistics in LaTeX*)
 
-let rec transf_depth tr =
-  List.fold_left (fun depth g -> max depth (goal_depth g)) 0 tr.S.transf_goals
-and goal_depth g =
-  S.PHstr.fold
-    (fun _st tr depth -> max depth (1 + transf_depth tr))
-    (S.goal_transformations g) 0
-
-let theory_depth t =
-  List.fold_left (fun depth g -> max depth (goal_depth g)) 0 t.S.theory_goals
-
-let file_depth f =
-  List.fold_left (fun depth t -> max depth (theory_depth t)) 0
-    f.S.file_theories
-
+(*
 let provers_latex_stats_goal provers goal =
-  S.goal_iter_proof_attempt (fun a ->
+  goal_iter_proof_attempt (fun a ->
     Hprover.replace provers a.S.proof_prover a.S.proof_prover) goal
 
 let provers_latex_stats provers theory =
@@ -71,6 +59,7 @@ let provers_latex_stats provers theory =
 let provers_latex_stats_file provers file =
   S.file_iter_proof_attempt (fun a ->
     Hprover.replace provers a.S.proof_prover a.S.proof_prover) file
+ *)
 
 let protect s =
   let b = Buffer.create 7 in
@@ -117,13 +106,13 @@ let print_tabular_foot fmt =
   fprintf fmt "\\hline \\end{tabular}@."
 
 
-let print_result_prov proofs prov fmt=
+let print_result_prov s proofs prov fmt=
   List.iter (fun p ->
   try
-    let pr = S.PHprover.find proofs p in
-    let s = pr.S.proof_state in
+    let pr = get_proof_attempt_node s (Hprover.find proofs p) in
+    let s = pr.proof_state in
       match s with
-	| Session.Done res ->
+	| Some res ->
 	    begin
 	      match res.Call_provers.pr_answer with
 		| Call_provers.Valid ->
@@ -132,10 +121,10 @@ let print_result_prov proofs prov fmt=
                   fprintf fmt "& \\invalid{%.2f} " res.Call_provers.pr_time
 		| Call_provers.Timeout ->
                   fprintf fmt "& \\timeout{%ds} "
-                    pr.S.proof_limit.Call_provers.limit_time
+                    pr.limit.Call_provers.limit_time
 		| Call_provers.OutOfMemory ->
                   fprintf fmt "& \\outofmemory{%dM} "
-                    pr.S.proof_limit.Call_provers.limit_mem
+                    pr.limit.Call_provers.limit_mem
 		| Call_provers.StepLimitExceeded ->
                   fprintf fmt "& \\steplimitexceeded "
 		| Call_provers.Unknown _ ->
@@ -146,16 +135,12 @@ let print_result_prov proofs prov fmt=
                   fprintf fmt "& \\highfailure "
 
 	    end
-	| Session.InternalFailure _ -> fprintf fmt "& Internal Failure"
-	| Session.Interrupted -> fprintf fmt "& Not yet run"
-	| Session.Unedited -> fprintf fmt "& Not yet edited"
-	| Session.Scheduled | Session.Running
-	| Session.JustEdited -> assert false
+	| None -> fprintf fmt "(no result)"
   with Not_found -> fprintf fmt "& \\noresult") prov;
   fprintf fmt "\\\\ @."
 
 
-let rec goal_latex_stat fmt prov depth depth_max subgoal g =
+let rec goal_latex_stat s fmt prov depth depth_max subgoal g =
   let column = column 1 depth prov
   in
     if depth > 0 then
@@ -190,11 +175,12 @@ let rec goal_latex_stat fmt prov depth depth_max subgoal g =
 	  if depth > 0 then fprintf fmt " & "
       end;
     if (depth <= 1) then
-      fprintf fmt "\\explanation{%s} " (protect (S.goal_user_name g))
+      fprintf fmt "\\explanation{%s} "
+              (protect (get_proof_name s g).Ident.id_string)
     else
       fprintf fmt " " ;
-    let proofs = S.goal_external_proofs g in
-      if (S.PHprover.length proofs) > 0 then
+    let proofs = get_proof_attempt_ids s g in
+      if (Hprover.length proofs) > 0 then
 	begin
 	  if depth_max <= 1 then
 	    begin
@@ -212,16 +198,16 @@ let rec goal_latex_stat fmt prov depth depth_max subgoal g =
 	    else
 	      for _i = depth to (depth_max - depth - 2) do
 		fprintf fmt "& " done;
-	  print_result_prov proofs prov fmt;
+	  print_result_prov s proofs prov fmt;
 	end;
-      let tr = S.goal_transformations g in
-	if S.PHstr.length tr > 0 then
-	  if S.PHprover.length proofs > 0 then
+      let tr = get_transformations s g in
+	if List.length tr > 0 then
+	  if Hprover.length proofs > 0 then
 	    fprintf fmt "\\cline{%d-%d}@." (depth + 2) column;
-	S.PHstr.iter (fun _st tr ->
-          let goals = tr.S.transf_goals in
+	List.iter (fun tr ->
+          let goals = get_sub_tasks s tr in
 	  let _ = List.fold_left (fun subgoal g ->
-              goal_latex_stat fmt prov (depth + 1) depth_max (subgoal) g;
+              goal_latex_stat s fmt prov (depth + 1) depth_max (subgoal) g;
                subgoal + 1) 0 goals in
 	    () ) tr
 
@@ -239,65 +225,70 @@ let style_2_row fmt ?(transf=false) depth prov subgoal expl=
   else
     fprintf fmt "\\subgoal{%s}{%d} " expl (subgoal + 1)
 
-let rec goal_latex2_stat fmt prov depth depth_max subgoal g =
-  let proofs = S.goal_external_proofs g in
-  if S.PHprover.length proofs > 0 then
+let rec goal_latex2_stat s fmt prov depth depth_max subgoal g =
+  let proofs = get_proof_attempt_ids s g in
+  if Hprover.length proofs > 0 then
     begin
-      style_2_row fmt depth prov subgoal (protect (S.goal_user_name g));
-      print_result_prov proofs prov fmt
+      style_2_row fmt depth prov subgoal
+                  (protect (get_proof_name s g).Ident.id_string);
+      print_result_prov s proofs prov fmt
     end
  else
     if (*depth = 0*) true then
       begin
-        style_2_row fmt depth prov subgoal (protect (S.goal_user_name g));
+        style_2_row fmt depth prov subgoal
+                    (protect (get_proof_name s g).Ident.id_string);
 	fprintf fmt "& \\multicolumn{%d}{|c|}{}\\\\ @."
           (List.length prov)
       end;
-  let tr = S.goal_transformations g in
-  if S.PHstr.length tr > 0 then
+  let tr = get_transformations s g in
+  if List.length tr > 0 then
     begin
-      S.PHstr.iter (fun _st tr ->
-        style_2_row fmt ~transf:true (depth+1) prov subgoal
-          (protect tr.S.transf_name);
-	fprintf fmt "& \\multicolumn{%d}{|c|}{}\\\\@."
-          (List.length prov);
-	let goals = tr.S.transf_goals in
-	let _ = List.fold_left (fun subgoal g ->
-	  goal_latex2_stat fmt prov (depth + 1) depth_max (subgoal) g;
-	  subgoal + 1) 0 goals in
-	() ) tr
+      List.iter
+        (fun tr ->
+         let name = get_transf_string s tr in
+         style_2_row fmt ~transf:true (depth+1) prov subgoal
+                     (protect name);
+	 fprintf fmt "& \\multicolumn{%d}{|c|}{}\\\\@."
+                 (List.length prov);
+	 let goals = get_sub_tasks s tr in
+	 let _ = List.fold_left
+                   (fun subgoal g ->
+	            goal_latex2_stat s fmt prov (depth + 1) depth_max (subgoal) g;
+	            subgoal + 1) 0 goals in
+	 () ) tr
     end
  else
-    if (S.PHprover.length proofs) == 0 && depth <> 0 then
-      fprintf fmt "\\\\@."
+   if (Hprover.length proofs) == 0 && depth <> 0 then
+     fprintf fmt "\\\\@."
 
-let latex_tabular_goal n fmt depth provers g =
+let latex_tabular_goal n s fmt depth provers g =
   print_tabular_head n depth provers fmt;
   if n == 1 then
-    goal_latex_stat fmt provers 0 depth 0 g
+    goal_latex_stat s fmt provers 0 depth 0 g
   else
-    goal_latex2_stat fmt provers 0 depth  0 g;
+    goal_latex2_stat s fmt provers 0 depth  0 g;
   print_tabular_foot fmt
 
-let latex_tabular_aux n fmt depth provers t =
+let latex_tabular_aux n s fmt depth provers t =
   if n == 1 then
-    List.iter (goal_latex_stat fmt provers 0 depth 0) t.S.theory_goals
+    List.iter (goal_latex_stat s fmt provers 0 depth 0) (theory_goals t)
   else
-    List.iter (goal_latex2_stat fmt provers 0 depth  0) t.S.theory_goals
+    List.iter (goal_latex2_stat s fmt provers 0 depth  0) (theory_goals t)
 
-let latex_tabular n fmt depth provers t =
+let latex_tabular n s fmt depth provers t =
   print_tabular_head n depth provers fmt;
-  latex_tabular_aux n fmt depth provers t;
+  latex_tabular_aux n s fmt depth provers t;
   print_tabular_foot fmt
 
 
-let latex_tabular_file n fmt depth provers f =
+let latex_tabular_file n s fmt depth provers f =
   print_tabular_head n depth provers fmt;
-  List.iter (latex_tabular_aux n fmt depth provers) f.S.file_theories;
+  List.iter (latex_tabular_aux n s fmt depth provers) (file_theories f);
   print_tabular_foot fmt
 
 
-let latex_longtable n fmt depth name provers t=
+let latex_longtable n s fmt depth name provers t=
   fprintf fmt "\\begin{longtable}";
   fprintf fmt "{| l |";
   for _i = 0 to (List.length provers) + depth do fprintf fmt "c |" done;
@@ -317,45 +308,43 @@ let latex_longtable n fmt depth name provers t=
   (* Last foot : nothing *)
   fprintf fmt "\\endfoot \\endlastfoot @.";
   if n == 1 then
-    List.iter (goal_latex_stat fmt provers 0 depth 0) t.S.theory_goals
+    List.iter (goal_latex_stat s fmt provers 0 depth 0) (theory_goals t)
   else
-    List.iter (goal_latex2_stat fmt provers 0 depth  0) t.S.theory_goals;
+    List.iter (goal_latex2_stat s fmt provers 0 depth  0) (theory_goals t);
   fprintf fmt "\\hline \\caption{%s statistics} @." (protect name);
   fprintf fmt "\\label{%s} \\end{longtable}@." (protect name)
 
 
-let theory_latex_stat n table dir t =
-  let provers = Hprover.create 9 in
-  provers_latex_stats provers t;
-  let provers = Hprover.fold (fun _ pr acc -> pr :: acc)
+let theory_latex_stat n s table dir t =
+  let provers = get_used_provers_theory s t in
+  let provers = Whyconf.Sprover.fold (fun pr acc -> pr :: acc)
     provers [] in
   let provers = List.sort Whyconf.Prover.compare provers in
-  let depth = theory_depth  t in
-  let name = t.S.theory_name.Ident.id_string in
+  let depth = theory_depth s t in
+  let name = (theory_name t).Ident.id_string in
   let ch = open_out (Filename.concat dir(name^".tex")) in
   let fmt = formatter_of_out_channel ch in
     if table = "tabular" then
       begin
-      latex_tabular n fmt depth provers t
+      latex_tabular n s fmt depth provers t
       end
     else
-      latex_longtable n fmt depth name provers t;
+      latex_longtable n s fmt depth name provers t;
     close_out ch
 
-let file_latex_stat n table  dir f =
-   List.iter (theory_latex_stat n table dir) f.S.file_theories
+let file_latex_stat n s table  dir f =
+   List.iter (theory_latex_stat n s table dir) (file_theories f)
 
-let standalone_goal_latex_stat n _table dir g =
-  let provers = Hprover.create 9 in
-  provers_latex_stats_goal provers g;
-  let provers = Hprover.fold (fun _ pr acc -> pr :: acc)
+let standalone_goal_latex_stat n s _table dir g =
+  let provers = get_used_provers_goal s g in
+  let provers = Whyconf.Sprover.fold (fun pr acc -> pr :: acc)
     provers [] in
   let provers = List.sort Whyconf.Prover.compare provers in
-  let depth = goal_depth g in
-  let name = (S.goal_name g).Ident.id_string in
-  let ch = open_out (Filename.concat dir(name^".tex")) in
+  let depth = goal_depth s g in
+  let name = (get_proof_name s g).Ident.id_string in
+  let ch = open_out (Filename.concat dir (name^".tex")) in
   let fmt = formatter_of_out_channel ch in
-  latex_tabular_goal n fmt depth provers g;
+  latex_tabular_goal n s fmt depth provers g;
 (*
   if table = "tabular" then
   begin
@@ -364,19 +353,18 @@ let standalone_goal_latex_stat n _table dir g =
   else
   latex_longtable_goal n fmt depth name provers g;
 *)
-close_out ch
+  close_out ch
 
-let file_latex_stat_all n _table dir f =
-  let provers = Hprover.create 9 in
-  provers_latex_stats_file provers f;
-  let provers = Hprover.fold (fun _ pr acc -> pr :: acc)
+let file_latex_stat_all n s _table dir f =
+  let provers = get_used_provers_file s f in
+  let provers = Whyconf.Sprover.fold (fun pr acc -> pr :: acc)
     provers [] in
   let provers = List.sort Whyconf.Prover.compare provers in
-  let depth = file_depth f in
-  let name = Filename.basename f.S.file_name in
+  let depth = file_depth s f in
+  let name = Filename.basename (file_name f) in
   let ch = open_out (Filename.concat dir(name^".tex")) in
   let fmt = formatter_of_out_channel ch in
-  latex_tabular_file n fmt depth provers f;
+  latex_tabular_file n s fmt depth provers f;
 (*
     if table = "tabular" then
       begin
@@ -390,82 +378,83 @@ let file_latex_stat_all n _table dir f =
 
 
 
-let element_latex_stat_goal g n table dir r =
+let element_latex_stat_goal g n s table dir r =
   begin
     match r with
       | [] -> ()
       | _ ->
         eprintf "Warning: only main goals are supported@.:"
   end;
-  standalone_goal_latex_stat n table dir g
+  standalone_goal_latex_stat n s table dir g
 
-let element_latex_stat_theory th n table dir e =
+let element_latex_stat_theory s th n table dir e =
   match e with
-    | [] -> theory_latex_stat n table dir th
+    | [] -> theory_latex_stat n s table dir th
     | g :: r ->
       try
         let goals =
-          List.map (fun g -> (S.goal_name g).Ident.id_string,g)
-            th.S.theory_goals
+          List.map (fun g ->
+                    (get_proof_name s g).Ident.id_string,g)
+            (theory_goals th)
         in
         let g = List.assoc g goals in
-        element_latex_stat_goal g n table dir r
+        element_latex_stat_goal g n s table dir r
       with Not_found ->
         eprintf "Warning: goal not found in path: %s@." g
 
-let element_latex_stat_file f n table dir e =
+let element_latex_stat_file f n s table dir e =
   match e with
-    | [] -> file_latex_stat_all n table dir f
+    | [] -> file_latex_stat_all n s table dir f
     | th :: r ->
       try
         let ths =
-          List.map (fun th -> (th.S.theory_name.Ident.id_string,th))
-            f.S.file_theories
+          List.map (fun th -> (theory_name th).Ident.id_string,th)
+            (file_theories f)
         in
         let th = List.assoc th ths in
-        element_latex_stat_theory th n table dir r
+        element_latex_stat_theory s th n table dir r
       with Not_found ->
         eprintf "Warning: theory not found in path: %s@." th
 
 
-let element_latex_stat files n table dir e =
+let element_latex_stat files n s table dir e =
   eprintf "Element %s@." e;
   match Strings.split '.' e with
     | [] -> ()
     | f :: r ->
       let found = ref false in
-      S.PHstr.iter
+      Hstr.iter
         (fun fname file ->
           let fname = Filename.basename fname in
           let fname = List.hd (Strings.split '.' fname) in
           if fname = f then
             begin
               found := true;
-              element_latex_stat_file file n table dir r
+              element_latex_stat_file file n s table dir r
             end)
         files;
       if not !found then
         eprintf "Warning: file not found for element %s@." e
 
 let print_latex_statistics n table dir session =
-  let files = session.S.session_files in
+  let files = get_files session in
   match !opt_elements with
     | None ->
-      S.PHstr.iter (fun _ f -> file_latex_stat n table dir f) files
+      Hstr.iter (fun _ f -> file_latex_stat n session table dir f) files
     | Some l ->
-      List.iter (element_latex_stat files n table dir) l
+      List.iter (element_latex_stat files n session table dir) l
 
 
 
 let table () = if !opt_longtable then "longtable" else "tabular"
 
 let run_one fname =
-  let project_dir = Session.get_project_dir fname in
-  let session,_use_shapes = Session.read_session project_dir in
+  let ses,_ = read_session fname in
+  let project_dir = get_dir ses in
   let dir = if !opt_output_dir = "" then project_dir else
       !opt_output_dir
   in
-  print_latex_statistics !opt_style (table ()) dir session
+  print_latex_statistics !opt_style (table ()) dir ses
 
 let run () =
   let _,_,should_exit1 = read_env_spec () in
